@@ -1,14 +1,13 @@
 #! perl
 # Copyright (C) 2009 The Perl Foundation
 
-use 5.010;
+use 5.10.1;
 use strict;
 use warnings;
 use Text::ParseWords;
 use Getopt::Long;
 use File::Spec;
 use Cwd;
-use Perl::OSType 'is_os_type';
 use lib 'tools/lib';
 use NQP::Configure qw(sorry slurp cmp_rev gen_nqp read_config
                       fill_template_text fill_template_file
@@ -36,7 +35,7 @@ MAIN: {
     GetOptions(\%options, 'help!', 'prefix=s',
                'sysroot=s', 'sdkroot=s',
                'backends=s', 'no-clean!',
-               'gen-nqp:s',
+               'with-nqp=s', 'gen-nqp:s',
                'gen-moar:s', 'moar-option=s@',
                'git-protocol=s',
                'make-install!', 'makefile-timing!',
@@ -68,6 +67,11 @@ MAIN: {
     }
     my @backends;
     my %backends;
+    if (my $nqp_bin  = $options{'with-nqp'}) {
+        die "Could not find $nqp_bin" unless -e $nqp_bin;
+        $options{backends} = qx{$nqp_bin -e 'print(nqp::getcomp("nqp").backend.name)'}
+            or die "Could not get backend information from $nqp_bin";
+    }
     if (defined $options{backends}) {
         $options{backends} = join ",", @known_backends
             if uc($options{backends}) eq 'ALL';
@@ -105,7 +109,7 @@ MAIN: {
             $backends{moar} = 1;
             $default_backend ||= 'moar';
         }
-        unless (%backends) {
+        unless (%backends or exists $options{'with-nqp'}) {
             die "No suitable nqp executables found! Please specify some --backends, or a --prefix that contains nqp-{p,j,m} executables\n\n"
               . "Example to build for all backends (which will take a while):\n"
               . "\tperl Configure.pl --backends=moar,jvm --gen-moar\n\n"
@@ -135,6 +139,13 @@ MAIN: {
     $config{'runner_suffix'} = $win ? '.bat' : '';
 
     my $make = 'make';
+    if ($^O eq 'solaris') {
+        if (not -X '/usr/bin/gmake') {
+            die "gmake is required to compile rakudo. Please install by 'pkg install gnu-make'";
+        }
+        $make = 'gmake';
+    }
+
     if ($win) {
         my $has_nmake = 0 == system('nmake /? >NUL 2>&1');
         my $has_cl    = `cl 2>&1` =~ /Microsoft Corporation/;
@@ -152,7 +163,7 @@ MAIN: {
         }
     }
 
-    for my $target (qw/common_bootstrap_sources moar_core_sources/) {
+    for my $target (qw/common_bootstrap_sources moar_core_sources jvm_core_sources/) {
         open my $FILELIST, '<', "tools/build/$target"
             or die "Cannot read 'tools/build/$target': $!";
         my @lines;
@@ -169,7 +180,7 @@ MAIN: {
 
     print $MAKEFILE "\n# Makefile code generated by Configure.pl:\n";
 
-    if (is_os_type('Unix')) {
+    if ( is_OS_type_Unix() ) {
         $config{mkpath} = 'mkdir -p --';
         $config{chmod}  = 'chmod --';
         $config{cp}     = 'cp --';
@@ -271,8 +282,8 @@ MAIN: {
         unless ($win) {
             $config{'m_cleanups'} = "  \$(M_GDB_RUNNER) \\\n  \$(M_VALGRIND_RUNNER)";
             $config{'m_all'}      = '$(M_GDB_RUNNER) $(M_VALGRIND_RUNNER)';
-            $config{'m_install'}  = "\t" . '$(PERL5) tools/build/create-moar-runner.pl "$(MOAR)" perl6.moarvm $(DESTDIR)$(PREFIX)/bin/perl6-gdb-m "$(PERL6_LANG_DIR)/runtime" "gdb" "$(M_LIBPATH)" "$(PERL6_LANG_DIR)/lib" "$(PERL6_LANG_DIR)/runtime"' . "\n"
-                                  . "\t" . '$(PERL5) tools/build/create-moar-runner.pl "$(MOAR)" perl6.moarvm $(DESTDIR)$(PREFIX)/bin/perl6-valgrind-m "$(PERL6_LANG_DIR)/runtime" "valgrind" "$(M_LIBPATH)" "$(PERL6_LANG_DIR)/lib" "$(PERL6_LANG_DIR)/runtime"';
+            $config{'m_install'}  = "\t" . '$(M_RUN_PERL6) tools/build/create-moar-runner.pl "$(MOAR)" perl6.moarvm $(DESTDIR)$(PREFIX)/bin/perl6-gdb-m "$(PERL6_LANG_DIR)/runtime" "gdb" "" "$(M_LIBPATH)" "$(PERL6_LANG_DIR)/lib" "$(PERL6_LANG_DIR)/runtime"' . "\n"
+                                  . "\t" . '$(M_RUN_PERL6) tools/build/create-moar-runner.pl "$(MOAR)" perl6.moarvm $(DESTDIR)$(PREFIX)/bin/perl6-valgrind-m "$(PERL6_LANG_DIR)/runtime" "valgrind" "" "$(M_LIBPATH)" "$(PERL6_LANG_DIR)/lib" "$(PERL6_LANG_DIR)/runtime"';
         }
 
         unless (@errors) {
@@ -337,6 +348,22 @@ MAIN: {
     exit 0;
 }
 
+sub is_OS_type_Unix {
+    # The following is a partial OS list taken from Perl::OSType module,
+    # copyright by David Golden. The up-to-date version of that module can
+    # be found at https://metacpan.org/pod/Perl::OSType
+
+    return 1 if grep $^O eq $_, qw/
+        aix       bsdos        beos   bitrig  dgux      dragonfly  dynixptx
+        freebsd   linux        haiku  hpux    iphoneos  irix       darwin
+        machten   midnightbsd  minix  mirbsd  next      openbsd    netbsd
+        dec_osf   nto          svr4   svr5    sco       sco_sv     unicos
+        unicosmk  solaris      sunos  cygwin  msys      os2        interix
+        gnu       gnukfreebsd  nto    qnx     android
+    /;
+
+    return 0;
+}
 
 #  Print some help text.
 sub print_help {
@@ -352,9 +379,11 @@ General Options:
     --backends=jvm,moar
                        Which backend(s) to use (or ALL for all of them)
     --gen-nqp[=branch]
-                       Download and build a copy of NQP
+                       Download, build, and install a copy of NQP before writing the Makefile
     --gen-moar[=branch]
-                       Download and build a copy of MoarVM to use
+                       Download, build, and install a copy of MoarVM to use before writing the Makefile
+    --with-nqp='/path/to/nqp'
+                       Provide path to already installed nqp
     --make-install     Install Rakudo after configuration is done
     --moar-option='--option=value'
                        Options to pass to MoarVM's Configure.pl
@@ -369,6 +398,11 @@ General Options:
     --makefile-timing  Enable timing of individual makefile commands
     --no-clean         Skip cleanup before installation
 
+Please note that the --gen-moar and --gen-nqp options are there for convenience
+only and will actually immediately - at Configure time - compile and install
+moar and nqp respectively. They will live under the path given to --prefix,
+unless other targeting options are used.
+
 Configure.pl also reads options from 'config.default' in the current directory.
 END
 
