@@ -11,11 +11,8 @@ my class X::Subscript::Negative { ... }
 
 my role  Numeric { ... }
 
-# We use a sentinel value to mark the end of an iteration.
-my constant IterationEnd = nqp::create(Mu);
-
 my class Any { # declared in BOOTSTRAP
-    # my class Any is Mu {
+    # my class Any is Mu
 
     multi method ACCEPTS(Any:D: Mu:D \a) { self === a }
     multi method ACCEPTS(Any:D: Mu:U \a) { False }
@@ -38,7 +35,13 @@ my class Any { # declared in BOOTSTRAP
     multi method DELETE-POS(Any:D: $pos) {
         Failure.new("Can not remove elements from a {self.^name}")
     }
-    multi method DELETE-POS(**@indices) {
+    multi method DELETE-POS(Any:D: \one, \two) is raw {
+        self.AT-POS(one).DELETE-POS(two)
+    }
+    multi method DELETE-POS(Any:D: \one, \two, \three) is raw {
+        self.AT-POS(one).AT-POS(two).DELETE-POS(three)
+    }
+    multi method DELETE-POS(Any:D: **@indices) {
         my $final := @indices.pop;
         Rakudo::Internals.WALK-AT-POS(self,@indices).DELETE-POS($final)
     }
@@ -168,7 +171,8 @@ my class Any { # declared in BOOTSTRAP
             ?? self.map({ .tree($count - 1) }).item
             !! self
     }
-    multi method tree(Any:D: *@ [&first, *@rest]) {
+    multi method tree(Any:D: @ [&first, *@rest]) { self.tree(&first, |@rest); }
+    multi method tree(Any:D: &first, *@rest) {
         nqp::istype(self, Iterable)
             ?? @rest ?? first(self.map({ .tree(|@rest) }))
                      !! first(self)
@@ -224,7 +228,13 @@ my class Any { # declared in BOOTSTRAP
     multi method EXISTS-POS(Any:D: Any:U \pos) {
         die "Cannot use '{pos.^name}' as an index";
     }
-    multi method EXISTS-POS(**@indices) {
+    multi method EXISTS-POS(Any:D: \one, \two) is raw {
+        self.AT-POS(one).EXISTS-POS(two)
+    }
+    multi method EXISTS-POS(Any:D: \one, \two,\three) is raw {
+        self.AT-POS(one).AT-POS(two).EXISTS-POS(three)
+    }
+    multi method EXISTS-POS(Any:D: **@indices) {
         my $final := @indices.pop;
         Rakudo::Internals.WALK-AT-POS(self,@indices).EXISTS-POS($final)
     }
@@ -274,7 +284,13 @@ my class Any { # declared in BOOTSTRAP
     multi method AT-POS(Any:   Any:U \pos) is raw {
         die "Cannot use '{pos.^name}' as an index";
     }
-    multi method AT-POS(**@indices) is raw {
+    multi method AT-POS(Any:D: \one, \two) is raw {
+        self.AT-POS(one).AT-POS(two)
+    }
+    multi method AT-POS(Any:D: \one, \two, \three) is raw {
+        self.AT-POS(one).AT-POS(two).AT-POS(three)
+    }
+    multi method AT-POS(Any:D: **@indices) is raw {
         my $final := @indices.pop;
         Rakudo::Internals.WALK-AT-POS(self,@indices).AT-POS($final)
     }
@@ -321,7 +337,13 @@ my class Any { # declared in BOOTSTRAP
     multi method ASSIGN-POS(Any:D: Any:U \pos, Mu \assignee) {
         die "Cannot use '{pos.^name}' as an index";
     }
-    multi method ASSIGN-POS(**@indices) {
+    multi method ASSIGN-POS(Any:D: \one, \two, Mu \assignee) is raw {
+        self.AT-POS(one).ASSIGN-POS(two, assignee)
+    }
+    multi method ASSIGN-POS(Any:D: \one, \two, \three, Mu \assignee) is raw {
+        self.AT-POS(one).AT-POS(two).ASSIGN-POS(three, assignee)
+    }
+    multi method ASSIGN-POS(Any:D: **@indices) {
         my \value := @indices.pop;
         my $final := @indices.pop;
         Rakudo::Internals.WALK-AT-POS(self,@indices).ASSIGN-POS($final,value)
@@ -334,13 +356,14 @@ my class Any { # declared in BOOTSTRAP
 #        my $final := @indices.pop;
 #        Rakudo::Internals.WALK-AT-POS(self,@indices).BIND-POS($final,value)
 
-        my int $elems = @indices.elems;
+        my int $elems = @indices.elems;   # reifies
         my \value  := @indices.AT-POS(--$elems);
         my $final  := @indices.AT-POS(--$elems);
         my $target := self;
         my int $i = -1;
         $target := $target.AT-POS(@indices.AT-POS($i))
           while nqp::islt_i(++$i,$elems);
+        X::Bind.new.throw if $target =:= self;
         $target.BIND-POS($final, value)
     }
 
@@ -400,7 +423,7 @@ my class Any { # declared in BOOTSTRAP
 
     method lazy-if($flag) { self }  # no-op on non-Iterables
 
-    method sum() {
+    method sum() is nodal {
         my \iter = self.iterator;
         my $sum = 0;
         my Mu $value;
@@ -418,8 +441,9 @@ proto sub infix:<===>(Mu $?, Mu $?) is pure { * }
 multi sub infix:<===>($?)    { Bool::True }
 multi sub infix:<===>(\a, \b) {
     nqp::p6bool(
-      nqp::eqaddr(a.WHAT,b.WHAT)
-      && nqp::iseq_s(nqp::unbox_s(a.WHICH), nqp::unbox_s(b.WHICH))
+      nqp::eqaddr(a,b)
+      || (nqp::eqaddr(a.WHAT,b.WHAT)
+           && nqp::iseq_s(nqp::unbox_s(a.WHICH), nqp::unbox_s(b.WHICH)))
     )
 }
 
@@ -517,14 +541,15 @@ sub SLICE_HUH(\SELF, @nogo, %d, %adv) {
 } #SLICE_HUH
 
 sub DELETEKEY(Mu \d, str $key) {
-    if nqp::existskey(d,$key) {
-        my Mu $value := nqp::atkey(d,$key);
-        nqp::deletekey(d,$key);
-        $value;
-    }
-    else {
-        Nil;
-    }
+    nqp::if(
+      nqp::existskey(d,$key),
+      nqp::stmts(
+        (my Mu $value := nqp::atkey(d,$key)),
+        (nqp::deletekey(d,$key)),
+        $value
+      ),
+      Nil
+    )
 } #DELETEKEY
 
 sub dd(|) {
