@@ -1,45 +1,51 @@
 my class Block { # declared in BOOTSTRAP
-    # class Block is Code {
+    # class Block is Code
     #     has Mu $!phasers;
     #     has Mu $!why;
 
     method of(Block:D:)      { nqp::getattr(self,Code,'$!signature').returns }
     method returns(Block:D:) { nqp::getattr(self,Code,'$!signature').returns }
 
-    method add_phaser(Str $name, &block) {
-        nqp::isnull($!phasers) &&
-            nqp::bindattr(self, Block, '$!phasers', nqp::hash());
-        nqp::existskey($!phasers, nqp::unbox_s($name)) ||
-            nqp::bindkey($!phasers, nqp::unbox_s($name), nqp::list());
-        if $name eq 'LEAVE' || $name eq 'KEEP' || $name eq 'UNDO' {
-            nqp::unshift(nqp::atkey($!phasers, nqp::unbox_s($name)), &block);
+    method add_phaser(Str:D \name, &block --> Nil) {
+        nqp::bindattr(self,Block,'$!phasers',nqp::hash)
+          unless nqp::attrinited(self,Block,'$!phasers');
+
+        my str $name = name;
+        nqp::bindkey($!phasers,$name,nqp::list)
+          unless nqp::existskey($!phasers,$name);
+
+        if nqp::iseq_s($name,'LEAVE') || nqp::iseq_s($name,'KEEP') || nqp::iseq_s($name,'UNDO') {
+            nqp::unshift(nqp::atkey($!phasers,$name),&block);
             self.add_phaser('!LEAVE-ORDER', &block);
         }
-        elsif $name eq 'NEXT' || $name eq '!LEAVE-ORDER' || $name eq 'POST' {
-            nqp::unshift(nqp::atkey($!phasers, nqp::unbox_s($name)), &block);
+        elsif nqp::iseq_s($name,'NEXT') || nqp::iseq_s($name,'!LEAVE-ORDER') || nqp::iseq_s($name,'POST') {
+            nqp::unshift(nqp::atkey($!phasers,$name),&block);
         }
         else {
-            nqp::push(nqp::atkey($!phasers, nqp::unbox_s($name)), &block);
+            nqp::push(nqp::atkey($!phasers,$name),&block);
         }
     }
 
-    method fire_phasers(str $name) {
-        if !nqp::isnull($!phasers) && nqp::existskey($!phasers, $name) {
+    method fire_phasers(str $name --> Nil) {
+        if nqp::attrinited(self,Block,'$!phasers') && nqp::existskey($!phasers, $name) {
             my Mu $iter := nqp::iterator(nqp::atkey($!phasers, $name));
-            nqp::shift($iter).() while $iter;
+            nqp::while($iter, nqp::shift($iter).(), :nohandler);
         }
     }
 
-    method has-phasers() { !nqp::isnull($!phasers) }
+    method has-phasers() { nqp::attrinited(self,Block,'$!phasers') }
 
-    method phasers(Str $name) {
-        unless nqp::isnull($!phasers) {
-            if nqp::existskey($!phasers, nqp::unbox_s($name)) {
-                return nqp::p6bindattrinvres(nqp::create(List), List, '$!reified',
-                    nqp::atkey($!phasers, nqp::unbox_s($name)));
-            }
-        }
-        ()
+    method has-phaser(Str:D \name) {
+        nqp::attrinited(self,Block,'$!phasers')
+          && nqp::existskey($!phasers,nqp::unbox_s(name))
+    }
+
+    method phasers(Str:D $name) {
+        nqp::attrinited(self,Block,'$!phasers')
+          && nqp::existskey($!phasers,nqp::unbox_s($name))
+          ?? nqp::p6bindattrinvres(nqp::create(List),List,'$!reified',
+               nqp::atkey($!phasers,nqp::unbox_s($name)))
+          !! ()
     }
 
     method assuming(Block:D $self: |primers) {
@@ -76,38 +82,32 @@ my class Block { # declared in BOOTSTRAP
 
             $perl = '' if $elide_agg_cont;
             unless $type eq "Any" {
-                my $i = 0; # broken FIRST workaround
+                my int $FIRST = 1; # broken FIRST workaround
                 while ($type ~~ / (.*?) \[ (.*) \] $$/) {
-                    my $slash0 = ~$0;
-                    my $slash1 = ~$1;
-#                   FIRST {  # seems broken
-                    unless ($i++) { # broken FIRST workaaround
-                        $perl = ~$/;
-                        if $elide_agg_cont {
-                           $perl = ~$slash1;
-                        }
+#                   FIRST {  # seems broken in setting
+                    if $FIRST { # broken FIRST workaround
+                        $perl = $elide_agg_cont
+                          ?? ~$1
+                          !! ~$/;
+                        $FIRST = 0;
                     }
-                    $type = ~$slash1;
-                    unless soft_indirect_name_lookup($slash0) {
-                        $perl = "";
+                    $type = ~$1;
+                    unless soft_indirect_name_lookup(~$0) {
+                        $perl = '';
                         last
                     };
                 }
-                unless soft_indirect_name_lookup($type) {
-                    $perl = "";
-                };
+                $perl = '' unless soft_indirect_name_lookup($type);
             }
-#Introspection fail.  There is no introspection to access these flags.
-#Skipped for now.
-#            if $!flags +& $SIG_ELEM_DEFINED_ONLY {
-#                $perl ~= ':D' if $perl ne '';
-#            } elsif $!flags +& $SIG_ELEM_UNDEFINED_ONLY {
-#                $perl ~= ':U' if $perl ne '';
-#            }
+            $perl ~= $parm.modifier if $perl ne '';
+
             my $name = $parm.name;
-            if not $name.defined or !$name.starts-with($sigil) {
-                $name = $sigil ~ $parm.twigil ~ ($name // "");
+            if !$name and $parm.raw {
+                $name = '$';
+            } elsif !$name or !$name.starts-with($sigil) {
+                $name = $sigil ~ $parm.twigil ~ ($name // '');
             }
+
             if $parm.slurpy {
                 $name = '*' ~ $name;
             } elsif $parm.named {
@@ -118,23 +118,14 @@ my class Block { # declared in BOOTSTRAP
             } elsif $parm.optional or $parm.default {
                 $name ~= '?';
             }
+
             if $parm.rw {
                 $rest ~= ' is rw';
             } elsif $parm.copy {
                 $rest ~= ' is copy';
             }
-            if  $parm.raw {
-                if     not $.name {
-                    if $name eq '$' and not $rest {
-                        $name = '\\';
-                    }
-                    elsif $name.starts-with('\\') and ($rest or $name ne '\\') {
-                        $name = '$' ~ $name.substr(1);
-                    }
-                }
-                if !$name.starts-with('\\') {
-                    $rest ~= ' is raw';
-                }
+            if $parm.raw {
+                $rest ~= ' is raw' unless $name.starts-with('\\');
             }
             if $name or $rest {
                 $perl ~= ($perl ?? ' ' !! '') ~ $name;
@@ -185,8 +176,8 @@ my class Block { # declared in BOOTSTRAP
         # but really Signature should be able to tell us that.
         #
         # Until then, we will add slurpy behaviors, assuming we
-        # do not aready have them, if we see a capture.
-        my $need_cap = $sig.count == Inf and not ($slurp_p and $slurp_n);
+        # do not already have them, if we see a capture.
+        my $need_cap = ($sig.count == Inf and not ($slurp_p and $slurp_n));
         if $need_cap {
             $need_cap = False;
             for $sig.params.grep(*.capture) {
@@ -286,7 +277,7 @@ my class Block { # declared in BOOTSTRAP
             }
             $p;
         }
-        if ($slurp_n and $slurp_n.capture and $slurp_n !=== $slurp_p) {
+        if ($slurp_n and $slurp_n.capture and !($slurp_n === $slurp_p)) {
             @phash.push(strip_parm($slurp_n));
         }
         my $error = False;
@@ -298,6 +289,7 @@ my class Block { # declared in BOOTSTRAP
         my $f;
         my $primed_sig = (flat @plist.map(&strip_parm), @phash,
                           ($slurp_p ?? strip_parm($slurp_p) !! ())).join(", ");
+        $primed_sig ~= ' --> ' ~ $sig.returns.^name;
 
         $f = EVAL sprintf(
             '{ my $res = (my proto __PRIMED_ANON (%s) { {*} });
@@ -315,16 +307,12 @@ my class Block { # declared in BOOTSTRAP
     }
 
     multi method perl(Block:D:) {
-        my $perl = '-> ';
-        # lose colon prefix and parens
-        $perl ~= substr(self.signature().perl,2,*-1);
-        $perl ~= ' { #`(' ~ self.WHICH ~ ') ... }';
-        $perl
+        "-> {self.signature.perl.substr(2,*-1)} \{ #`({self.WHICH}) ... \}"
     }
 
     method WHY() {
         if nqp::isnull($!why) {
-            Any
+            Nil
         } else {
             $!why.set_docee(self);
             $!why
