@@ -13,7 +13,7 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
       unless nqp::istype(T,Int);
 
     multi method WHICH(Blob:D:) {
-        self.^name ~ '|' ~ nqp::sha1(self.join(","))
+        self.^name ~ '|' ~ nqp::sha1(self.decode("latin-1"))
     }
 
     multi method new(Blob:) { nqp::create(self) }
@@ -70,14 +70,18 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
     }
 
     multi method AT-POS(Blob:D: int \pos) {
-        self!fail-range(pos)
-          if nqp::isge_i(pos,nqp::elems(self)) || nqp::islt_i(pos,0);
-        nqp::atpos_i(self, pos);
+        nqp::if(
+          (nqp::isge_i(pos,nqp::elems(self)) || nqp::islt_i(pos,0)),
+          self!fail-range(pos),
+          nqp::atpos_i(self,pos)
+        )
     }
     multi method AT-POS(Blob:D: Int:D \pos) {
-        self!fail-range(pos)
-          if nqp::isge_i(pos,nqp::elems(self)) || nqp::islt_i(pos,0);
-        nqp::atpos_i(self,pos);
+        nqp::if(
+          (nqp::isge_i(pos,nqp::elems(self)) || nqp::islt_i(pos,0)),
+          self!fail-range(pos),
+          nqp::atpos_i(self,pos)
+        )
     }
 
     multi method Bool(Blob:D:) { nqp::p6bool(nqp::elems(self)) }
@@ -160,6 +164,19 @@ my role Blob[::T = uint8] does Positional[T] does Stringy is repr('VMArray') is
         $subbuf
     }
 
+    method reverse(Blob:D:) {
+        my int $elems = nqp::elems(self);
+        my int $last  = nqp::sub_i($elems,1);
+        my $reversed := nqp::setelems(nqp::create(self),$elems);
+        my int $i     = -1;
+        nqp::while(
+          nqp::islt_i(($i = nqp::add_i($i,1)),$elems),
+          nqp::bindpos_i($reversed,nqp::sub_i($last,$i),
+            nqp::atpos_i(self,$i))
+        );
+        $reversed
+    }
+
     method COMPARE(Blob:D: Blob:D \other) {
         my $other := nqp::decont(other);
         my int $elems = nqp::elems(self);
@@ -463,38 +480,32 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
 
     my $empty := nqp::list_i;
     multi method splice(Buf:D \SELF:) { my $buf = SELF; SELF = Buf.new; $buf }
-    multi method splice(Buf:D: Int $offset, $size = Whatever, :$SINK) {
+    multi method splice(Buf:D: Int $offset, $size = Whatever) {
         my int $remove = self!remove($offset,$size);
-        if $SINK {
-            nqp::splice(self,$empty,$offset,$remove);
-            Nil
-        }
-        else {
-            my $result := $remove
-              ?? self.subbuf($offset,$remove)  # until something smarter
-              !! nqp::create(self);
-            nqp::splice(self,$empty,$offset,$remove);
-            $result
-        }
+        my $result := $remove
+          ?? self.subbuf($offset,$remove)  # until something smarter
+          !! nqp::create(self);
+        nqp::splice(self,$empty,$offset,$remove);
+        $result
     }
-    multi method splice(Buf:D: Int $offset, $size, int $got, :$SINK) {
-        self!splice-native($offset,$size,$got,$SINK)
+    multi method splice(Buf:D: Int $offset, $size, int $got) {
+        self!splice-native($offset,$size,$got)
     }
-    multi method splice(Buf:D: Int $offset, $size, Int $got, :$SINK) {
-        self!splice-native($offset,$size,$got,$SINK)
+    multi method splice(Buf:D: Int $offset, $size, Int $got) {
+        self!splice-native($offset,$size,$got)
     }
-    multi method splice(Buf:D: Int $offset, $size, Mu $got, :$SINK) {
+    multi method splice(Buf:D: Int $offset, $size, Mu $got) {
         self!fail-typecheck('splice',$got)
     }
-    multi method splice(Buf:D: Int $offset, $size, Buf:D $buf, :$SINK) {
-        self!splice-native($offset,$size,$buf,$SINK)
+    multi method splice(Buf:D: Int $offset, $size, Buf:D $buf) {
+        self!splice-native($offset,$size,$buf)
     }
-    multi method splice(Buf:D: Int $offset, $size, int @values, :$SINK) {
-        self!splice-native($offset,$size,@values,$SINK)
+    multi method splice(Buf:D: Int $offset, $size, int @values) {
+        self!splice-native($offset,$size,@values)
     }
-    multi method splice(Buf:D: Int $offset, $size, @values, :$SINK) {
+    multi method splice(Buf:D: Int $offset, $size, @values) {
         self!splice-native($offset,$size,
-          self!push-list("splic",nqp::create(self),@values),$SINK)
+          self!push-list("splic",nqp::create(self),@values))
     }
 
     method !remove(\offset,\size) {
@@ -505,27 +516,20 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
             !! size.Int
     }
 
-    method !splice-native(Buf:D: Int $offset, $size, \x, $SINK) {
+    method !splice-native(Buf:D: Int $offset, $size, \x) {
         my int $remove = self!remove($offset,$size);
-        if $SINK {
-            nqp::splice(
-              self,nqp::islist(x) ?? x !! nqp::list_i(x),$offset,$remove);
-            Nil
-        }
-        else {
-            my $result := $remove
-              ?? self.subbuf($offset,$remove)  # until something smarter
-              !! nqp::create(self);
-            nqp::splice(
-              self,nqp::islist(x) ?? x !! nqp::list_i(x),$offset,$remove);
-            $result
-        }
+        my $result := $remove
+          ?? self.subbuf($offset,$remove)  # until something smarter
+          !! nqp::create(self);
+        nqp::splice(
+          self,nqp::islist(x) ?? x !! nqp::list_i(x),$offset,$remove);
+        $result
     }
 
     multi method push(Buf:D: int $got) { nqp::push_i(self,$got); self }
     multi method push(Buf:D: Int $got) { nqp::push_i(self,$got); self }
     multi method push(Buf:D: Mu $got) { self!fail-typecheck('push',$got) }
-    multi method push(Buf:D: Buf:D $buf) {
+    multi method push(Buf:D: Blob:D $buf) {
         nqp::splice(self,$buf,nqp::elems(self),0)
     }
     multi method push(Buf:D: **@values) { self!pend(@values,'push') }
@@ -533,7 +537,7 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
     multi method append(Buf:D: int $got) { nqp::push_i(self,$got); self }
     multi method append(Buf:D: Int $got) { nqp::push_i(self,$got); self }
     multi method append(Buf:D: Mu $got) { self!fail-typecheck('append',$got) }
-    multi method append(Buf:D: Buf:D $buf) {
+    multi method append(Buf:D: Blob:D $buf) {
         nqp::splice(self,$buf,nqp::elems(self),0)
     }
     multi method append(Buf:D: int @values) {
@@ -545,13 +549,13 @@ my role Buf[::T = uint8] does Blob[T] is repr('VMArray') is array_type(T) {
     multi method unshift(Buf:D: int $got) { nqp::unshift_i(self,$got); self }
     multi method unshift(Buf:D: Int $got) { nqp::unshift_i(self,$got); self }
     multi method unshift(Buf:D: Mu $got) { self!fail-typecheck('unshift',$got) }
-    multi method unshift(Buf:D: Buf:D $buf) { nqp::splice(self,$buf,0,0) }
+    multi method unshift(Buf:D: Blob:D $buf) { nqp::splice(self,$buf,0,0) }
     multi method unshift(Buf:D: **@values) { self!pend(@values,'unshift') }
 
     multi method prepend(Buf:D: int $got) { nqp::unshift_i(self,$got); self }
     multi method prepend(Buf:D: Int $got) { nqp::unshift_i(self,$got); self }
     multi method prepend(Buf:D: Mu $got) { self!fail-typecheck('prepend',$got) }
-    multi method prepend(Buf:D: Buf:D $buf)  { nqp::splice(self,$buf,0,0)    }
+    multi method prepend(Buf:D: Blob:D $buf)  { nqp::splice(self,$buf,0,0)    }
     multi method prepend(Buf:D: int @values) { nqp::splice(self,@values,0,0) }
     multi method prepend(Buf:D:  @values) { self!pend(@values,'prepend') }
     multi method prepend(Buf:D: *@values) { self!pend(@values,'prepend') }
@@ -662,6 +666,7 @@ multi sub pack(@template, *@items) {
     return Buf.new(@bytes);
 }
 
+multi sub infix:<~>(Blob:D \a) { a }
 multi sub infix:<~>(Blob:D $a, Blob:D $b) {
     my $res := ($a.WHAT === $b.WHAT ?? $a !! Buf).new;
     my $adc := nqp::decont($a);
