@@ -1,16 +1,16 @@
 my class Code does Callable { # declared in BOOTSTRAP
-    # class Code is Any {
-    #     has Mu $!do;                # Low level code object
-    #     has Mu $!signature;         # Signature object
-    #     has Mu $!compstuff;         # Place for the compiler to hang stuff
+    # class Code is Any
+    #     has Code $!do;              # Low level code object
+    #     has Signature $!signature;  # Signature object
+    #     has @!compstuff;            # Place for the compiler to hang stuff
 
     multi method ACCEPTS(Code:D $self: Mu $topic) {
         $self.count ?? $self($topic) !! $self()
     }
 
-    method arity(Code:D:) { $!signature.arity }
+    method arity(Code:D:) { nqp::getattr_i($!signature,Signature,'$!arity') }
 
-    method count(Code:D:) { $!signature.count }
+    method count(Code:D:) { nqp::getattr($!signature,Signature,'$!count') }
 
     method signature(Code:D:) { $!signature }
 
