@@ -3,15 +3,14 @@ my class Complex is Cool does Numeric {
     has num $.re;
     has num $.im;
 
-    proto method new(|) { * }
-    multi method new(Real \re, Real \im) {
-        nqp::create(self).BUILD(re, im);
-    }
-    submethod BUILD(Num() \re, Num() \im) {
+    method !SET-SELF(Num() \re, Num() \im) {
         $!re = re;
         $!im = im;
-        self;
+        self
     }
+    proto method new(|) { * }
+    multi method new() { self.new: 0, 0 }
+    multi method new(Real \re, Real \im) { nqp::create(self)!SET-SELF(re, im) }
 
     multi method WHICH(Complex:D:) {
         nqp::box_s(
@@ -32,8 +31,9 @@ my class Complex is Cool does Numeric {
     }
 
     method coerce-to-real(Complex:D: $exception-target) {
-        unless $!im == 0e0 { fail X::Numeric::Real.new(target => $exception-target, reason => "imaginary part not zero", source => self);}
-        $!re;
+        $!im ≅ 0e0
+          ?? $!re
+          !! Failure.new(X::Numeric::Real.new(target => $exception-target, reason => "imaginary part not zero", source => self))
     }
     multi method Real(Complex:D:) { self.coerce-to-real(Real); }
 
@@ -48,10 +48,16 @@ my class Complex is Cool does Numeric {
 
     method Complex() { self }
     multi method Str(Complex:D:) {
-        my Str $i = nqp::isnanorinf($!im) ?? '\\i' !! 'i';
-        $!im < 0e0
-            ?? nqp::p6box_s($!re) ~ '-' ~ nqp::p6box_s(nqp::abs_n($!im)) ~ $i
-            !! nqp::p6box_s($!re) ~ '+' ~ nqp::p6box_s($!im) ~ $i;
+        nqp::concat(
+          $!re,
+          nqp::concat(
+            nqp::if(nqp::iseq_i(nqp::ord($!im),45),'','+'),
+            nqp::concat(
+              $!im,
+              nqp::if(nqp::isnanorinf($!im),'\\i','i')
+            )
+          )
+        )
     }
 
     multi method perl(Complex:D:) {
@@ -79,8 +85,10 @@ my class Complex is Cool does Numeric {
     }
 
     method sqrt(Complex:D:) {
-        my Num ($mag, $angle) = self.polar;
-        $mag.sqrt.unpolar($angle/2);
+        my Num $abs = self.abs;
+        my Num $re = (($abs + self.re)/2).sqrt;
+        my Num $im = (($abs - self.re)/2).sqrt;
+        Complex.new($re, self.im < 0 ?? -$im !! $im);
     }
 
     multi method exp(Complex:D:) {
@@ -208,7 +216,7 @@ my class Complex is Cool does Numeric {
     multi method round(Complex:D:) {
         Complex.new( self.re.round, self.im.round );
     }
-    multi method round(Complex:D: Real(Real) $scale) {
+    multi method round(Complex:D: Real() $scale) {
         Complex.new( self.re.round($scale), self.im.round($scale) );
     }
 
@@ -217,7 +225,9 @@ my class Complex is Cool does Numeric {
     }
 
     method narrow(Complex:D:) {
-        $!im == 0e0
+        self == 0e0 ?? 0 !!
+        $!re == 0e0 ?? self !!
+        $!im / $!re ≅ 0e0
             ?? $!re.narrow
             !! self;
     }
@@ -418,22 +428,54 @@ multi sub infix:</>(Real \a, Complex:D \b) returns Complex:D {
 }
 
 multi sub infix:<**>(Complex:D \a, Complex:D \b) returns Complex:D {
-    (a.re == 0e0 && a.im == 0e0) ?? Complex.new(0e0, 0e0) !! (b * a.log).exp
+    (a.re == 0e0 && a.im == 0e0)
+        ?? ( b.re == 0e0 && b.im == 0e0
+                ?? Complex.new(1e0, 0e0)
+                !! Complex.new(0e0, 0e0)
+           )
+        !! (b * a.log).exp
 }
 multi sub infix:<**>(Num(Real) \a, Complex:D \b) returns Complex:D {
-    a == 0e0 ?? Complex.new(0e0, 0e0) !! (b * a.log).exp
+    a == 0e0
+        ?? ( b.re == 0e0 && b.im == 0e0
+                ?? Complex.new(1e0, 0e0)
+                !! Complex.new(0e0, 0e0)
+           )
+        !! (b * a.log).exp
 }
 multi sub infix:<**>(Complex:D \a, Num(Real) \b) returns Complex:D {
-    (b * a.log).exp
+    b == 0e0 ?? Complex.new(1e0, 0e0) !! (b * a.log).exp
 }
 
 multi sub infix:<==>(Complex:D \a, Complex:D \b) returns Bool:D { a.re == b.re && a.im == b.im }
 multi sub infix:<==>(Complex:D \a, Num(Real) \b) returns Bool:D { a.re == b    && a.im == 0e0  }
 multi sub infix:<==>(Num(Real) \a, Complex:D \b) returns Bool:D { a    == b.re && 0e0  == b.im }
 multi sub infix:<===>(Complex:D \a, Complex:D \b) returns Bool:D {
-    a.WHAT =:= b.WHAT && a == b
+    a.WHAT =:= b.WHAT && a.re === b.re && a.im === b.im
 }
 
+multi sub infix:<≅>(Complex:D \a, Complex:D \b) returns Bool:D { a.re ≅ b.re && a.im ≅ b.im || a <=> b =:= Same }
+multi sub infix:<≅>(Complex:D \a, Num(Real) \b) returns Bool:D { a ≅ b.Complex }
+multi sub infix:<≅>(Num(Real) \a, Complex:D \b) returns Bool:D { a.Complex ≅ b }
+
+# Meaningful only for sorting purposes, of course.
+# We delegate to Real::cmp rather than <=> because parts might be NaN.
+multi sub infix:<cmp>(Complex:D \a, Complex:D \b) returns Order:D { a.re cmp b.re || a.im cmp b.im }
+multi sub infix:<cmp>(Num(Real) \a, Complex:D \b) returns Order:D { a cmp b.re || 0 cmp b.im }
+multi sub infix:<cmp>(Complex:D \a, Num(Real) \b) returns Order:D { a.re cmp b || a.im cmp 0 }
+
+multi sub infix:«<=>»(Complex:D \a, Complex:D \b) returns Order:D {
+    my $tolerance = a && b
+        ?? (a.re.abs + b.re.abs) / 2 * $*TOLERANCE  # Scale slop to average real parts.
+        !! $*TOLERANCE;                             # Don't want tolerance 0 if either arg is 0.
+    # Fail unless imaginary parts are relatively negligible, compared to real parts.
+    infix:<≅>(a.im, 0e0, :$tolerance) && infix:<≅>(b.im, 0e0, :$tolerance)
+      ?? a.re <=> b.re
+      !! Failure.new(X::Numeric::Real.new(target => Real, reason => "Complex is not numerically orderable", source => "Complex"))
+}
+multi sub infix:«<=>»(Num(Real) \a, Complex:D \b) returns Order:D { a.Complex <=> b }
+multi sub infix:«<=>»(Complex:D \a, Num(Real) \b) returns Order:D { a <=> b.Complex }
+
 proto sub postfix:<i>(\a) returns Complex:D is pure { * }
 multi sub postfix:<i>(Real      \a) returns Complex:D { Complex.new(0e0, a);     }
 multi sub postfix:<i>(Complex:D \a) returns Complex:D { Complex.new(-a.im, a.re) }
