@@ -4,13 +4,34 @@ my class Cursor does NQPCursorRole {
     my Mu $NO_CAPS    := nqp::hash();
 
     multi method Bool(Cursor:D:) {
-        nqp::getattr_i(self, Cursor, '$!pos') >= nqp::getattr_i(self, Cursor, '$!from')
+        nqp::p6bool(
+          nqp::isge_i(
+            nqp::getattr_i(self, Cursor, '$!pos'),
+            nqp::getattr_i(self, Cursor, '$!from')
+          )
+        )
+    }
+
+    method STR() {
+        nqp::if(
+          nqp::istype((my $match := nqp::getattr(self,Cursor,'$!match')),Match)
+            && nqp::isconcrete($match),
+          $match.Str,
+          self!MATCH.Str
+        )
     }
 
     method MATCH() {
-        my $match := nqp::getattr(self, Cursor, '$!match');
-        return $match if nqp::istype($match, Match) && nqp::isconcrete($match);
-        $match := nqp::create(Match);
+        nqp::if(
+          nqp::istype((my $match := nqp::getattr(self,Cursor,'$!match')),Match)
+            && nqp::isconcrete($match),
+          $match,
+          self!MATCH
+        )
+    }
+
+    method !MATCH() {
+        my $match := nqp::create(Match);
         nqp::bindattr($match, Match, '$!orig', nqp::findmethod(self, 'orig')(self));
         my int $from = nqp::getattr_i(self, Cursor, '$!from');
         my int $to   = nqp::getattr_i(self, Cursor, '$!pos');
@@ -142,8 +163,8 @@ my class Cursor does NQPCursorRole {
                 }
             }
         }
-        nqp::bindattr($match, Capture, '$!list', nqp::isconcrete($list) ?? $list !! $EMPTY_LIST);
-        nqp::bindattr($match, Capture, '$!hash', $hash);
+        nqp::bindattr($match, Capture, '@!list', nqp::isconcrete($list) ?? $list !! $EMPTY_LIST);
+        nqp::bindattr($match, Capture, '%!hash', $hash);
         nqp::bindattr(self, Cursor, '$!match', $match);
 
         # Once we've produced the captures, and if we know we're finished and
@@ -156,8 +177,42 @@ my class Cursor does NQPCursorRole {
         $match;
     }
 
-    method MATCH_SAVE() {
-        nqp::getattr_i(self, Cursor, '$!pos') < 0 ?? Nil !! self.MATCH()
+    method CURSOR_NEXT() {   # from !cursor_next in nqp
+        nqp::if(
+          nqp::defined($!restart),
+          $!restart(self),
+          nqp::stmts(
+            (my $cur := self."!cursor_start_cur"()),
+            $cur."!cursor_fail"(),
+            $cur
+          )
+        )
+    }
+
+    method CURSOR_OVERLAP() {  # adapted from !cursor_more in nqp
+        nqp::stmts(
+          (my $new := nqp::create(self)),
+          nqp::bindattr(  $new,$?CLASS,'$!shared',$!shared),
+          nqp::bindattr_i($new,$?CLASS,'$!from',-1),
+          nqp::bindattr_i($new,$?CLASS,'$!pos',nqp::add_i($!from,1)),
+          $!regexsub($new)
+        )
+    }
+
+    method CURSOR_MORE() {  # adapted from !cursor_more in nqp
+        nqp::stmts(
+          (my $new := nqp::create(self)),
+          nqp::bindattr(  $new,$?CLASS,'$!shared',$!shared),
+          nqp::bindattr_i($new,$?CLASS,'$!from',-1),
+          nqp::bindattr_i($new,$?CLASS,'$!pos',
+            nqp::if(
+              nqp::isge_i($!from,$!pos),
+              nqp::add_i($!from,1),
+              $!pos
+            )
+          ),
+          $!regexsub($new)
+        )
     }
 
     # INTERPOLATE will iterate over the string $tgt beginning at position 0.
