@@ -15,7 +15,8 @@ class Distro does Systemic {
       :$!auth,
       :$!path-sep,
       :$!signature  = Blob,
-      :$!desc = Str,
+      :$!desc = Str
+      --> Nil
     ) {
         $!name = $name.lc;    # lowercase
         $!name ~~ s:g/" "//;  # spaceless
@@ -61,7 +62,7 @@ sub INITIALIZE-A-DISTRO-NOW() {
         }
         $auth := 'Apple Computer, Inc.'; # presumably
     }
-    elsif '/etc/os-release'.IO.e {
+    elsif Rakudo::Internals.FILETEST-E('/etc/os-release') {
         $_ := '/etc/os-release'.IO.slurp.subst(:g, /'"'/,'');
         $auth    := ~$0 if m/^^ HOME_URL   \= (\N*) /;
         $name    := ~$0 if m/^^ ID         \= (\N*) /;
@@ -85,7 +86,7 @@ sub INITIALIZE-A-DISTRO-NOW() {
 }
 
 # set up $*DISTRO
-multi sub INITIALIZE_DYNAMIC('$*DISTRO') {
+Rakudo::Internals.REGISTER-DYNAMIC: '$*DISTRO', {
     PROCESS::<$DISTRO> := INITIALIZE-A-DISTRO-NOW();
 }
 
