@@ -21,6 +21,9 @@ my role Enumeration {
     multi method Numeric(::?CLASS:D:) { $!value.Numeric }
     multi method Int(::?CLASS:D:)     { $!value.Int }
 
+    # Make sure we always accept any element of the enumeration
+    multi method ACCEPTS(::?CLASS:D: ::?CLASS:U) { True }
+
     method CALL-ME(|) {
         my $x := nqp::atpos(nqp::p6argvmarray(), 1).AT-POS(0);
         nqp::istype($x, ::?CLASS)
@@ -53,10 +56,9 @@ sub ENUM_VALUES(*@args) {
             %res{$_} = $prev.=succ;
         }
     }
-    my $r := nqp::create(Map);
-    nqp::bindattr($r, Map, '$!storage',
-        nqp::getattr(%res, Map, '$!storage'));
-    $r;
+    nqp::p6bindattrinvres(
+      nqp::create(Map),Map,'$!storage',nqp::getattr(%res,Map,'$!storage')
+    )
 }
 
 Metamodel::EnumHOW.set_composalizer(-> $type, $name, %enum_values {
