@@ -7,12 +7,12 @@
     while $enviter {
         $envelem := nqp::shift($enviter);
         $key = nqp::p6box_s(nqp::iterkey_s($envelem));
-        %ENV{$key} = nqp::p6box_s(nqp::iterval($envelem));
+        %ENV{$key} = val(nqp::p6box_s(nqp::iterval($envelem)));
     }
     PROCESS::<%ENV> := %ENV;
 }
 
-multi sub INITIALIZE_DYNAMIC('$*CWD') {
+Rakudo::Internals.REGISTER-DYNAMIC: '$*CWD', {
 #    PROCESS::<$CWD> = nqp::p6box_s(nqp::cwd());
     my $CWD := nqp::p6box_s(nqp::cwd());
     PROCESS::<$CWD> = IO::Path.new($CWD, :$CWD); # need :CWD to prevent looping
