@@ -64,6 +64,8 @@ my class Exception {
         nqp::throw($!ex)
     }
     method rethrow(Exception:D:) {
+        nqp::bindattr(self, Exception, '$!ex', nqp::newexception())
+            unless nqp::isconcrete($!ex);
         nqp::setpayload($!ex, nqp::decont(self));
         nqp::rethrow($!ex)
     }
@@ -105,7 +107,8 @@ my class X::SecurityPolicy::Eval is X::SecurityPolicy {
                     .Str;
                 }
             }
-        } ~ " (use MONKEY-SEE-NO-EVAL to override,\nbut only if you're VERY sure your data contains no injection attacks)";
+        } ~ " (use the MONKEY-SEE-NO-EVAL pragma to override this error,\n"
+          ~ "but only if you're VERY sure your data contains no injection attacks)";
     }
     method Numeric() { $.payload.Numeric }
     method from-slurpy (|cap) {
@@ -148,7 +151,7 @@ my class X::Dynamic::NotFound is Exception {
     }
 }
 my class X::Method::NotFound is Exception {
-    has $.invocant;
+    has Mu $.invocant;
     has $.method;
     has $.typename;
     has Bool $.private = False;
@@ -236,6 +239,15 @@ my class CX::Succeed does X::Control {
 my class CX::Proceed does X::Control {
     method message() { "<proceed control exception>" }
 }
+my class CX::Return does X::Control {
+    method message() { "<return control exception>" }
+}
+my class CX::Emit does X::Control {
+    method message() { "<emit control exception>" }
+}
+my class CX::Done does X::Control {
+    method message() { "<done control exception>" }
+}
 
 sub EXCEPTION(|) {
     my Mu $vm_ex   := nqp::shift(nqp::p6argvmarray());
@@ -246,13 +258,13 @@ sub EXCEPTION(|) {
     } else {
         my int $type = nqp::getextype($vm_ex);
         my $ex;
-        if $type == nqp::const::CONTROL_NEXT {
+        if $type +& nqp::const::CONTROL_NEXT {
             $ex := CX::Next.new();
         }
-        elsif $type == nqp::const::CONTROL_REDO {
+        elsif $type +& nqp::const::CONTROL_REDO {
             $ex := CX::Redo.new();
         }
-        elsif $type == nqp::const::CONTROL_LAST {
+        elsif $type +& nqp::const::CONTROL_LAST {
             $ex := CX::Last.new();
         }
         elsif $type == nqp::const::CONTROL_TAKE {
@@ -269,6 +281,17 @@ sub EXCEPTION(|) {
         elsif $type == nqp::const::CONTROL_PROCEED {
             $ex := CX::Proceed.new();
         }
+        elsif $type == nqp::const::CONTROL_RETURN {
+            $ex := CX::Return.new();
+        }
+        elsif $type == nqp::const::CONTROL_EMIT {
+            $ex := CX::Emit.new();
+        }
+        elsif $type == nqp::const::CONTROL_DONE {
+            $ex := CX::Done.new();
+        }
+#?if !moar
+        # for MoarVM this check is done in src/Perl6/Metamodel/BOOTSTRAP.nqp, cmp 222d16b0b9
         elsif !nqp::isnull_s(nqp::getmessage($vm_ex)) &&
                 nqp::p6box_s(nqp::getmessage($vm_ex)) ~~ /"Method '" (.*?) "' not found for invocant of class '" (.+)\'$/ {
             $ex := X::Method::NotFound.new(
@@ -276,9 +299,10 @@ sub EXCEPTION(|) {
                 typename => ~$1,
             );
         }
+#?endif
         else {
             $ex := nqp::create(X::AdHoc);
-            nqp::bindattr($ex, X::AdHoc, '$!payload', nqp::p6box_s(nqp::getmessage($vm_ex)));
+            nqp::bindattr($ex, X::AdHoc, '$!payload', nqp::p6box_s(nqp::getmessage($vm_ex) // 'unknown exception'));
         }
         nqp::bindattr($ex, Exception, '$!ex', $vm_ex);
         $ex;
@@ -305,8 +329,17 @@ do {
 
     sub print_exception(|) {
         my Mu $ex := nqp::atpos(nqp::p6argvmarray(), 0);
+        my $e := EXCEPTION($ex);
+
+        if %*ENV<RAKUDO_EXCEPTIONS_HANDLER> -> $handler {
+            my $class := ::("Exceptions::$handler");
+            unless nqp::istype($class,Failure) {
+                temp %*ENV<RAKUDO_EXCEPTIONS_HANDLER> = ""; # prevent looping
+                return unless $class.process($e)
+            }
+        }
+
         try {
-            my $e := EXCEPTION($ex);
             my $v := $e.vault-backtrace;
             my Mu $err := nqp::getstderr();
 
@@ -338,16 +371,23 @@ do {
     }
 
     sub print_control(|) {
-        my Mu $ex := nqp::atpos(nqp::p6argvmarray(), 0);
-        my int $type = nqp::getextype($ex);
-        my $backtrace = Backtrace.new(nqp::backtrace($ex), 0);
-        if ($type == nqp::const::CONTROL_WARN) {
-            my Mu $err := nqp::getstderr();
-            my $msg = nqp::p6box_s(nqp::getmessage($ex));
-            nqp::printfh($err, $msg.chars ?? "$msg" !! "Warning");
-            nqp::printfh($err, $backtrace.first-none-setting-line);
-            nqp::resume($ex)
-        }
+        nqp::stmts(
+          (my Mu $ex := nqp::atpos(nqp::p6argvmarray(),0)),
+          (my int $type = nqp::getextype($ex)),
+          (my $backtrace = Backtrace.new(nqp::backtrace($ex))),
+          nqp::if(
+            nqp::iseq_i($type,nqp::const::CONTROL_WARN),
+            nqp::stmts(
+              (my Mu $err := nqp::getstderr),
+              (my str $msg = nqp::getmessage($ex)),
+              nqp::printfh($err,nqp::if(nqp::chars($msg),$msg,"Warning")),
+              nqp::printfh($err, "\n"),
+              nqp::printfh($err, $backtrace.first-none-setting-line),
+              nqp::resume($ex)
+            )
+          )
+        );
+
         my $label = $type +& nqp::const::CONTROL_LABELED ?? "labeled " !! "";
         if $type +& nqp::const::CONTROL_LAST {
             X::ControlFlow.new(illegal => "{$label}last", enclosing => 'loop construct', :$backtrace).throw;
@@ -541,7 +581,11 @@ my role X::Comp is Exception {
             my ($red,$clear,$green,$yellow,$eject) =
               Rakudo::Internals.error-rcgye;
             my $r = $sorry ?? self.sorry_heading() !! "";
-            $r ~= "$.message\nat $.filename():$.line";
+            $r ~= $.filename eq '<unknown file>'
+              ?? $.line == 1
+                ?? $.message
+                !! "$.message\nat line $.line"
+              !! "$.message\nat $.filename():$.line";
             $r ~= "\n------> $green$.pre$yellow$eject$red$.post$clear" if defined $.pre;
             if $expect && @.highexpect {
                 $r ~= "\n    expecting any of:";
@@ -563,7 +607,11 @@ my role X::Comp is Exception {
     }
     method sorry_heading() {
         my ($red, $clear) = Rakudo::Internals.error-rcgye;
-        "$red==={$clear}SORRY!$red===$clear Error while compiling $.filename\n"
+        "$red==={$clear}SORRY!$red===$clear Error while compiling{
+          $.filename eq '<unknown file>'
+            ?? ':'
+            !! " $.filename"
+        }\n"
     }
     method SET_FILE_LINE($file, $line) {
         $!filename = $file;
@@ -710,8 +758,14 @@ Parenthesize as \\(...) if you intended a capture of a single numeric value./
 my class X::Worry::P5::LeadingZero is X::Worry::P5 {
     has $.value;
     method message {
-qq/Leading 0 does not indicate octal in Perl 6.
-Please use 0o$!value if you mean that./
+        if $!value ~~ /<[89]>/ {
+            "Leading 0 is not allowed. For octals, use '0o' prefix,"
+            ~ " but note that $!value is not a valid octal number";
+        }
+        else {
+            'Leading 0 does not indicate octal in Perl 6.'
+                ~ " Please use 0o$!value if you mean that.";
+        }
     }
 }
 
@@ -809,7 +863,7 @@ my class X::Placeholder::NonPlaceholder does X::Comp {
 
 my class X::Placeholder::Mainline is X::Placeholder::Block {
     method message() {
-        "Cannot use placeholder parameter $.placeholder in the mainline"
+        "Cannot use placeholder parameter $.placeholder outside of a sub or block"
     }
 }
 
@@ -911,7 +965,9 @@ my class X::Redeclaration does X::Comp {
     has $.postfix = '';
     has $.what    = 'symbol';
     method message() {
-        "Redeclaration of $.what $.symbol$.postfix";
+        "Redeclaration of $.what '$.symbol'"
+          ~ (" $.postfix" if $.postfix)
+          ~ (" (did you mean to declare a multi-sub?)" if $.what eq 'routine');
     }
 }
 
@@ -1167,6 +1223,17 @@ my class X::Invalid::Value is Exception {
     }
 }
 
+my class X::Invalid::ComputedValue is Exception {
+    has $.method;
+    has $.name;
+    has $.value;
+    has $.reason;
+    method message {
+        "$.name on $.method computed to $.value, which cannot be used"
+            ~ (" because $.reason" if $.reason);
+    }
+}
+
 my class X::Value::Dynamic does X::Comp {
     has $.what;
     method message() { "$.what value must be known at compile time" }
@@ -1177,7 +1244,13 @@ my class X::Syntax::Name::Null does X::Syntax {
 }
 
 my class X::Syntax::UnlessElse does X::Syntax {
-    method message() { '"unless" does not take "else", please rewrite using "if"' }
+    has $.keyword;
+    method message() { qq|"unless" does not take "$!keyword", please rewrite using "if"| }
+}
+
+my class X::Syntax::WithoutElse does X::Syntax {
+    has $.keyword;
+    method message() { qq|"without" does not take "$!keyword", please rewrite using "with"| }
 }
 
 my class X::Syntax::KeywordAsFunction does X::Syntax {
@@ -1228,7 +1301,7 @@ my class X::Syntax::Variable::Twigil does X::Syntax {
     has $.what = 'variable';
     has $.twigil;
     has $.scope;
-    has $.additional;
+    has $.additional = '';
     method message() { "Cannot use $.twigil twigil on '$.scope' $.what$.additional" }
 }
 
@@ -1449,6 +1522,12 @@ my class X::Syntax::Number::LiteralType does X::Syntax {
 my class X::Syntax::NonAssociative does X::Syntax {
     has $.left;
     has $.right;
+    method message() {
+        "Operators '$.left' and '$.right' are non-associative and require parentheses";
+    }
+}
+
+my class X::Syntax::NonListAssociative is X::Syntax::NonAssociative {
     method message() {
         "Only identical operators may be list associative; since '$.left' and '$.right' differ, they are non-associative and you need to clarify with parentheses";
     }
@@ -1491,7 +1570,10 @@ my class X::Syntax::Regex::NullRegex does X::Syntax {
 }
 
 my class X::Syntax::Regex::MalformedRange does X::Syntax {
-    method message() { 'Malformed Range' }
+    method message() {
+        'Malformed Range. If attempting to use variables for end points, '
+        ~ 'wrap the entire range in curly braces.'
+    }
 }
 
 my class X::Syntax::Regex::Unspace does X::Syntax {
@@ -1593,7 +1675,7 @@ my class X::Syntax::Extension::SpecialForm does X::Syntax {
 my class X::Syntax::InfixInTermPosition does X::Syntax {
     has $.infix;
     method message() {
-        "Preceding context expects a term, but found infix $.infix instead";
+        "Preceding context expects a term, but found infix {$.infix.trim} instead";
     }
 }
 
@@ -1617,7 +1699,12 @@ my class X::Attribute::NoPackage does X::Comp {
 }
 my class X::Attribute::Required does X::MOP {
     has $.name;
-    method message() { "The attribute '$.name' is required, but you did not provide a value for it." }
+    has $.why;
+    method message() {
+        $.why && nqp::istype($.why,Str)
+          ?? "The attribute '$.name' is required because $.why,\nbut you did not provide a value for it."
+          !! "The attribute '$.name' is required, but you did not provide a value for it."
+    }
 }
 my class X::Attribute::Scope::Package does X::Comp {
     has $.scope;
@@ -1672,7 +1759,21 @@ my class X::Constructor::Positional is Exception {
 }
 
 my class X::Hash::Store::OddNumber is Exception {
-    method message() { "Odd number of elements found where hash initializer expected" }
+    has $.found;
+    has $.last;
+    method message() {
+        my $msg =
+          "Odd number of elements found where hash initializer expected";
+        if $.found == 1 {
+            $msg ~= $.last
+              ?? ":\nOnly saw: $.last.perl()"
+              !! ":\nOnly saw 1 element"
+        }
+        else {
+            $msg ~= ":\nFound $.found (implicit) elements";
+            $msg ~= ":\nLast element seen: $.last.perl()" if $.last;
+        }
+    }
 }
 
 my class X::Pairup::OddNumber is Exception {
@@ -1743,6 +1844,14 @@ my class X::Str::Match::x is Exception {
     }
 }
 
+my class X::Str::Subst::Adverb is Exception {
+    has $.name;
+    has $.got;
+    method message() {
+        "Cannot use :$.name adverb in Str.subst, got $.got"
+    }
+}
+
 my class X::Str::Trans::IllegalKey is Exception {
     has $.key;
     method message {
@@ -1854,7 +1963,7 @@ my class X::Composition::NotComposable does X::Comp {
     has $.target-name;
     has $.composer;
     method message() {
-        $.composer.^name ~ " is not composable, so $.target-name cannot compose it";
+        $!composer.^name ~ " is not composable, so $!target-name cannot compose it";
     }
 }
 
@@ -1894,30 +2003,36 @@ my class X::TypeCheck::Binding is X::TypeCheck {
     has $.symbol;
     method operation { 'binding' }
     method message() {
-        if $.symbol {
-            self.priors() ~
-            "Type check failed in $.operation $.symbol; expected $.expectedn but got $.gotn";
-        } else {
-            self.priors() ~
-            "Type check failed in $.operation; expected $.expectedn but got $.gotn";
-        }
+        my $to = $.symbol.defined && $.symbol ne '$'
+            ?? " to $.symbol" !! "";
+        my $expected = $.expected =:= $.got
+            ?? "expected type $.expectedn cannot be itself"
+            !! "expected $.expectedn but got $.gotn";
+        self.priors() ~ "Type check failed in $.operation$to; $expected";
     }
 }
 my class X::TypeCheck::Return is X::TypeCheck {
     method operation { 'returning' }
     method message() {
+        my $expected = $.expected =:= $.got
+            ?? "expected return type $.expectedn cannot be itself " ~
+               "(perhaps $.operation a :D type object?)"
+            !! "expected $.expectedn but got $.gotn";
         self.priors() ~
-        "Type check failed for return value; expected $.expectedn but got $.gotn";
+        "Type check failed for return value; $expected";
     }
 }
 my class X::TypeCheck::Assignment is X::TypeCheck {
     has $.symbol;
     method operation { 'assignment' }
     method message {
-        self.priors() ~ do
-            $.symbol.defined && $.symbol ne '$'
-            ?? "Type check failed in assignment to $.symbol; expected $.expectedn but got $.gotn"
-            !! "Type check failed in assignment; expected $.expectedn but got $.gotn";
+        my $to = $.symbol.defined && $.symbol ne '$'
+            ?? " to $.symbol" !! "";
+        my $expected = $.expected =:= $.got
+            ?? "expected type $.expectedn cannot be itself " ~
+               "(perhaps Nil was assigned to a :D which had no default?)"
+            !! "expected $.expectedn but got $.gotn";
+        self.priors() ~ "Type check failed in assignment$to; $expected";
     }
 }
 my class X::TypeCheck::Argument is X::TypeCheck {
@@ -1995,8 +2110,8 @@ my class X::Inheritance::Unsupported does X::Comp {
     has $.child-typename;
     has $.parent;
     method message {
-        $.parent.^name ~ ' does not support inheritance, so '
-        ~ $.child-typename ~ ' cannot inherit from it';
+        $!parent.^name ~ ' does not support inheritance, so '
+        ~ $!child-typename ~ ' cannot inherit from it';
     }
 }
 
@@ -2145,8 +2260,13 @@ my class X::Numeric::Confused is Exception {
     has $.num;
     has $.base;
     method message() {
-        "This call only converts base-$.base strings to numbers; value {$.num.perl} is of type {$.num.WHAT.^name}, so cannot be converted!\n"
-            ~ "(If you really wanted to convert {$.num.perl} to a base-$.base string, use {$.num.perl}.base($.base) instead.)";
+        "This call only converts base-$.base strings to numbers; value "
+        ~ "{$.num.perl} is of type {$.num.WHAT.^name}, so cannot be converted!"
+        ~ (
+            "\n(If you really wanted to convert {$.num.perl} to a base-$.base"
+                ~ " string, use {$.num.perl}.base($.base) instead.)"
+            if $.num.perl.^can('base')
+        );
     }
 }
 
@@ -2185,6 +2305,17 @@ my class X::Multi::NoMatch is Exception {
     has $.capture;
     method message {
         my @cand = $.dispatcher.dispatchees.map(*.signature.gist);
+        my @un-rw-cand;
+        if first / 'is rw' /, @cand {
+            my $rw-capture = Capture.new(
+                :list( $!capture.list.map({ $ = $_ })                  ),
+                :hash( $!capture.hash.map({ .key => $ = .value }).hash ),
+            );
+            @un-rw-cand = $.dispatcher.dispatchees».signature.grep({
+                $rw-capture ~~ $^cand
+            })».gist;
+        }
+
         my $where = so first / where /, @cand;
         my @bits;
         my @priors;
@@ -2204,8 +2335,8 @@ my class X::Multi::NoMatch is Exception {
                     @bits.push(':' ~ ('!' x !.value) ~ .key);
                 }
                 else {
-                    try @bits.push(":$(.key)($($where ?? .value.?perl !! .value.WHAT.?perl ))");
-                    @bits.push($_.value.^name) if $!;
+                    try @bits.push(":$(.key)\($($where ?? .value.?perl !! .value.WHAT.?perl ))");
+                    @bits.push(':' ~ .value.^name) if $!;
                 }
             }
         }
@@ -2219,10 +2350,18 @@ my class X::Multi::NoMatch is Exception {
         }
         my $cap = '(' ~ @bits.join(", ") ~ ')';
         @priors = flat "Earlier failures:\n", @priors, "\nFinal error:\n " if @priors;
-        @priors.join ~
-        join "\n    ",
-            "Cannot resolve caller $.dispatcher.name()$cap; none of these signatures match:",
-            @cand;
+        @priors.join ~ "Cannot resolve caller $.dispatcher.name()$cap; " ~ (
+            @un-rw-cand
+            ?? "the following candidates\nmatch the type but require "
+                ~ 'mutable arguments:' ~  join("\n    ", '', @un-rw-cand) ~ (
+                        "\n\nThe following do not match for other reasons:"
+                        ~  join("\n    ", '', sort keys @cand ∖ @un-rw-cand)
+                    unless @cand == @un-rw-cand
+                )
+            !! join "\n    ",
+                'none of these signatures match:',
+                @cand
+        );
     }
 }
 
@@ -2259,7 +2398,7 @@ my class X::PhaserExceptions is Exception {
 }
 
 nqp::bindcurhllsym('P6EX', nqp::hash(
-  'X::TypeCheck::Binding', 
+  'X::TypeCheck::Binding',
   sub (Mu $got, Mu $expected, $symbol?) {
       X::TypeCheck::Binding.new(:$got, :$expected, :$symbol).throw;
   },
@@ -2284,7 +2423,7 @@ nqp::bindcurhllsym('P6EX', nqp::hash(
       X::NoDispatcher.new(:$redispatcher).throw;
   },
   'X::Method::NotFound',
-  sub ($invocant, $method, $typename, $private = False) {
+  sub (Mu $invocant, $method, $typename, $private = False) {
       X::Method::NotFound.new(:$invocant, :$method, :$typename, :$private).throw
   },
   'X::Multi::Ambiguous',
@@ -2483,7 +2622,7 @@ my class X::Language::Unsupported is Exception {
 my class X::Proc::Unsuccessful is Exception {
     has $.proc;
     method message() {
-        "The spawned process exited unsuccessfully (exit code: $.proc.exitcode())"
+        "The spawned command '{$.proc.command[0]}' exited unsuccessfully (exit code: $.proc.exitcode())"
     }
 }
 
@@ -2505,8 +2644,34 @@ my class X::CompUnit::UnsatisfiedDependency is Exception {
         my $name = $.specification.short-name;
         my $line = $.specification.source-line-number;
         is-core($name)
-            ?? "{$name} is a builtin type. You can use it without loading a module."
-            !! "Could not find $.specification at line $line in:\n" ~ $*REPO.repo-chain.map(*.Str).join("\n").indent(4)
+            ?? "{$name} is a builtin type, not an external module"
+            !! "Could not find $.specification at line $line in:\n"
+                ~ $*REPO.repo-chain.map(*.Str).join("\n").indent(4)
+                ~ ($.specification ~~ / $<name>=.+ '::from' $ /
+                    ?? "\n\nIf you meant to use the :from adverb, use"
+                        ~ " a single colon for it: $<name>:from<...>\n"
+                    !! ''
+                )
+    }
+}
+
+my class Exceptions::JSON {
+    method process($ex) {
+        nqp::printfh(
+          nqp::getstderr,
+          Rakudo::Internals::JSON.to-json( $ex.^name => Hash.new(
+            (message => $ex.?message),
+            $ex.^attributes.grep(*.has_accessor).map: {
+                with .name.substr(2) -> $attr {
+                    $attr => (
+                        (.defined and not $_ ~~ Real|Positional|Associative)
+                        ?? .Str !! $_
+                    ) given $ex."$attr"()
+                }
+            }
+          ))
+        );
+        False  # done processing
     }
 }
 
