@@ -8,6 +8,7 @@ my class IO::Handle does IO {
     has $.chomp is rw = Bool::True;
     has $.nl-in = ["\x0A", "\r\n"];
     has Str:D $.nl-out is rw = "\n";
+    has str $!encoding = 'utf8';
 
     method open(IO::Handle:D:
       :$r, :$w, :$x, :$a, :$update,
@@ -68,8 +69,7 @@ my class IO::Handle does IO {
 #?if !jvm
             Rakudo::Internals.SET_LINE_ENDING_ON_HANDLE($!PIO, $!nl-in = $nl-in);
 #?endif
-            nqp::setencoding($!PIO, Rakudo::Internals.NORMALIZE_ENCODING($enc))
-              unless $bin;
+            self.encoding( $bin ?? 'bin' !! $enc );
             return self;
         }
 
@@ -100,8 +100,7 @@ my class IO::Handle does IO {
         $!chomp = $chomp;
         $!nl-out = $nl-out;
         Rakudo::Internals.SET_LINE_ENDING_ON_HANDLE($!PIO, $!nl-in = $nl-in);
-        nqp::setencoding($!PIO, Rakudo::Internals.NORMALIZE_ENCODING($enc))
-          unless $bin;
+        self.encoding( $bin ?? 'bin' !! $enc );
         self;
     }
 
@@ -128,16 +127,24 @@ my class IO::Handle does IO {
 
     method get(IO::Handle:D:) {
         my str $str;
-        if $!chomp {
-            $str = nqp::readlinechompfh($!PIO);
-            # loses last empty line because EOF is set too early, RT #126598
-            nqp::chars($str) || !nqp::eoffh($!PIO) ?? $str !! Nil
-        }
-        else {
-            $str = nqp::readlinefh($!PIO);
-            # no need to check EOF
-            nqp::chars($str) ?? $str !! Nil
-        }
+        nqp::if($!chomp,
+            nqp::stmts(
+                ($str = nqp::readlinechompfh($!PIO)),
+                # loses last empty line because EOF is set too early, RT #126598
+                nqp::if(nqp::chars($str) || !nqp::eoffh($!PIO),
+                    $str,
+                    Nil
+                )
+            ),
+            nqp::stmts(
+                ($str = nqp::readlinefh($!PIO)),
+                # no need to check EOF
+                nqp::if(nqp::chars($str),
+                    $str,
+                    Nil
+                )
+            )
+        )
     }
 
     method getc(IO::Handle:D:) {
@@ -180,10 +187,10 @@ my class IO::Handle does IO {
                   )
                 )
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 my str $str = $!handle.readchars($!size);
                 nqp::while(
-                  nqp::iseq_i(nqp::chars($str),$!size), 
+                  nqp::iseq_i(nqp::chars($str),$!size),
                   nqp::stmts(
                     $target.push(nqp::p6box_s($str)),
                     ($str = $!handle.readchars($!size))
@@ -191,21 +198,6 @@ my class IO::Handle does IO {
                 );
                 $target.push(nqp::p6box_s($str)) if nqp::chars($str);
                 $!handle.close if $!close;
-                IterationEnd
-            }
-            method count-only() {
-                my int $found = 0;
-                my str $str = $!handle.readchars($!size);
-                nqp::while(
-                  nqp::iseq_i(nqp::chars($str),$!size),
-                  nqp::stmts(
-                    ($found = nqp::add_i($found,1)),
-                    ($str = $!handle.readchars($!size)),
-                  )
-                );
-                ++$found if nqp::chars($str);
-                $!handle.close if $!close;
-                nqp::p6box_i($found)
             }
         }.new(self, $size, +$close));
     }
@@ -275,7 +267,6 @@ my class IO::Handle does IO {
                         $!elems = nqp::elems($!strings);
                     }
                 }
-                Nil
             }
             method pull-one() {
                 if $!elems {
@@ -294,7 +285,7 @@ my class IO::Handle does IO {
                     }
                 }
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 while $!elems {
                     while $!elems {
                         $target.push(nqp::p6box_s(nqp::shift_s($!strings)));
@@ -303,17 +294,6 @@ my class IO::Handle does IO {
                     self!next-chunk until $!elems || $!done;
                 }
                 $!handle.close if $!close;
-                IterationEnd
-            }
-            method count-only() {
-                my int $found;
-                while $!elems {
-                    $found  = $found + $!elems;
-                    $!elems = 0;
-                    self!next-chunk until $!elems || $!done;
-                }
-                $!handle.close if $!close;
-                nqp::p6box_i($found)
             }
         }.new(self, $comber, +$close));
     }
@@ -344,7 +324,6 @@ my class IO::Handle does IO {
                 $!str   = $!handle.readchars;
                 $!index = 0;
                 $!chars = nqp::chars($!str);
-                Nil
             }
             method pull-one() {
                 self!next-chunk if !$!index == $!chars;
@@ -364,7 +343,7 @@ my class IO::Handle does IO {
                     IterationEnd;
                 }
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 $target.push('') if $!first;
                 while $!index < $!chars {
                     $target.push(
@@ -374,16 +353,6 @@ my class IO::Handle does IO {
                 }
                 $target.push('') if $!last;
                 $!handle.close if $!close;
-                IterationEnd
-            }
-            method count-only() {
-                my int $found = $!first + $!last;
-                while $!chars {
-                    $found = $found + $!chars;
-                    self!next-chunk();
-                }
-                $!handle.close if $!close;
-                nqp::p6box_i($found)
             }
         }.new(self, +$close, $COMB));
     }
@@ -448,7 +417,6 @@ my class IO::Handle does IO {
                     }
                     $!elems = nqp::elems($!strings);
                 }
-                Nil
             }
             method pull-one() {
                 if $!elems {
@@ -471,7 +439,7 @@ my class IO::Handle does IO {
                     }
                 }
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 while $!elems {
                     while $!elems {
                         $target.push(nqp::p6box_s(nqp::shift($!strings)));
@@ -481,17 +449,6 @@ my class IO::Handle does IO {
                 }
                 $target.push(nqp::p6box_s($!str));
                 $!handle.close if $!close;
-                IterationEnd
-            }
-            method count-only() {
-                my int $found = 1;
-                while $!elems {
-                    $found = $found + $!elems;
-                    $!elems = 0;
-                    self!next-chunk until $!elems || $!done;
-                }
-                $!handle.close if $!close;
-                nqp::p6box_i($found)
             }
         }.new(self, $splitter, +$close));
     }
@@ -561,7 +518,7 @@ my class IO::Handle does IO {
                     IterationEnd
                 }
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 my int $chars;
                 my int $left;
                 my int $nextpos;
@@ -584,30 +541,6 @@ my class IO::Handle does IO {
                 $target.push(nqp::p6box_s(nqp::substr($!str,$!pos)))
                   if $!pos < $chars;
                 $!handle.close if $close;
-                IterationEnd
-            }
-            method count-only() {
-                my int $found;
-                my int $chars;
-                my int $left;
-                my int $nextpos;
-
-                while ($chars = nqp::chars($!str)) && $!searching {
-                    while ($left = $chars - $!pos) > 0 {
-                        $nextpos = nqp::findcclass(
-                          nqp::const::CCLASS_WHITESPACE,$!str,$!pos,$left);
-                        last unless $left = $chars - $nextpos; # broken word
-
-                        $found = $found + 1;
-
-                        $!pos = nqp::findnotcclass(
-                          nqp::const::CCLASS_WHITESPACE,$!str,$nextpos,$left);
-                    }
-                    self!next-chunk;
-                }
-                $found = $found + 1 if $!pos < $chars;
-                $!handle.close if $!close;
-                nqp::p6box_i($found)
             }
         }.new(self, $close));
     }
@@ -637,26 +570,16 @@ my class IO::Handle does IO {
                     IterationEnd
                 }
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 my $line;
                 $target.push($line) while ($line := $!handle.get).DEFINITE;
                 $!handle.close if $close;
-                IterationEnd
-            }
-            method count-only() {
-                my $line;
-                my int $seen;
-                $seen = $seen + 1 while ($line := $!handle.get).DEFINITE;
-                $!handle.close if $!close;
-                $seen
             }
         }.new(self, $close));
     }
 
     method read(IO::Handle:D: Int(Cool:D) $bytes) {
-        my $buf := buf8.new();
-        nqp::readfh($!PIO, $buf, nqp::unbox_i($bytes));
-        $buf;
+        nqp::readfh($!PIO,buf8.new,nqp::unbox_i($bytes))
     }
 
     method readchars(Int(Cool:D) $chars = $*DEFAULT-READ-ELEMS) {
@@ -730,6 +653,9 @@ my class IO::Handle does IO {
         nqp::unlockfh($!PIO);
     }
 
+    method printf(IO::Handle:D: |c) {
+        nqp::printfh($!PIO, sprintf |c);
+    }
 
     proto method print(|) { * }
     multi method print(IO::Handle:D: str:D \x --> True) {
@@ -768,16 +694,18 @@ my class IO::Handle does IO {
     }
 
     proto method slurp-rest(|) { * }
-    multi method slurp-rest(IO::Handle:D: :$bin! where *.so) returns Buf {
-        my $res := buf8.new();
+    multi method slurp-rest(IO::Handle:D: :$bin! where *.so, :$close) returns Buf {
+        LEAVE self.close if $close;
+        my $res := buf8.new;
         loop {
             my $buf := nqp::readfh($!PIO,buf8.new,0x100000);
-            last unless nqp::elems($buf);
-            $res.push($buf);
+            nqp::elems($buf)
+              ?? $res.append($buf)
+              !! return $res
         }
-        $res
     }
-    multi method slurp-rest(IO::Handle:D: :$enc, :$bin) returns Str {
+    multi method slurp-rest(IO::Handle:D: :$enc, :$bin, :$close) returns Str {
+        LEAVE self.close if $close;
         self.encoding($enc) if $enc.defined;
         nqp::p6box_s(nqp::readallfh($!PIO));
     }
@@ -804,10 +732,13 @@ my class IO::Handle does IO {
         nqp::flushfh($!PIO);
     }
 
-    method encoding(IO::Handle:D: $enc?) {
-        $enc.defined
-          ?? nqp::setencoding($!PIO,Rakudo::Internals.NORMALIZE_ENCODING($enc))
-          !! $!PIO.encoding
+    proto method encoding(|) { * }
+    multi method encoding(IO::Handle:D:) { $!encoding }
+    multi method encoding(IO::Handle:D: $enc) {
+        $enc eq 'bin'
+          ?? ($!encoding = 'bin')
+          !! nqp::setencoding($!PIO,
+               $!encoding = Rakudo::Internals.NORMALIZE_ENCODING($enc))
     }
 
     submethod DESTROY(IO::Handle:D:) {
