@@ -13,7 +13,7 @@ my class IO::Path is Cool {
         nqp::p6bool(nqp::iseq_s($.abspath, nqp::unbox_s(other.abspath)));
     }
 
-    multi method ACCEPTS(IO::Path:D: Mu \that) {
+    multi method ACCEPTS(IO::Path:D: Cool:D \that) {
         nqp::p6bool(nqp::iseq_s($.abspath,nqp::unbox_s(IO::Path.new(|that).abspath)));
     }
 
@@ -29,6 +29,7 @@ my class IO::Path is Cool {
         self.bless(:$path, :$SPEC, :$CWD)!fap;
     }
 
+    proto method new(|) {*}
     multi method new(IO::Path: Str(Cool) $path, :$SPEC = $*SPEC, :$CWD = $*CWD) {
         die "Must specify something as a path: did you mean '.' for the current directory?" unless $path.chars;
         self.bless(:$path, :$SPEC, :$CWD);
@@ -47,9 +48,7 @@ my class IO::Path is Cool {
     }
 
     method abspath() {
-        $!abspath //= $!path.starts-with('-')
-          ?? ''
-          !! $!SPEC.rel2abs($!path,$!CWD);
+        $!abspath //= $!SPEC.rel2abs($!path,$!CWD);
     }
     method is-absolute() {
         $!is-absolute //= $!SPEC.is-absolute($!path);
@@ -233,7 +232,7 @@ my class IO::Path is Cool {
     }
 
     method child (IO::Path:D: $child) {
-        self.bless(:path($!SPEC.catfile($!path,$child)), :$!SPEC, :$!CWD);
+        self.bless(:path($!SPEC.join('', $!path, $child)), :$!SPEC, :$!CWD);
     }
 
     proto method chdir(|) { * }
@@ -407,15 +406,18 @@ my class IO::Path is Cool {
               :path($.abspath), :os-error(.Str) );
         } }
 
-        my str $abspath = $.abspath.ends-with($!SPEC.dir-sep)
-          ?? $.abspath
-          !! $.abspath ~ $!SPEC.dir-sep;
+        my str $dir-sep  = $!SPEC.dir-sep;
+        my int $relative = !$absolute && !$.is-absolute;
 
-        my str $path = $.path eq '.' | $!SPEC.dir-sep
+        my str $abspath = $.abspath.ends-with($dir-sep)
+          ?? $.abspath
+          !! $.abspath ~ $dir-sep;
+
+        my str $path = $!path eq '.' || $!path eq $dir-sep
           ?? ''
-          !! $.path.ends-with($!SPEC.dir-sep)
-            ?? $.path
-            !! $.path ~ $!SPEC.dir-sep;
+          !! $!path.ends-with($dir-sep)
+            ?? $!path
+            !! $!path ~ $dir-sep;
 
         my Mu $dirh := nqp::opendir(nqp::unbox_s($.abspath));
         gather {
@@ -432,23 +434,26 @@ my class IO::Path is Cool {
                 }
             }
 #?endif
-            loop {
-                my str $str_elem = nqp::nextfiledir($dirh);
-                if nqp::isnull_s($str_elem) || nqp::chars($str_elem) == 0 {
-                    nqp::closedir($dirh);
-                    last;
-                }
-
-                if $test.ACCEPTS($str_elem) {
-                    $Str
-                      ?? !$absolute && !$.is-absolute
-                        ?? take $path ~ $str_elem
-                        !! take $abspath ~ $str_elem
-                      !! !$absolute && !$.is-absolute
-                        ?? take IO::Path.new($path ~ $str_elem,:$!SPEC,:$CWD)
-                        !! take IO::Path.new-from-absolute-path($abspath ~ $str_elem,:$!SPEC,:$CWD);
-                }
-            }
+            nqp::until(
+              nqp::isnull_s(my str $str_elem = nqp::nextfiledir($dirh))
+                || nqp::iseq_i(nqp::chars($str_elem),0),
+              nqp::if(
+                $test.ACCEPTS($str_elem),
+                nqp::if(
+                  $Str,
+                  (take
+                    nqp::concat(nqp::if($relative,$path,$abspath),$str_elem)),
+                  nqp::if(
+                    $relative,
+                    (take IO::Path.new(
+                      nqp::concat($path,$str_elem),:$!SPEC,:$CWD)),
+                    (take IO::Path.new-from-absolute-path(
+                      nqp::concat($abspath,$str_elem),:$!SPEC,:$CWD))
+                  )
+                )
+              )
+            );
+            nqp::closedir($dirh);
         }
     }
 
@@ -459,21 +464,22 @@ my class IO::Path is Cool {
 
         my Mu $PIO := nqp::getattr(nqp::decont($handle),IO::Handle,'$!PIO');
         if $bin {
+            my $res;
             # normal file
             if Rakudo::Internals.FILETEST-S(self.abspath) -> int $size {
-                nqp::readfh($PIO,buf8.new,$size)
+                $res := nqp::readfh($PIO,buf8.new,$size)
             }
             # spooky file with zero size?
             else {
-                my $res := buf8.new();
+                $res := buf8.new();
                 loop {
                     my $buf := nqp::readfh($PIO,buf8.new,0x100000);
                     last unless nqp::elems($buf);
-                    $res.push($buf);
+                    $res.append($buf);
                 }
-                $handle.close;
-                $res
             }
+            $handle.close;
+            $res
         }
         else {
             $handle.encoding($enc) if $enc.defined;
@@ -538,83 +544,83 @@ my class IO::Path is Cool {
     method d(--> Bool) {
         $.e
           ?? ?Rakudo::Internals.FILETEST-D($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<d>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<d>))
     }
 
     method f(--> Bool) {
         $.e
           ?? ?Rakudo::Internals.FILETEST-F($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<f>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<f>))
     }
 
     method s(--> Int) {
         $.e
           ?? $.f
             ?? Rakudo::Internals.FILETEST-S($!abspath)
-            !! fail X::IO::NotAFile.new(:path(~self),:trying<s>)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<s>)
+            !! Failure.new(X::IO::NotAFile.new(:path(~self),:trying<s>))
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<s>))
     }
 
     method l(--> Bool) {
-        $.e
+        ?Rakudo::Internals.FILETEST-LE($.abspath)
           ?? ?Rakudo::Internals.FILETEST-L($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<l>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<l>))
     }
 
     method r(--> Bool) {
         $.e
           ?? ?Rakudo::Internals.FILETEST-R($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<r>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<r>))
     }
 
     method w(--> Bool) {
         $.e
           ?? ?Rakudo::Internals.FILETEST-W($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<w>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<w>))
     }
 
     method rw(--> Bool) {
         $.e
-          ?? Rakudo::Internals.FILETEST-RW($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<rw>)
+          ?? ?Rakudo::Internals.FILETEST-RW($!abspath)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<rw>))
     }
 
     method x(--> Bool) {
         $.e
           ?? ?Rakudo::Internals.FILETEST-X($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<x>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<x>))
     }
 
     method rwx(--> Bool) {
         $.e
           ?? ?Rakudo::Internals.FILETEST-RWX($!abspath)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<rwx>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<rwx>))
     }
 
     method z(--> Bool) {
         $.e
           ?? $.f
             ?? ?Rakudo::Internals.FILETEST-Z($!abspath)
-            !! fail X::IO::NotAFile.new(:path(~self),:trying<z>)
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<z>)
+            !! Failure.new( X::IO::NotAFile.new(:path(~self),:trying<z>))
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<z>))
     }
 
     method modified(--> Instant) {
         $.e
           ?? Instant.from-posix(Rakudo::Internals.FILETEST-MODIFIED($!abspath))
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<modified>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<modified>))
     }
 
     method accessed(--> Instant) {
         $.e
           ?? Instant.from-posix(Rakudo::Internals.FILETEST-ACCESSED($!abspath))
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<accessed>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<accessed>))
     }
 
     method changed(--> Instant) {
         $.e
           ?? Instant.from-posix(Rakudo::Internals.FILETEST-CHANGED($!abspath))
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<changed>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<changed>))
     }
 
     method mode(--> IntStr) {
@@ -623,7 +629,7 @@ my class IO::Path is Cool {
               (my int $mode = nqp::stat($!abspath, nqp::const::STAT_PLATFORM_MODE) +& 0o7777),
               IntStr.new($mode, sprintf('%04o', $mode))
             )
-          !! fail X::IO::DoesNotExist.new(:path(~self),:trying<mode>)
+          !! Failure.new(X::IO::DoesNotExist.new(:path(~self),:trying<mode>))
     }
 }
 
