@@ -3,10 +3,10 @@ my class X::Numeric::DivideByZero { ... }
 my class X::NYI::BigInt { ... }
 
 my class Int { ... }
-my subset UInt of Int where * >= 0;
+my subset UInt of Int where {not .defined or $_ >= 0};
 
 my class Int does Real { # declared in BOOTSTRAP
-    # class Int is Cool {
+    # class Int is Cool
     #     has bigint $!value is box_target;
 
     multi method WHICH(Int:D:) {
@@ -19,7 +19,9 @@ my class Int does Real { # declared in BOOTSTRAP
         );
     }
     multi method new($value) {
-        nqp::box_i($value, self.WHAT);
+        # clone to ensure we return a new object for any cached
+        # numeric constants
+        $value.Int.clone;
     }
     multi method perl(Int:D:) {
         self.Str;
@@ -68,7 +70,7 @@ my class Int does Real { # declared in BOOTSTRAP
     }
     multi method base(Int:D: Int(Cool) $base, $digits?) {
         2 <= $base <= 36
-          ?? $digits
+          ?? $digits && ! nqp::istype($digits, Whatever)
             ?? $digits < 0
               ?? Failure.new(X::OutOfRange.new(
                    :what('digits argument to base'),:got($digits),:range<0..*>))
@@ -94,17 +96,18 @@ my class Int does Real { # declared in BOOTSTRAP
                     $more
                       ?? $mod
                         ?? take $more mod $mod
-                        !! fail X::Numeric::DivideByZero.new(
+                        !! Failure.new(X::Numeric::DivideByZero.new:
                              using => 'polymod', numerator => $more)
                       !! last;
                     $more = $more div $mod;
                 }
+                take $more if $more;
             }
             else {
                 for @mods -> $mod {
                     $mod
                       ?? take $more mod $mod
-                      !! fail X::Numeric::DivideByZero.new(
+                      !! Failure.new(X::Numeric::DivideByZero.new:
                            using => 'polymod', numerator => $more);
                     $more = $more div $mod;
                 }
@@ -257,7 +260,7 @@ multi sub infix:<*>(Int:D \a, Int:D \b) returns Int {
     nqp::mul_I(nqp::decont(a), nqp::decont(b), Int);
 }
 multi sub infix:<*>(int $a, int $b) returns int {
-    nqp::mul_i($a, $b)
+    nqp::mul_i($a, $b);
 }
 
 multi sub infix:<div>(Int:D \a, Int:D \b) {
@@ -281,14 +284,14 @@ multi sub infix:<%>(int $a, int $b) returns int {
 }
 
 multi sub infix:<**>(Int:D \a, Int:D \b) {
-    b >= 0 ?? nqp::pow_I(nqp::decont(a), nqp::decont(b), Num, Int)
-           !! 1 / nqp::pow_I(nqp::decont(a), nqp::decont(-b), Num, Int)
-    or a == 0 ?? 0 !! Failure.new(X::Numeric::Overflow.new)
+    my \power = nqp::pow_I(nqp::decont(a), nqp::decont(b >= 0 ?? b !! -b), Num, Int);
+    # when a**b is too big nqp::pow_I returns Inf
+    nqp::istype(power, Num) ?? Failure.new(X::Numeric::Overflow.new) !!
+        (b >= 0 ?? power !! 1 / power);
 }
 
 multi sub infix:<**>(int $a, int $b) returns int {
-    nqp::pow_i($a, $b)
-        or $a == 0 ?? 0 !! Failure.new(X::Numeric::Overflow.new)
+    nqp::pow_i($a, $b);
 }
 
 multi sub infix:<lcm>(Int:D \a, Int:D \b) returns Int {
@@ -358,50 +361,50 @@ multi sub infix:«>=»(int $a, int $b) {
 multi sub infix:<+|>(Int:D \a, Int:D \b) {
     nqp::bitor_I(nqp::decont(a), nqp::decont(b), Int)
 }
-multi sub infix:<+|>(int $a, int $b) {
-    nqp::bitor_i($a, $b)
-}
+#multi sub infix:<+|>(int $a, int $b) { RT#128655
+#    nqp::bitor_i($a, $b)
+#}
 
 multi sub infix:<+&>(Int:D \a, Int:D \b) {
     nqp::bitand_I(nqp::decont(a), nqp::decont(b), Int)
 }
-multi sub infix:<+&>(int $a, int $b) {
-    nqp::bitand_i($a, $b)
-}
+#multi sub infix:<+&>(int $a, int $b) { RT#128655
+#    nqp::bitand_i($a, $b)
+#}
 
 multi sub infix:<+^>(Int:D \a, Int:D \b) {
     nqp::bitxor_I(nqp::decont(a), nqp::decont(b), Int)
 }
-multi sub infix:<+^>(int $a, int $b) {
-    nqp::bitxor_i($a, $b);
-}
+#multi sub infix:<+^>(int $a, int $b) { RT#128655
+#    nqp::bitxor_i($a, $b);
+#}
 
 multi sub infix:«+<»(Int:D \a, Int:D \b) returns Int:D {
     nqp::isbig_I(nqp::decont(b))
       ?? Failure.new(X::NYI::BigInt.new(:op('+<'),:big(b)))
       !! nqp::bitshiftl_I(nqp::decont(a), nqp::unbox_i(b), Int)
 }
-multi sub infix:«+<»(int $a, int $b) {
-    nqp::bitshiftl_i($a, $b);
-}
+#multi sub infix:«+<»(int $a, int $b) { RT#128655
+#    nqp::bitshiftl_i($a, $b);
+#}
 
 multi sub infix:«+>»(Int:D \a, Int:D \b) returns Int:D {
     nqp::isbig_I(nqp::decont(b))
       ?? Failure.new(X::NYI::BigInt.new(:op('+>'),:big(b)))
       !! a < 0 && b > 31
-        ?? -1 # temp fix for #126942, remove if fixed otherwise
+        ?? -1 # temp fix for RT#126942, remove if fixed otherwise
         !! nqp::bitshiftr_I(nqp::decont(a), nqp::unbox_i(b), Int)
 }
-multi sub infix:«+>»(int $a, int $b) {
-    nqp::bitshiftr_i($a, $b)
-}
+#multi sub infix:«+>»(int $a, int $b) { RT#128655
+#    nqp::bitshiftr_i($a, $b)
+#}
 
 multi sub prefix:<+^>(Int:D \a) {
     nqp::bitneg_I(nqp::decont(a), Int);
 }
-multi sub prefix:<+^>(int $a) {
-    nqp::bitneg_i($a);
-}
+#multi sub prefix:<+^>(int $a) { RT#128655
+#    nqp::bitneg_i($a);
+#}
 
 proto sub chr($) is pure  {*}
 multi sub chr(Int:D  \x) returns Str:D { x.chr     }
