@@ -21,9 +21,7 @@ my role Iterable {
             has Iterator $!nested;
 
             method new(\source) {
-                my \iter = nqp::create(self);
-                nqp::bindattr(iter, self, '$!source', source);
-                iter
+                nqp::p6bindattrinvres(nqp::create(self),self,'$!source',source)
             }
 
             my constant NO_RESULT_YET = nqp::create(Mu);
@@ -49,44 +47,26 @@ my role Iterable {
                 );
                 $result
             }
-            method push-all($target) {
-                my $got;
-                my $nested;
+
+            method push-all($target --> IterationEnd) {
+                my $iter := $!nested || $!source;
                 nqp::until(
-                  nqp::eqaddr(($got := $!source.pull-one),IterationEnd),
-                  nqp::if(  # doesn't sink
-                    nqp::istype($got,Iterable) && nqp::not_i(nqp::iscont($got)),
-                    nqp::stmts(
-                      ($nested := $got.flat.iterator),
-                      nqp::until(  # doesn't sink
-                        nqp::eqaddr(($got := $nested.pull-one),IterationEnd),
-                        $target.push($got)
-                      )
-                    ),
-                    $target.push($got)
-                  )
+                       nqp::eqaddr((my $got := $iter.pull-one), IterationEnd)
+                    && nqp::eqaddr($iter, $!source),
+                    nqp::if(
+                        nqp::eqaddr($got, IterationEnd),
+                        nqp::stmts(
+                            ($!nested := Iterator),
+                            ($iter    := $!source),
+                        ),
+                        nqp::if(
+                               nqp::istype($got,Iterable)
+                            && nqp::not_i(nqp::iscont($got)),
+                            ($iter := $got.flat.iterator),
+                            $target.push($got),
+                        ),
+                    )
                 );
-                IterationEnd
-            }
-            method count-only() {
-                my int $found;
-                my $got;
-                my $nested;
-                nqp::until(
-                  nqp::eqaddr(($got := $!source.pull-one),IterationEnd),
-                  nqp::if(
-                    nqp::istype($got,Iterable) && nqp::not_i(nqp::iscont($got)),
-                    nqp::stmts(
-                      ($nested := $got.flat.iterator),
-                      nqp::until(
-                        nqp::eqaddr($nested.pull-one,IterationEnd),
-                        ($found = nqp::add_i($found,1))
-                      )
-                    ),
-                    ($found = nqp::add_i($found,1))
-                  )
-                );
-                nqp::p6box_i($found)
             }
         }.new(self.iterator))
     }
@@ -102,9 +82,9 @@ my role Iterable {
             has $!iterator;
 
             method new(\iterable) {
-                my \iter = nqp::create(self);
-                nqp::bindattr(iter, self, '$!iterable', iterable);
-                iter
+                nqp::p6bindattrinvres(
+                  nqp::create(self),self,'$!iterable',iterable
+                )
             }
 
             method pull-one() is raw {
