@@ -1,9 +1,14 @@
 my class X::Hash::Store::OddNumber { ... }
 
 my class Map does Iterable does Associative { # declared in BOOTSTRAP
-    # my class Map is Iterable is Cool {
+    # my class Map is Iterable is Cool
     #   has Mu $!storage;
 
+    multi method WHICH(Map:D:) {
+        self.^name
+          ~ '|'
+          ~ self.keys.sort.map( { $_.WHICH ~ '(' ~ self.AT-KEY($_) ~ ')' } )
+    }
     method new(*@args) {
         @args
           ?? nqp::create(self).STORE(@args)
@@ -96,7 +101,7 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
                   IterationEnd
                 )
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 nqp::while(
                   $!iter,
                   nqp::stmts(  # doesn't sink
@@ -104,8 +109,7 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
                      $target.push(
                        Pair.new(nqp::iterkey_s($!iter), nqp::iterval($!iter)))
                   )
-                );
-                IterationEnd
+                )
             }
         }.new(self))
     }
@@ -116,12 +120,11 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
                   ?? nqp::iterkey_s(nqp::shift($!iter))
                   !! IterationEnd
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 nqp::while(
                   $!iter,
                   $target.push(nqp::iterkey_s(nqp::shift($!iter)))
-                );
-                IterationEnd
+                )
             }
         }.new(self))
     }
@@ -146,33 +149,19 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
                   )
                 )
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 nqp::while(  # doesn't sink
                   $!iter,
                   nqp::stmts(
                     $target.push(nqp::iterkey_s(nqp::shift($!iter))),
                     $target.push(nqp::iterval($!iter))
                   )
-                );
-                IterationEnd
+                )
             }
         }.new(self))
     }
     multi method values(Map:D:) {
-        Seq.new(class :: does Rakudo::Internals::MappyIterator {
-            method pull-one() is raw {
-                $!iter
-                  ?? nqp::iterval(nqp::shift($!iter))
-                  !! IterationEnd
-            }
-            method push-all($target) {
-                nqp::while(  # doesn't sink
-                  $!iter,
-                  $target.push(nqp::iterval(nqp::shift($!iter)))
-                );
-                IterationEnd
-            }
-        }.new(self))
+        Seq.new(Rakudo::Internals::MappyIterator-values.new(self))
     }
     multi method antipairs(Map:D:) {
         Seq.new(class :: does Rakudo::Internals::MappyIterator {
@@ -186,7 +175,7 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
                   IterationEnd
                 );
             }
-            method push-all($target) {
+            method push-all($target --> IterationEnd) {
                 nqp::while(
                   $!iter,
                   nqp::stmts(  # doesn't sink
@@ -194,8 +183,7 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
                     $target.push(
                       Pair.new( nqp::iterval($!iter), nqp::iterkey_s($!iter) ))
                   )
-                );
-                IterationEnd
+                )
             }
         }.new(self))
     }
@@ -214,48 +202,73 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
           !! Nil
     }
 
-    method STORE(\to_store) {
-        method !STORE_LIST(\x --> Nil) {
-            self.STORE_AT_KEY(.key,.value) for x.list;
-        }
+    method !STORE_MAP(\map --> Nil) {
+        nqp::if(
+          nqp::defined(my $other := nqp::getattr(map,Map,'$!storage')),
+          nqp::stmts(
+            (my $iter := nqp::iterator($other)),
+            nqp::while(
+              $iter,
+              nqp::stmts(
+                nqp::shift($iter),
+                self.STORE_AT_KEY(
+                  nqp::iterkey_s($iter),nqp::iterval($iter)
+                )
+              )
+            )
+          )
+        )
+    }
 
+    method STORE(\to_store) {
         $!storage := nqp::hash();
         my $iter  := to_store.iterator;
         my Mu $x;
         my Mu $y;
 
-        nqp::istype($x,Pair)
-          ?? self.STORE_AT_KEY($x.key, $x.value)
-          !! nqp::istype($x, Map) && !nqp::iscont($x)
-            ?? self!STORE_LIST($x)
-            !! (($y := $iter.pull-one) =:= IterationEnd)
-              ?? nqp::istype($x,Failure)
-                ?? $x.throw
-                !! X::Hash::Store::OddNumber.new.throw
-              !! self.STORE_AT_KEY($x,$y)
-                until ($x := $iter.pull-one) =:= IterationEnd;
+        nqp::until(
+          nqp::eqaddr(($x := $iter.pull-one),IterationEnd),
+          nqp::if(
+            nqp::istype($x,Pair),
+            self.STORE_AT_KEY(
+              nqp::getattr(nqp::decont($x),Pair,'$!key'),
+              nqp::getattr(nqp::decont($x),Pair,'$!value')
+            ),
+            nqp::if(
+              (nqp::istype($x,Map) && nqp::not_i(nqp::iscont($x))),
+              self!STORE_MAP($x),
+              nqp::if(
+                nqp::eqaddr(($y := $iter.pull-one),IterationEnd),
+                nqp::if(
+                  nqp::istype($x,Failure),
+                  $x.throw,
+                  X::Hash::Store::OddNumber.new(
+                    found => nqp::add_i(nqp::mul_i(nqp::elems($!storage),2),1),
+                    last  => $x
+                  ).throw
+                ),
+                self.STORE_AT_KEY($x,$y)
+              )
+            )
+          )
+        );
+        
         self
     }
 
     proto method STORE_AT_KEY(|) { * }
     multi method STORE_AT_KEY(Str:D \key, Mu \value --> Nil) {
-        $!storage := nqp::hash unless nqp::defined($!storage);
         nqp::bindkey($!storage, nqp::unbox_s(key), value)
     }
     multi method STORE_AT_KEY(\key, Mu \value --> Nil) {
-        $!storage := nqp::hash unless nqp::defined($!storage);
         nqp::bindkey($!storage, nqp::unbox_s(key.Str), value)
     }
 
     method Capture(Map:D:) {
-        if nqp::defined($!storage) {
-            my $cap := nqp::create(Capture);
-            nqp::bindattr($cap,Capture,'$!hash',$!storage);
-            $cap
-        }
-        else {
-            nqp::create(Capture)
-        }
+        nqp::defined($!storage)
+          ?? nqp::p6bindattrinvres(
+               nqp::create(Capture),Capture,'%!hash',$!storage)
+          !! nqp::create(Capture)
     }
 
     method FLATTENABLE_LIST() { nqp::list() }
@@ -272,26 +285,37 @@ my class Map does Iterable does Associative { # declared in BOOTSTRAP
     }
 
     method hash() { self }
-    method clone(Map:D: *%) is raw {
-        my \clone = nqp::create(self);
-        nqp::bindattr(clone, Map, '$!storage', nqp::clone($!storage));
-        clone
-    }
+    method clone(Map:D:) is raw { self }
 }
 
 multi sub infix:<eqv>(Map:D \a, Map:D \b) {
-    if a =:= b {
-        True
-    }
-    elsif a.WHAT =:= b.WHAT && a.elems == b.elems {
-        return False
-          unless b.EXISTS-KEY($_) && a.AT-KEY($_) eqv b.AT-KEY($_)
-          for a.keys;
-        True
-    }
-    else {
-        False
-    }
+    nqp::p6bool(
+      nqp::unless(
+        nqp::eqaddr(a,b),
+        nqp::if(
+          nqp::eqaddr(a.WHAT,b.WHAT),
+          nqp::if(
+            nqp::iseq_i((my int $elems = a.elems),b.elems),
+            nqp::unless(
+              nqp::iseq_i($elems,0),
+              nqp::stmts(
+                (my $amap := nqp::getattr(nqp::decont(a),Map,'$!storage')),
+                (my $bmap := nqp::getattr(nqp::decont(b),Map,'$!storage')),
+                (my $iter := nqp::iterator($amap)),
+                nqp::while(
+                  $iter
+                  && nqp::existskey($bmap,
+                    my str $key = nqp::iterkey_s(nqp::shift($iter)))
+                  && nqp::atkey($amap,$key) eqv nqp::atkey($bmap,$key),
+                  ($elems = nqp::sub_i($elems,1))
+                ),
+                nqp::iseq_i($elems,0)    # checked all, so ok
+              )
+            )
+          )
+        )
+      )
+    )
 }
 
 # vim: ft=perl6 expandtab sw=4
