@@ -1,6 +1,7 @@
 my class MixHash does Mixy {
 
 #--- interface methods
+    multi method WHICH(MixHash:D:) { self.Mu::WHICH }
     multi method AT-KEY(MixHash:D: \k) is raw {
         Proxy.new(
           FETCH => {
@@ -28,18 +29,14 @@ my class MixHash does Mixy {
 
 #--- coercion methods
     method Mix(:$view) {
-        if $view {
-            my \mix = nqp::create(Mix);
-            nqp::bindattr(mix,Mix,'%!elems',%!elems);
-            mix
-        }
-        else {
-            Mix.new-from-pairs(%!elems.values)
-        }
+        nqp::p6bindattrinvres(
+          nqp::create(Mix),Mix,'%!elems',
+          $view ?? %!elems !! %!elems.clone
+        )
     }
     method MixHash { self }
-    method Bag     { Bag.new-from-pairs(%!elems.values) }
-    method BagHash { BagHash.new-from-pairs(%!elems.values) }
+    method Bag     {     Bag.new-from-pairs(%!elems.values.grep(*.value > 0).map({.key => .value.Int})) }
+    method BagHash { BagHash.new-from-pairs(%!elems.values.grep(*.value > 0).map({.key => .value.Int})) }
 }
 
 # vim: ft=perl6 expandtab sw=4
