@@ -3,6 +3,9 @@ my class X::Method::NotFound         { ... }
 my class X::Method::InvalidQualifier { ... }
 my class X::Attribute::Required      { ... }
 
+# We use a sentinel value to mark the end of an iteration.
+my constant IterationEnd = nqp::create(Mu);
+
 my class Mu { # declared in BOOTSTRAP
 
     method self { self }
@@ -23,7 +26,13 @@ my class Mu { # declared in BOOTSTRAP
 
     proto method WHICH(|) {*}
     multi method WHICH(Mu:U:) {
-        nqp::box_s(nqp::unbox_s(self.^name), ObjAt);
+        nqp::box_s(
+            nqp::concat(
+                nqp::concat(nqp::unbox_s(self.^name), '|U'),
+                nqp::objectid(self)
+            ),
+            ObjAt
+        )
     }
     multi method WHICH(Mu:D:) {
         nqp::box_s(
@@ -37,6 +46,9 @@ my class Mu { # declared in BOOTSTRAP
 
     proto method split(|) { * }
 
+    method emit {
+        emit self;
+    }
     method take {
         take self;
     }
@@ -269,7 +281,8 @@ my class Mu { # declared in BOOTSTRAP
                                     nqp::atpos($task,2)
                                   ),
                                   X::Attribute::Required.new(
-                                    name => nqp::atpos($task,2)
+                                    name => nqp::atpos($task,2),
+                                    why  => nqp::atpos($task,3)
                                   ).throw
                                 ),
 
@@ -431,12 +444,17 @@ my class Mu { # declared in BOOTSTRAP
     multi method Str(Mu:U \v:) {
         my $name = (defined($*VAR_NAME) ?? $*VAR_NAME !! try v.VAR.?name) // '';
         $name   ~= ' ' if $name ne '';
-        warn "Use of uninitialized value {$name}of type {self.^name} in string context\n" ~
-             "Any of .^name, .perl, .gist, or .say can stringify undefined things, if needed.";
+        warn "Use of uninitialized value {$name}of type {self.^name} in string"
+                ~ " context.\nMethods .^name, .perl, .gist, or .say can be"
+                ~ " used to stringify it to something meaningful.";
         ''
     }
     multi method Str(Mu:D:) {
-        self.^name ~ '<' ~ nqp::tostr_I(nqp::objectid(self)) ~ '>'
+        nqp::if(
+          nqp::eqaddr(self,IterationEnd),
+          "IterationEnd",
+          self.^name ~ '<' ~ nqp::tostr_I(nqp::objectid(self)) ~ '>'
+        )
     }
 
     proto method Stringy(|) { * }
@@ -455,7 +473,7 @@ my class Mu { # declared in BOOTSTRAP
     method note() { note(self) }
 
     method gistseen(Mu:D \SELF: $id, $gist, *%named) {
-        if nqp::isnull(nqp::getlexdyn('$*gistseen')) == 0 {
+        if nqp::not_i(nqp::isnull(nqp::getlexdyn('$*gistseen'))) {
             my \sems := $*gistseen;
             my str $WHICH = nqp::unbox_s(self.WHICH);
             if nqp::existskey(sems,$WHICH) && nqp::atkey(sems,$WHICH) {
@@ -483,7 +501,7 @@ my class Mu { # declared in BOOTSTRAP
     multi method gist(Mu:D:) { self.perl }
 
     method perlseen(Mu:D \SELF: $id, $perl, *%named) {
-        if nqp::isnull(nqp::getlexdyn('$*perlseen')) == 0 {
+        if nqp::not_i(nqp::isnull(nqp::getlexdyn('$*perlseen'))) {
             my \sems := $*perlseen;
             my str $WHICH = nqp::unbox_s(self.WHICH);
             if nqp::existskey(sems,$WHICH) && nqp::atkey(sems,$WHICH) {
@@ -509,14 +527,18 @@ my class Mu { # declared in BOOTSTRAP
     proto method perl(|) { * }
     multi method perl(Mu:U:) { self.^name }
     multi method perl(Mu:D:) {
-        self.perlseen(self.^name, {
-            my @attrs;
-            for self.^attributes().flat.grep: { .has_accessor } -> $attr {
-                my $name := substr($attr.Str,2);
-                @attrs.push: $name ~ ' => ' ~ $attr.get_value(self).perl
-            }
-            self.^name ~ '.new' ~ ('(' ~ @attrs.join(', ') ~ ')' if @attrs)
-        })
+        nqp::if(
+          nqp::eqaddr(self,IterationEnd),
+          "IterationEnd",
+          self.perlseen(self.^name, {
+              my @attrs;
+              for self.^attributes().flat.grep: { .has_accessor } -> $attr {
+                  my $name := substr($attr.Str,2);
+                  @attrs.push: $name ~ ' => ' ~ $attr.get_value(self).perl
+              }
+              self.^name ~ '.new' ~ ('(' ~ @attrs.join(', ') ~ ')' if @attrs)
+          })
+        )
     }
 
     proto method DUMP(|) { * }
@@ -601,7 +623,7 @@ my class Mu { # declared in BOOTSTRAP
     }
 
     method clone(*%twiddles) {
-        my $cloned := nqp::clone(nqp::decont(self));
+        my $cloned := nqp::clone(self);
         if %twiddles.elems {
             for self.^attributes.flat -> $attr {
                 my $name    := $attr.name;
@@ -609,7 +631,8 @@ my class Mu { # declared in BOOTSTRAP
 
                 nqp::bindattr($cloned, $package, $name,
                   nqp::clone(nqp::getattr($cloned, $package, $name).VAR)
-                ) unless nqp::objprimspec($attr.type);
+                ) if nqp::attrinited(self, $package, $name)
+                    and nqp::not_i(nqp::objprimspec($attr.type));
 
                 my $acc_name := substr($name,2);
                 nqp::getattr($cloned, $package, $name) =
@@ -622,10 +645,12 @@ my class Mu { # declared in BOOTSTRAP
                 unless nqp::objprimspec($attr.type) {
                     my $name     := $attr.name;
                     my $package  := $attr.package;
-                    my $attr_val := nqp::getattr($cloned, $package, $name);
-                    nqp::bindattr($cloned,
-                      $package, $name, nqp::clone($attr_val.VAR))
-                        if nqp::iscont($attr_val);
+                    if nqp::attrinited(self, $package, $name) {
+                        my $attr_val := nqp::getattr($cloned, $package, $name);
+                        nqp::bindattr($cloned,
+                          $package, $name, nqp::clone($attr_val.VAR))
+                            if nqp::iscont($attr_val);
+                    }
                 }
             }
         }
@@ -642,7 +667,7 @@ my class Mu { # declared in BOOTSTRAP
             }
         }
         my $capture := nqp::create(Capture);
-        nqp::bindattr($capture,Capture,'$!hash',$attrs) if nqp::elems($attrs);
+        nqp::bindattr($capture,Capture,'%!hash',$attrs) if nqp::elems($attrs);
         $capture
     }
 
@@ -663,30 +688,26 @@ my class Mu { # declared in BOOTSTRAP
         self.^find_method_qualified($type, $name)(SELF, |c)
     }
 
-    method dispatch:<!>(Mu \SELF: $name, Mu $type, |c) is raw {
-        my $meth := $type.^find_private_method($name);
+    method dispatch:<!>(Mu \SELF: \name, Mu \type, |c) is raw {
+        my $meth := type.^find_private_method(name);
         $meth ??
             $meth(SELF, |c) !!
             X::Method::NotFound.new(
               invocant => SELF,
-              method   => '!' ~ $name,
-              typename => $type.^name,
+              method   => '!' ~ name,
+              typename => type.^name,
               :private,
             ).throw;
     }
 
-    method dispatch:<.^>(Mu \SELF: $name, |c) is raw {
-        self.HOW."$name"(SELF, |c)
-    }
-
-    method dispatch:<.=>(\mutate: $name, |c) is raw {
+    method dispatch:<.=>(\mutate: \name, |c) is raw {
         $/ := nqp::getlexcaller('$/');
-        mutate = mutate."$name"(|c)
+        mutate = mutate."{name}"(|c)
     }
 
-    method dispatch:<.?>(Mu \SELF: $name, |c) is raw {
-        nqp::can(SELF, $name) ??
-            SELF."$name"(|c) !!
+    method dispatch:<.?>(Mu \SELF: \name, |c) is raw {
+        nqp::can(SELF, name) ??
+            SELF."{name}"(|c) !!
             Nil
     }
 
@@ -702,31 +723,29 @@ my class Mu { # declared in BOOTSTRAP
         @result
     }
 
-    method dispatch:<.*>(Mu \SELF: $name, |c) {
+    method dispatch:<.*>(Mu \SELF: \name, |c) {
         my @mro = SELF.^mro;
         my int $mro_count = @mro.elems;
         my $results := nqp::list;
         my int $i = -1;
         while nqp::islt_i(++$i,$mro_count) {
             my $obj = @mro[$i];
-            my $meth = ($obj.^method_table){$name};
-            $meth = ($obj.^submethod_table){$name} if !$meth && $i == 0;
+            my $meth = ($obj.^method_table){name};
+            $meth = ($obj.^submethod_table){name} if !$meth && $i == 0;
             nqp::push($results,$meth(SELF, |c))    if $meth;
         }
-        my $list := nqp::create(List);
-        nqp::bindattr($list, List, '$!reified', $results);
-        $list
+        nqp::p6bindattrinvres(nqp::create(List),List,'$!reified',$results)
     }
 
-    method dispatch:<hyper>(Mu \SELF: $name, |c) {
-        my $listcan = List.can($name);
+    method dispatch:<hyper>(Mu \SELF: \name, |c) {
+        my $listcan = List.can(name);
         $listcan && $listcan[0].?nodal
           ?? c
-            ?? HYPER( sub (\obj) is nodal { obj."$name"(|c) }, SELF )
-            !! HYPER( sub (\obj) is nodal { obj."$name"() }, SELF )
+            ?? HYPER( sub (\obj) is nodal { obj."{name}"(|c) }, SELF )
+            !! HYPER( sub (\obj) is nodal { obj."{name}"() }, SELF )
           !! c
-            ?? HYPER( -> \obj { obj."$name"(|c) }, SELF )
-            !! HYPER( -> \obj { obj."$name"() }, SELF )
+            ?? HYPER( -> \obj { obj."{name}"(|c) }, SELF )
+            !! HYPER( -> \obj { obj."{name}"() }, SELF )
     }
 
     method WALK(:$name!, :$canonical, :$ascendant, :$descendant, :$preorder, :$breadth,
@@ -819,26 +838,43 @@ multi sub infix:<=:=>(Mu \a, Mu \b) {
 
 proto sub infix:<eqv>(Any $?, Any $?) is pure { * }
 multi sub infix:<eqv>($?)            { Bool::True }
-multi sub infix:<eqv>(Any $a, Any $b) {
-    # Last ditch snapshot semantics.  We shouldn't come here too often, so
-    # please do not change this to be faster but wronger.  (Instead, add
-    # specialized multis for datatypes that can be tested piecemeal.)
-    $a.WHAT === $b.WHAT and $a.perl eq $b.perl;
+
+# Last ditch snapshot semantics.  We shouldn't come here too often, so
+# please do not change this to be faster but wronger.  (Instead, add
+# specialized multis for datatypes that can be tested piecemeal.)
+multi sub infix:<eqv>(Any:U \a, Any:U \b) {
+    nqp::p6bool(nqp::eqaddr(nqp::decont(a),nqp::decont(b)))
+}
+multi sub infix:<eqv>(Any:D \a, Any:U \b) { False }
+multi sub infix:<eqv>(Any:U \a, Any:D \b) { False }
+multi sub infix:<eqv>(Any:D \a, Any:D \b) {
+    nqp::p6bool(
+      nqp::eqaddr(a,b)
+        || (nqp::eqaddr(a.WHAT,b.WHAT) && nqp::iseq_s(a.perl,b.perl))
+    )
 }
 
 multi sub infix:<eqv>(@a, @b) {
-    if @a =:= @b {
-        True
-    }
-    elsif @a.WHAT =:= @b.WHAT && (my int $n = @a.elems) == @b.elems {
-        my int $i = -1;
-        return False unless @a.AT-POS($i) eqv @b.AT-POS($i)
-          while nqp::islt_i(++$i,$n);
-        True
-    }
-    else {
-        False
-    }
+    nqp::p6bool(
+      nqp::unless(
+        nqp::eqaddr(@a,@b),                                    # identity
+        nqp::if(
+          nqp::eqaddr(@a.WHAT,@b.WHAT),                        # same type
+          nqp::if(
+            nqp::iseq_i((my int $elems = @a.elems),@b.elems),  # same # elems
+            nqp::stmts(
+              (my int $i = -1),
+              nqp::while(
+                nqp::islt_i(($i = nqp::add_i($i,1)),$elems)    # not exhausted
+                  && @a.AT-POS($i) eqv @b.AT-POS($i),          # still same
+                nqp::null
+              ),
+              nqp::iseq_i($i,$elems)                      # exhausted = success!
+            )
+          )
+        )
+      )
+    )
 }
 
 sub DUMP(|args (*@args, :$indent-step = 4, :%ctx?)) {
