@@ -1,5 +1,5 @@
 my class ObjAt { # declared in BOOTSTRAP
-    # class ObjAt is Any {
+    # class ObjAt is Any
     #     has str $!value;
     method new(str $s) {
         nqp::box_s($s, self.WHAT)
@@ -20,6 +20,9 @@ my class ObjAt { # declared in BOOTSTRAP
     multi method gist(ObjAt:D:) {
         nqp::p6box_s(nqp::unbox_s(self));
     }
+    multi method perl(ObjAt:D:) {
+        self.^name ~ ".new(" ~ nqp::p6box_s(nqp::unbox_s(self)).perl ~ ")"
+    }
 }
 
 # vim: ft=perl6 expandtab sw=4
