@@ -2,16 +2,22 @@ my class Pair does Associative {
     has $.key is default(Nil);
     has $.value is rw is default(Nil);
 
-    multi method new(Mu \key, Mu \value) {
-        my \p := nqp::create(self);
-        nqp::bindattr(p, Pair, '$!key', nqp::decont(key));
-        nqp::bindattr(p, Pair, '$!value', value);
+    multi method new(Pair: Cool:D \key, Mu \value) {
+        my \p := nqp::p6bindattrinvres(
+          nqp::create(self),Pair,'$!key',nqp::decont(key));
+        nqp::bindattr(p,Pair,'$!value',value);
         p
     }
-    multi method new(Mu :$key, Mu :$value) {
-        my \p := nqp::create(self);
-        nqp::bindattr(p, Pair, '$!key', $key);
-        nqp::bindattr(p, Pair, '$!value', $value);
+    multi method new(Pair: Mu \key, Mu \value) {
+        my \p := nqp::p6bindattrinvres(
+          nqp::create(self),Pair,'$!key',nqp::decont(key));
+        nqp::bindattr(p,Pair,'$!value',value);
+        p
+    }
+    multi method new(Pair: Mu :$key, Mu :$value) {
+        my \p := nqp::p6bindattrinvres(
+          nqp::create(self),Pair,'$!key',$key);
+        nqp::bindattr(p,Pair,'$!value',$value);
         p
     }
 
@@ -78,7 +84,12 @@ my class Pair does Associative {
 }
 
 multi sub infix:<eqv>(Pair:D \a, Pair:D \b) {
-    a =:= b || (a.WHAT =:= b.WHAT && a.key eqv b.key && a.value eqv b.value)
+    nqp::p6bool(
+      nqp::eqaddr(a,b)
+        || (nqp::eqaddr(a.WHAT,b.WHAT)
+             && a.key   eqv b.key
+             && a.value eqv b.value)
+    )
 }
 
 multi sub infix:<cmp>(Pair:D \a, Pair:D \b) {
