@@ -1,11 +1,12 @@
 class Perl does Systemic {
     has Compiler $.compiler;
 
-    submethod BUILD (
+    submethod BUILD(
       :$!name      = 'Perl 6',
       :$!auth      = "The Perl Foundation",
-      :$!version   = Version.new("6b"),
-      :$!compiler  = Compiler.new,
+      :$!version   = Version.new(nqp::getcomp('perl6').language_version()),
+      :$!compiler  = Compiler.new
+      --> Nil
     ) { }
 
     method VMnames { <moar jvm > }
@@ -24,7 +25,7 @@ class Perl does Systemic {
     method KERNELnames { <darwin linux freebsd openbsd netbsd dragonfly win32> }
 }
 
-multi sub INITIALIZE_DYNAMIC('$*PERL') {
+Rakudo::Internals.REGISTER-DYNAMIC: '$*PERL', {
     PROCESS::<$PERL> := Perl.new;
 }
 multi sub postcircumfix:<{ }> (Perl $d, "compiler" )   {
