@@ -5,6 +5,7 @@ my class Proc {
     has $.exitcode = -1;  # distinguish uninitialized from 0 status
     has $.pid;
     has $.signal;
+    has @.command;
 
     has $!in_fh;
     has $!out_fh;
@@ -12,8 +13,9 @@ my class Proc {
     has int $!flags;
 
     submethod BUILD(:$in = '-', :$out = '-', :$err = '-', :$exitcode,
-                    Bool :$bin, Bool :$chomp = True, Bool :$merge,
+                    Bool :$bin, Bool :$chomp = True, Bool :$merge, :$command,
                     Str:D :$enc = 'utf8', Str:D :$nl = "\n", :$signal --> Nil) {
+        @!command = |$command if $command;
         if nqp::istype($in, IO::Handle) && $in.DEFINITE {
             $!in_fh := nqp::getattr(nqp::decont($in), IO::Handle, '$!PIO');
             $!flags += nqp::const::PIPE_INHERIT_IN;
@@ -94,6 +96,7 @@ my class Proc {
     }
 
     method spawn(*@args ($, *@), :$cwd = $*CWD, :$env) {
+        @!command = @args;
         my %env := $env ?? $env.hash !! %*ENV;
         self.status(nqp::p6box_i(nqp::spawn(
             CLONE-LIST-DECONTAINERIZED(@args),
@@ -106,6 +109,7 @@ my class Proc {
     }
 
     method shell($cmd, :$cwd = $*CWD, :$env) {
+        @!command = $cmd;
         my %env := $env ?? $env.hash !! %*ENV;
         self.status(nqp::p6box_i(nqp::shell(
             nqp::unbox_s($cmd),
