@@ -15,16 +15,26 @@ my class X::Promise::Vowed is Exception {
     has $.promise;
     method message() { "Access denied to keep/break this Promise; already vowed" }
 }
+my role X::Promise::Broken {
+    has $.result-backtrace;
+    multi method gist(::?CLASS:D:) {
+        "Tried to get the result of a broken Promise\n" ~
+            ((try $!result-backtrace ~ "\n") // '') ~
+            "Original exception:\n" ~
+            callsame().indent(4)
+    }
+}
 my class Promise {
     has $.scheduler;
     has $.status;
-    has $!result;
+    has $!result is default(Nil);
     has int $!vow_taken;
     has $!lock;
     has $!cond;
     has @!thens;
+    has Mu $!dynamic_context;
 
-    submethod BUILD(:$!scheduler = $*SCHEDULER) {
+    submethod BUILD(:$!scheduler = $*SCHEDULER --> Nil) {
         $!lock            := nqp::create(Lock);
         $!cond            := $!lock.condition();
         $!status           = Planned;
@@ -37,7 +47,7 @@ my class Promise {
     trusts Vow;
     my class Vow {
         has $.promise;
-        method keep(\result) {
+        method keep(Mu \result) {
             $!promise!Promise::keep(result)
         }
         method break(\exception) {
@@ -61,11 +71,11 @@ my class Promise {
     multi method keep(Promise:D:) {
         self.vow.keep(True)
     }
-    multi method keep(Promise:D: \result) {
+    multi method keep(Promise:D: Mu \result) {
         self.vow.keep(result)
     }
 
-    method !keep(\result) {
+    method !keep(Mu \result) {
         $!lock.protect({
             $!result := result;
             $!status = Kept;
@@ -104,8 +114,9 @@ my class Promise {
     method result(Promise:D:) {
         # One important missing optimization here is that if the promise is
         # not yet started, then the work can be done immediately by the
-        # thing that is blocking on it.
-        if $!status == Planned {
+        # thing that is blocking on it. (Note the while loop is there to cope
+        # with spurious wake-ups).
+        while $!status == Planned {
             $!lock.protect({
                 # Re-check planned to avoid data race.
                 $!cond.wait() if $!status == Planned;
@@ -115,12 +126,12 @@ my class Promise {
             $!result
         }
         elsif $!status == Broken {
-            $!result.throw
+            ($!result but X::Promise::Broken(Backtrace.new)).rethrow
         }
     }
 
     multi method Bool(Promise:D:) {
-        so $!status == any(Broken, Kept)
+        so $!status == Broken || $!status == Kept
     }
 
     method cause(Promise:D:) {
@@ -137,7 +148,7 @@ my class Promise {
 
     method then(Promise:D: &code) {
         nqp::lock($!lock);
-        if $!status == Broken | Kept {
+        if $!status == Broken || $!status == Kept {
             # Already have the result, start immediately.
             nqp::unlock($!lock);
             Promise.start( { code(self) }, :$!scheduler);
@@ -157,10 +168,11 @@ my class Promise {
     }
 
     method start(Promise:U: &code, :&catch, :$scheduler = $*SCHEDULER, |c) {
-        my $p   = Promise.new(:$scheduler);
+        my $p := Promise.new(:$scheduler);
+        nqp::bindattr($p, Promise, '$!dynamic_context', nqp::ctx());
         my $vow = $p.vow;
         $scheduler.cue(
-            { $vow.keep(code(|c)) },
+            { my $*PROMISE := $p; $vow.keep(code(|c)) },
             :catch(-> $ex { catch($ex) if &catch; $vow.break($ex); }) );
         $p
     }
@@ -171,30 +183,31 @@ my class Promise {
         $scheduler.cue({ $vow.keep(True) }, :in($seconds));
         $p
     }
+    method at(Promise:U: $at, :$scheduler = $*SCHEDULER) {
+        self.in( $at - now, :$scheduler )
+    }
 
     method anyof(Promise:U: *@p) { self!until_n_kept(@p,   1, 'anyof') }
     method allof(Promise:U: *@p) { self!until_n_kept(@p, +@p, 'allof') }
 
     method !until_n_kept(@promises, Int $N, Str $combinator) {
+        my $p = Promise.new;
+        unless @promises {
+            $p.keep;
+            return $p
+        }
+
         X::Promise::Combinator.new(:$combinator).throw
-          if Rakudo::Internals.NOT_ALL_DEFINED_TYPE(@promises, Promise);
+          unless Rakudo::Internals.ALL_DEFINED_TYPE(@promises, Promise);
 
         my int $n  = $N;
         my int $c  = $n;
         my $lock  := nqp::create(Lock);
-        my $p      = Promise.new;
         my $vow    = $p.vow;
         for @promises -> $cand {
             $cand.then({
-                if .status == Kept {
-                    if $lock.protect({ $c = $c - 1 }) == 0 {
-                        $vow.keep(True)
-                    }
-                }
-                else {
-                    if $lock.protect({ my int $o = $c; $c = $c - ($n + 1); $o }) > 0 {
-                        $vow.break(.cause)
-                    }
+                if $lock.protect({ $c = $c - 1 }) == 0 {
+                    $vow.keep(True)
                 }
             })
         }
@@ -202,26 +215,24 @@ my class Promise {
     }
 
     method Supply(Promise:D:) {
-        my $s = Supply.new;
-        self.then({
-            if self.status == Kept {
-                $s.emit(self.result);
-                $s.done();
-            }
-            else {
-                $s.quit(self.cause);
-            }
-        });
-        $s
+        Supply.on-demand: -> $s {
+            self.then({
+                if self.status == Kept {
+                    $s.emit(self.result);
+                    $s.done();
+                }
+                else {
+                    $s.quit(self.cause);
+                }
+            });
+        }
     }
-
-    # experimental
-    method Str(Promise:D:)     { self.result.Str     }
-    method Numeric(Promise:D:) { self.result.Numeric }
 }
 
-multi sub infix:<eqv>(Promise:D $a, Promise:D $b) {
-    infix:<eqv>($a.result, $b.result);
+multi sub infix:<eqv>(Promise:D \a, Promise:D \b) {
+    nqp::p6bool(
+      nqp::eqaddr(a,b) || a.result eqv b.result
+    )
 }
 
 # vim: ft=perl6 expandtab sw=4
