@@ -104,26 +104,8 @@ my class Range is Cool does Iterable does Positional {
         if $!is-int
           && !nqp::isbig_I(nqp::decont($!min))
           && !nqp::isbig_I(nqp::decont($!max)) {
-            class :: does Iterator {
-                has int $!i;
-                has int $!n;
-
-                method !SET-SELF(\i,\n) { $!i = i - 1; $!n = n; self }
-                method new(\i,\n)   { nqp::create(self)!SET-SELF(i,n) }
-
-                method pull-one() {
-                    ( $!i = $!i + 1 ) <= $!n ?? $!i !! IterationEnd
-                }
-                method push-all($target) {
-                    my int $i = $!i;
-                    my int $n = $!n;
-                    $target.push(nqp::p6box_i($i)) while ($i = $i + 1) <= $n;
-                    $!i = $i;
-                    IterationEnd
-                }
-                method count-only() { nqp::p6box_i($!n - $!i) }
-                method sink-all()   { $!i = $!n; IterationEnd }
-            }.new($!min + $!excludes-min, $!max - $!excludes-max)
+            Rakudo::Internals.IntRangeIterator(
+              $!min + $!excludes-min, $!max - $!excludes-max)
         }
 
         # doesn't make much sense, but there you go
@@ -170,15 +152,15 @@ my class Range is Cool does Iterable does Positional {
                              ?? nqp::chr($!i)
                              !! IterationEnd
                        }
-                       method push-all($target) {
+                       method push-all($target --> IterationEnd) {
                            my int $i = $!i;
                            my int $n = $!n;
                            $target.push(nqp::chr($i)) while ($i = $i + 1) <= $n;
                            $!i = $i;
-                           IterationEnd
                        }
                        method count-only() { nqp::p6box_i($!n - $!i) }
-                       method sink-all()   { $!i = $!n; IterationEnd }
+                       method bool-only() { nqp::p6bool(nqp::isgt_i($!n,$!i)) }
+                       method sink-all(--> IterationEnd) { $!i = $!n }
                    }.new($min, $!excludes-max ?? $!max.pred !! $!max)
                 !! SEQUENCE($min,$!max,:exclude_end($!excludes-max)).iterator
         }
@@ -210,7 +192,7 @@ my class Range is Cool does Iterable does Positional {
                         IterationEnd
                     }
                 }
-                method push-all($target) {
+                method push-all($target --> IterationEnd) {
                     my Mu $i = $!i;
                     my Mu $e = $!e;
                     if $!exclude {
@@ -225,30 +207,8 @@ my class Range is Cool does Iterable does Positional {
                             $i = $i.succ;
                         }
                     }
-                    IterationEnd
-                }
-                method count-only {
-                    my Mu $i = $!i;
-                    my Mu $e = $!e;
-                    my int $found;
-                    if $!exclude {
-                        while $i before $e {
-                            $found = $found + 1;
-                            $i     = $i.succ;
-                        }
-                    }
-                    else {
-                        while not $i after $e {
-                            $found = $found + 1;
-                            $i     = $i.succ;
-                        }
-                    }
-                    nqp::p6box_i($found)
-                }
-                method sink-all {
-                    $!i = $!e;
-                    IterationEnd
                 }
+                method sink-all(--> IterationEnd) { $!i = $!e }
             }.new($!excludes-min ?? $!min.succ !! $!min,$!excludes-max,$!max)
         }
     }
@@ -270,15 +230,15 @@ my class Range is Cool does Iterable does Positional {
                 method pull-one() {
                     ( $!i = $!i - 1 ) >= $!n ?? $!i !! IterationEnd
                 }
-                method push-all($target) {
+                method push-all($target --> IterationEnd) {
                     my int $i = $!i;
                     my int $n = $!n;
                     $target.push(nqp::p6box_i($i)) while ($i = $i - 1) >= $n;
                     $!i = $i;
-                    IterationEnd
                 }
                 method count-only() { nqp::p6box_i($!i - $!n) }
-                method sink-all()   { $!i = $!n; IterationEnd }
+                method bool-only() { nqp::p6bool(nqp::isgt_i($!i,$!n)) }
+                method sink-all(--> IterationEnd)   { $!i = $!n }
             }.new($!max - $!excludes-max, $!min + $!excludes-min)
         }
 
@@ -326,15 +286,15 @@ my class Range is Cool does Iterable does Positional {
                              ?? nqp::chr($!i)
                              !! IterationEnd
                        }
-                       method push-all($target) {
+                       method push-all($target --> IterationEnd) {
                            my int $i = $!i;
                            my int $n = $!n;
                            $target.push(nqp::chr($i)) while ($i = $i - 1) >= $n;
                            $!i = $i;
-                           IterationEnd
                        }
                        method count-only() { nqp::p6box_i($!i - $!n) }
-                       method sink-all()   { $!i = $!n; IterationEnd }
+                       method bool-only() { nqp::p6bool(nqp::isgt_i($!i,$!n)) }
+                       method sink-all(--> IterationEnd) { $!i = $!n }
                    }.new($max, $!excludes-min ?? $!min.succ !! $!min)
                 !! SEQUENCE($max,$!min,:exclude_end($!excludes-min)).iterator
         }
@@ -366,7 +326,7 @@ my class Range is Cool does Iterable does Positional {
                         IterationEnd
                     }
                 }
-                method push-all($target) {
+                method push-all($target --> IterationEnd) {
                     my Mu $i = $!i;
                     my Mu $e = $!e;
                     if $!exclude {
@@ -381,30 +341,8 @@ my class Range is Cool does Iterable does Positional {
                             $i = $i.pred;
                         }
                     }
-                    IterationEnd
-                }
-                method count-only {
-                    my Mu $i = $!i;
-                    my Mu $e = $!e;
-                    my int $found;
-                    if $!exclude {
-                        while $i after $e {
-                            $found = $found + 1;
-                            $i     = $i.pred;
-                        }
-                    }
-                    else {
-                        while not $i before $e {
-                            $found = $found + 1;
-                            $i     = $i.pred;
-                        }
-                    }
-                    nqp::p6box_i($found)
-                }
-                method sink-all {
-                    $!i = $!e;
-                    IterationEnd
                 }
+                method sink-all(--> IterationEnd) { $!i = $!e }
             }.new($!excludes-max ?? $!max.pred !! $!max,$!excludes-min,$!min)
         }
     }
@@ -543,10 +481,9 @@ my class Range is Cool does Iterable does Positional {
                           ?? $!min + nqp::rand_I($!elems, Int)
                           !! IterationEnd
                     }
-                    method push-all($target) {
+                    method push-all($target --> IterationEnd) {
                         $target.push($!min + nqp::rand_I($!elems, Int))
                           while $!todo--;
-                        IterationEnd
                     }
                 }.new($!min + $!excludes-min,$elems,0 max $todo))
               !! self.list.roll($todo)
@@ -591,7 +528,7 @@ my class Range is Cool does Iterable does Positional {
                             IterationEnd
                         }
                     }
-                    method push-all($target) {
+                    method push-all($target --> IterationEnd) {
                         my str $key;
                         while $!todo {
                             my Int $value = $!min + nqp::rand_I($!elems, Int);
@@ -602,7 +539,6 @@ my class Range is Cool does Iterable does Positional {
                                 nqp::bindkey($!seen,$key,1);
                             }
                         }
-                        IterationEnd
                     }
                 }.new($!min + $!excludes-min,$elems,0 max $todo))
               !! self.list.pick($todo)
@@ -628,29 +564,31 @@ my class Range is Cool does Iterable does Positional {
         my $min    = $!min [&op] $value;
         my $max    = $!max [&op] $value;
         my $is-int = nqp::istype($min,Int) && nqp::istype($max,Int);
-        self.clone( :$min, :$max, :$is-int );
+        my $clone := self.clone( :$min, :$max );
+        nqp::bindattr_i($clone, $clone.WHAT, '$!is-int', $is-int);
+        $clone;
     }
 
     method push(|) is nodal {
         X::Immutable.new(:typename<Range>,:method<push>).throw
-    }   
+    }
     method append(|) is nodal {
         X::Immutable.new(:typename<Range>,:method<append>).throw
-    }   
+    }
     method unshift(|) is nodal {
         X::Immutable.new(:typename<Range>,:method<unshift>).throw
-    }   
+    }
     method prepend(|) is nodal {
         X::Immutable.new(:typename<Range>,:method<prepend>).throw
-    }   
+    }
     method shift(|) is nodal {
         X::Immutable.new(:typename<Range>,:method<shift>).throw
-    }   
+    }
     method pop(|) is nodal {
         X::Immutable.new(:typename<Range>, :method<pop>).throw
-    }   
+    }
 
-    method sum() {
+    method sum() is nodal {
         my ($start,$stop) = self.int-bounds || nextsame;
         my $elems = 0 max $stop - $start + 1;
         ($start + $stop) * $elems div 2;
@@ -673,25 +611,27 @@ my class Range is Cool does Iterable does Positional {
         if $!excludes-min || $!excludes-max {
             if $!excludes-min {
                 if $!excludes-max {
-                    $value = $range.rand while $value == 0 || $value == $range;
+                    $value = $range.rand
+                        while $value+$!min == $!min || $value+$!min == $!max;
                 }
                 else {
-                    $value = $range.rand while $value == 0;
+                    $value = $range.rand while $value+$!min == $!min;
                 }
             }
             else {  # $!excludes-max
-                $value = $range;
-                $value = $range.rand while $value == $range;
+                repeat {
+                    $value = $range.rand
+                } while $value+$!min == $!max;
             }
         }
         else {
             $value = $range.rand
         }
-        $!min + $value
+        $value + $!min;
     }
 
     method in-range($got, $what?) {
-        self.ACCEPTS($got) 
+        self.ACCEPTS($got)
           || X::OutOfRange.new(:what($what // 'Value'),:$got,:range(self)).throw
     }
 
@@ -721,12 +661,20 @@ sub prefix:<^>($max) is pure {
 }
 
 multi sub infix:<eqv>(Range:D \a, Range:D \b) {
-    a =:= b
-      || (a.WHAT =:= b.WHAT
-        && a.min eqv b.min
-        && a.max eqv b.max
-        && a.excludes-min eqv b.excludes-min
-        && a.excludes-max eqv b.excludes-max)
+    nqp::p6bool(
+      nqp::eqaddr(a,b)
+        || (nqp::eqaddr(a.WHAT,b.WHAT)
+             && a.min eqv b.min
+             && a.max eqv b.max
+             && nqp::iseq_i(
+               nqp::getattr_i(nqp::decont(a),Range,'$!excludes-min'),
+               nqp::getattr_i(nqp::decont(b),Range,'$!excludes-min')
+             )
+             && nqp::iseq_i(
+               nqp::getattr_i(nqp::decont(a),Range,'$!excludes-max'),
+               nqp::getattr_i(nqp::decont(b),Range,'$!excludes-max')
+             ))
+    )
 }
 
 multi sub infix:<+>(Range:D \a, Real:D \b) { a.clone-with-op(&[+], b) }
