@@ -64,25 +64,26 @@ my role Real does Numeric {
 
     method polymod(Real:D: +@mods) {
         my $more = self;
-        my $inf = @mods.is-lazy;
+        my $lazy = @mods.is-lazy;
         fail X::OutOfRange.new(
           what => 'invocant to polymod', got => $more, range => "0..*"
         ) if $more < 0;
         gather {
             for @mods -> $mod {
-                last if $inf and not $more;
-                fail X::Numeric::DivideByZero.new(
+                last if $lazy and not $more;
+                Failure.new(X::Numeric::DivideByZero.new:
                   using => 'polymod', numerator => $more
                 ) unless $mod;
                 take my $rem = $more % $mod;
                 $more -= $rem;
                 $more /= $mod;
             }
-            take $more unless $inf;
+            take $more if ($lazy and $more) or not $lazy;
         }
     }
 
-    method base(Int:D $base, $digits?) {
+    method base(Int:D $base, $digits? is copy) {
+        $digits = Nil if nqp::istype($digits, Whatever);
         fail X::OutOfRange.new(
                 what => 'digits argument to base', got => $digits, range => "0..*"
             ) if $digits.defined and $digits < 0;
