@@ -1,9 +1,11 @@
 my class Scalar { # declared in BOOTSTRAP
-    # class Scalar is Any {
+    # class Scalar is Any
     #     has Mu $!descriptor;
     #     has Mu $!value;
     #     has Mu $!whence;
 
+    method new(|) { X::Cannot::New.new(class => self.WHAT).throw }
+
     multi method WHICH(Scalar:D:) {
         'Scalar|' ~ nqp::objectid($!descriptor);
     }
