@@ -40,9 +40,7 @@ my class Seq is Cool does Iterable does PositionalBindFailover {
     # The only valid way to create a Seq directly is by giving it the
     # iterator it will consume and maybe memoize.
     method new(Iterator:D $iter) {
-        my $seq := nqp::create(self);
-        nqp::bindattr($seq, Seq, '$!iter', nqp::decont($iter));
-        $seq
+        nqp::p6bindattrinvres(nqp::create(self),Seq,'$!iter',nqp::decont($iter))
     }
 
     method new-consumed() {
@@ -52,16 +50,22 @@ my class Seq is Cool does Iterable does PositionalBindFailover {
     method is-ready(Seq:D:) { $!iter.DEFINITE }
 
     method iterator(Seq:D:) {
-        my \iter = $!iter;
-        X::Seq::Consumed.new.throw unless iter.DEFINITE;
-        $!iter := Iterator;
-        iter
+        nqp::if(
+          (my \iter = $!iter).DEFINITE,
+          nqp::stmts(
+            ($!iter := Iterator),
+            iter
+          ),
+          X::Seq::Consumed.new.throw
+        )
     }
 
     method is-lazy(Seq:D:) {
-        my \iter = $!iter;
-        X::Seq::Consumed.new.throw unless iter.DEFINITE;
-        iter.is-lazy
+        nqp::if(
+          $!iter.DEFINITE,
+          $!iter.is-lazy,
+          X::Seq::Consumed.new.throw
+        )
     }
 
     method eager {
@@ -81,21 +85,47 @@ my class Seq is Cool does Iterable does PositionalBindFailover {
     }
 
     method elems() {
-        self.is-lazy
-          ?? Failure.new(X::Cannot::Lazy.new(action => '.elems'))
-          !! self.iterator.count-only;
+        nqp::if(
+          self.is-lazy,
+          Failure.new(X::Cannot::Lazy.new(action => '.elems')),
+          nqp::if(
+            ($!iter.DEFINITE && nqp::can($!iter,'count-only')),
+            $!iter.count-only,
+            self.cache.elems
+          )
+        )
     }
 
     method Numeric() {
-        self.cache.Numeric
+        nqp::if(
+          ($!iter.DEFINITE && nqp::can($!iter,'count-only')),
+          $!iter.count-only,
+          self.cache.Numeric
+        )
     }
 
     method Int() {
-        self.cache.Int
+        nqp::if(
+          ($!iter.DEFINITE && nqp::can($!iter,'count-only')),
+          $!iter.count-only,
+          self.cache.Int
+        )
     }
 
     method Bool(Seq:D:) {
-        self.cache.Bool
+        nqp::if(
+          $!iter.DEFINITE,
+          nqp::if(
+            nqp::can($!iter,'bool-only'),
+            $!iter.bool-only,
+            nqp::if(
+              nqp::can($!iter,'count-only'),
+              ?$!iter.count-only,
+              self.cache.Bool
+            )
+          ),
+          self.cache.Bool
+        )
     }
 
     multi method Str(Seq:D:) {
@@ -462,25 +492,37 @@ sub GATHER(&block) {
 }
 
 multi sub infix:<eqv>(Seq:D \a, Seq:D \b) {
-
-    # we're us
-    return True  if a =:= b;
-
-    # not same container type
-    return False unless a.WHAT =:= b.WHAT;
-
-    my \ia := a.iterator;
-    my \ib := b.iterator;
-    my $va;
-    my $vb;
-
-    # same until a-list exhausted
-    return False
-      if    ($vb := ib.pull-one) =:= IterationEnd || !($va eqv $vb)
-      until ($va := ia.pull-one) =:= IterationEnd;
-
-    # b-list also exhausted?
-    ib.pull-one =:= IterationEnd
+    nqp::p6bool(
+      nqp::unless(
+        nqp::eqaddr(a,b),
+        nqp::if(
+          nqp::eqaddr(a.WHAT,b.WHAT),
+          nqp::if(
+            nqp::iseq_i(
+              (my \ia := a.iterator).is-lazy,
+              (my \ib := b.iterator).is-lazy
+            ),
+            nqp::if(
+              ia.is-lazy,
+              (die "Cannot eqv lazy Sequences"),
+              nqp::stmts(
+                nqp::until(
+                  nqp::stmts(
+                    (my \pa := ia.pull-one),
+                    (my \pb := ib.pull-one),
+                    nqp::eqaddr(pa,IterationEnd)
+                      || nqp::eqaddr(pb,IterationEnd)
+                      || nqp::not_i(pa eqv pb)
+                  ),
+                  nqp::null
+                ),
+                nqp::eqaddr(pa,pb)  # exhausted if both IterationEnd
+              )
+            )
+          )
+        )
+      )
+    )
 }
 
 # vim: ft=perl6 expandtab sw=4
