@@ -1,23 +1,41 @@
 my role Setty does QuantHash {
     has %!elems; # key.WHICH => key
 
-    submethod BUILD(:%!elems --> Nil)  { }
-    method default(--> Bool) { False }
+    method !SET-SELF(%!elems) { self }
+    multi method new(Setty: +@args --> Setty) {
+        nqp::stmts(
+          (my $elems := nqp::hash),
+          (my $iter  := @args.iterator),
+          nqp::until(
+            nqp::eqaddr((my $pulled := $iter.pull-one),IterationEnd),
+            nqp::bindkey($elems,$pulled.WHICH,$pulled)
+          ),
+          nqp::create(self)!SET-SELF($elems)
+        )
+    }
+    method new-from-pairs(*@pairs --> Setty) {
+        nqp::stmts(
+          (my $elems := nqp::hash),
+          (my $iter  := @pairs.iterator),
+          nqp::until(
+            nqp::eqaddr((my $pulled := $iter.pull-one),IterationEnd),
+            nqp::if(
+              nqp::istype($pulled,Pair),
+              nqp::if(
+                $pulled.value,
+                nqp::bindkey($elems,$pulled.key.WHICH,$pulled.key)
+              ),
+              nqp::bindkey($elems,$pulled.WHICH,$pulled)
+            )
+          ),
+          nqp::create(self)!SET-SELF($elems)
+        )
+    }
+
+    method default(--> False) { }
 
     multi method keys(Setty:D:) {
-        %!elems.values
-    }
-    multi method kv(Setty:D:) {
-        %!elems.values.map: -> \key { |(key,self.ISINSET(key.WHICH)) }
-    }
-    multi method values(Setty:D:) {
-        %!elems.values.map: -> \key { self.ISINSET(key.WHICH) }
-    }
-    multi method pairs(Setty:D:) {
-        %!elems.values.map: -> \key { Pair.new(key,self.ISINSET(key.WHICH)) }
-    }
-    multi method antipairs(Setty:D:) {
-        %!elems.values.map: -> \key { Pair.new(True,key) }
+        Seq.new(Rakudo::Internals::MappyIterator-values.new(%!elems))
     }
 
     method elems(Setty:D: --> Int) { %!elems.elems }
@@ -32,24 +50,6 @@ my role Setty does QuantHash {
         e;
     }
 
-    multi method new(Setty: +@args) {
-        my %e;
-        %e{$_.WHICH} = $_ for @args;
-        self.bless(:elems(%e))
-    }
-    method new-from-pairs(*@pairs --> Setty) {
-        my %e;
-        for @pairs {
-            when Pair {
-                %e{.key.WHICH} //= $_.key if .value;
-            }
-            default {
-                %e{.WHICH} //= $_;
-            }
-        }
-        self.bless(:elems(%e));
-    }
-
     multi method ACCEPTS(Setty:U: $other) {
         $other.^does(self)
     }
@@ -76,15 +76,33 @@ my role Setty does QuantHash {
         ~ ')';
     }
 
-    method grab(Setty:D: $count = 1) {
+    proto method grab(|) { * }
+    multi method grab(Setty:D:) {
+        %!elems.DELETE-KEY(%!elems.keys.pick)
+    }
+    multi method grab(Setty:D: Callable:D $calculate) {
+        self.grab($calculate(%!elems.elems))
+    }
+    multi method grab(Setty:D: $count) {
         (%!elems{ %!elems.keys.pick($count) }:delete).cache;
     }
-    method grabpairs(Setty:D: $count = 1) {
+
+    proto method grabpairs(|) { * }
+    multi method grabpairs(Setty:D:) {
+        Pair.new(%!elems.DELETE-KEY(%!elems.keys.pick),True)
+    }
+    multi method grabpairs(Setty:D: Callable:D $calculate) {
+        self.grabpairs($calculate(%!elems.elems))
+    }
+    multi method grabpairs(Setty:D: $count) {
         (%!elems{ %!elems.keys.pick($count) }:delete).map( { ($_=>True) } );
     }
 
     proto method pick(|) { * }
     multi method pick(Setty:D:)       { %!elems.values.pick()       }
+    multi method pick(Setty:D: Callable:D $calculate) {
+        %!elems.values.pick($calculate(%!elems.elems))
+    }
     multi method pick(Setty:D: $count) { %!elems.values.pick($count) }
 
     proto method roll(|) { * }
@@ -99,6 +117,8 @@ my role Setty does QuantHash {
 
     method Bag { Bag.new( %!elems.values ) }
     method BagHash { BagHash.new( %!elems.values ) }
+    method Mix { Mix.new( %!elems.values ) }
+    method MixHash { MixHash.new( %!elems.values ) }
 
     # TODO: WHICH will require the capability for >1 pointer in ObjAt
 }
