@@ -7,18 +7,24 @@ my role SlippyIterator does Iterator {
     # The current Slip we're iterating.
     has $!slip-iter;
 
-    method start-slip(Slip:D $slip) {
+    proto method start-slip(|) { * }
+    multi method start-slip(Slip:U $slip) {
+        $slip
+    }
+    multi method start-slip(Slip:D $slip) {
         $!slipping = 1;
         $!slip-iter := $slip.iterator;
         self.slip-one()
     }
 
     method slip-one() {
-        my \result = $!slip-iter.pull-one;
-        if result =:= IterationEnd {
-            $!slipping = 0;
-            $!slip-iter := Mu;
-        }
+        nqp::if(
+          nqp::eqaddr((my \result := $!slip-iter.pull-one),IterationEnd),
+          nqp::stmts(
+            ($!slipping = 0),
+            ($!slip-iter := Mu)
+          )
+        );
         result
     }
 }
