@@ -3,8 +3,7 @@ my class StrDistance is Cool {
     has Str $.after;
     has Int $!distance;
 
-    submethod BUILD(Str() :$!before, :$!after) {
-    }
+    submethod BUILD(Str() :$!before, :$!after --> Nil) { }
 
     method Bool() {
         $.before ne $.after
@@ -18,6 +17,10 @@ my class StrDistance is Cool {
         self.Int
     }
 
+    method Str {
+        $.after
+    }
+
     multi method Int(StrDistance:D:) {
         $!distance //= do {
             my @s = *, |$.before.comb;
