@@ -10,8 +10,11 @@ my class Tap {
     }
 
     method close() {
-        &!on-close() if &!on-close;
-        True;
+        if &!on-close {
+            my \close-result = &!on-close();
+            await close-result if nqp::istype(close-result, Promise);
+        }
+        True
     }
 }
 
@@ -26,7 +29,7 @@ my role Tappable {
     method tap() { ... }
     method live() { ... }    # Taps into a live data source
     method serial() { ... }  # Promises no concurrent emits
-    method sane() { ... }    # Matches emit* [done|quit] grammar
+    method sane() { ... }    # Matches emit* [done|quit]? grammar
 }
 
 # A few Supply-related exception types.
@@ -121,10 +124,11 @@ my class Supply {
             submethod BUILD(:$!scheduler, :$!interval, :$!delay --> Nil) { }
 
             method tap(&emit, |) {
+                my $i = 0;
+                my $lock = Lock.new;
                 my $cancellation = $!scheduler.cue(
                     {
-                        state $i = 0;
-                        emit($i++);
+                        emit($lock.protect: { $i++ });
                         CATCH { $cancellation.cancel if $cancellation }
                     },
                     :every($!interval), :in($!delay)
@@ -454,7 +458,7 @@ my class Supply {
 
         X::Supply::Combinator.new(
            combinator => 'merge'
-        ).throw if Rakudo::Internals.NOT_ALL_DEFINED_TYPE(@s,Supply);
+        ).throw unless Rakudo::Internals.ALL_DEFINED_TYPE(@s,Supply);
 
         return @s[0].sanitize  if +@s == 1; # nothing to be done
 
@@ -619,7 +623,7 @@ my class Supply {
     method unique(Supply:D $self: :&as, :&with, :$expires) {
         supply {
             if $expires {
-                if &with and &with !=== &[===] {
+                if &with and !(&with === &[===]) {
                     my @seen;  # really Mu, but doesn't work in settings
                     my Mu $target;
                     if &as {
@@ -686,7 +690,7 @@ my class Supply {
                 }
             }
             else { # !$!expires
-                if &with and &with !=== &[===] {
+                if &with and !(&with === &[===]) {
                     my @seen;  # really Mu, but doesn't work in settings
                     my Mu $target;
                     if &as {
@@ -1100,7 +1104,7 @@ my class Supply {
 
         X::Supply::Combinator.new(
            combinator => 'zip'
-        ).throw if Rakudo::Internals.NOT_ALL_DEFINED_TYPE(@s,Supply);
+        ).throw unless Rakudo::Internals.ALL_DEFINED_TYPE(@s,Supply);
 
         return @s[0]  if +@s == 1;          # nothing to be done
 
@@ -1129,7 +1133,7 @@ my class Supply {
 
         X::Supply::Combinator.new(
            combinator => 'zip-latest'
-        ).throw if Rakudo::Internals.NOT_ALL_DEFINED_TYPE(@s,Supply);
+        ).throw unless Rakudo::Internals.ALL_DEFINED_TYPE(@s,Supply);
 
         return @s[0] if +@s == 1;           # nothing to do.
 
@@ -1219,8 +1223,6 @@ my class Supply {
                     @buffer.push(val);
                 }
                 LAST {
-                    $control.done if $control;
-                    $status.done  if $status;
                     if $status {
                         emit-status("done");
                         $status.done;
@@ -1408,7 +1410,7 @@ my class Supplier {
             })
         }
 
-        method emit(\value) {
+        method emit(\value --> Nil) {
             my $snapshot := $!tappers;
             if nqp::isconcrete($snapshot) {
                 my int $n = nqp::elems($snapshot);
@@ -1418,7 +1420,7 @@ my class Supplier {
             }
         }
 
-        method done() {
+        method done(--> Nil) {
             my $snapshot := $!tappers;
             if nqp::isconcrete($snapshot) {
                 my int $n = nqp::elems($snapshot);
@@ -1428,7 +1430,7 @@ my class Supplier {
             }
         }
 
-        method quit($ex) {
+        method quit($ex --> Nil) {
             my $snapshot := $!tappers;
             if nqp::isconcrete($snapshot) {
                 my int $n = nqp::elems($snapshot);
@@ -1521,7 +1523,7 @@ my class Supplier::Preserving is Supplier {
             })
         }
 
-        method emit(\value) {
+        method emit(\value --> Nil) {
             my int $sent = 0;
             my $snapshot := $!tappers;
             if nqp::isconcrete($snapshot) {
@@ -1536,7 +1538,7 @@ my class Supplier::Preserving is Supplier {
             }
         }
 
-        method done() {
+        method done(--> Nil) {
             my int $sent = 0;
             my $snapshot := $!tappers;
             if nqp::isconcrete($snapshot) {
@@ -1551,7 +1553,7 @@ my class Supplier::Preserving is Supplier {
             }
         }
 
-        method quit($ex) {
+        method quit($ex --> Nil) {
             my int $sent = 0;
             my $snapshot := $!tappers;
             if nqp::isconcrete($snapshot) {
@@ -1591,10 +1593,75 @@ sub SUPPLY(&block) {
         has &.emit;
         has &.done;
         has &.quit;
-        has $.lock;
-        has $.active is rw;
-        has %.active-taps;
         has @.close-phasers;
+        has $!active = 1;
+        has $!lock = Lock.new;
+        has %!active-taps;
+        has @!queued-operations;
+
+        method increment-active() {
+            $!lock.protect: { ++$!active }
+        }
+
+        method decrement-active() {
+            $!lock.protect: { --$!active }
+        }
+
+        method get-and-zero-active() {
+            $!lock.protect: {
+                my $result = $!active;
+                $!active = 0;
+                $result
+            }
+        }
+
+        method add-active-tap($tap --> Nil) {
+            $!lock.protect: { %!active-taps{nqp::objectid($tap)} = $tap }
+        }
+
+        method delete-active-tap($tap --> Nil) {
+            $!lock.protect: { %!active-taps{nqp::objectid($tap)}:delete }
+        }
+
+        method consume-active-taps() {
+            my @active;
+            $!lock.protect: {
+                @active = %!active-taps.values;
+                %!active-taps = ();
+            }
+            @active
+        }
+
+        method run-operation(&op --> Nil) {
+            if $!active {
+                my $run-now = False;
+                $!lock.protect({
+                    if @!queued-operations {
+                        @!queued-operations.push({
+                            op();
+                            self!maybe-another();
+                        });
+                    }
+                    else {
+                        @!queued-operations.push(&op);
+                        $run-now = True;
+                    }
+                });
+                if $run-now {
+                    op();
+                    self!maybe-another();
+                }
+            }
+        }
+
+        method !maybe-another(--> Nil) {
+            my &another;
+            $!lock.protect({
+                @!queued-operations.shift;
+                &another = @!queued-operations[0] if $!active && @!queued-operations;
+            });
+            &another && another();
+        }
     }
 
     Supply.new(class :: does Tappable {
@@ -1603,75 +1670,78 @@ sub SUPPLY(&block) {
         submethod BUILD(:&!block --> Nil) { }
 
         method tap(&emit, &done, &quit) {
-            my $state = SupplyBlockState.new(
-                :&emit, :&done, :&quit,
-                lock => Lock.new,
-                active => 1);
+            my $state = SupplyBlockState.new(:&emit, :&done, :&quit);
             self!run-supply-code(&!block, $state);
-            $state.close-phasers.push(.clone) for &!block.phasers('CLOSE');
+            if nqp::istype(&!block,Block) {
+                $state.close-phasers.push(.clone) for &!block.phasers('CLOSE')
+            }
             self!deactivate-one($state);
             Tap.new(-> { self!teardown($state) })
         }
 
         method !run-supply-code(&code, $state) {
-            my &*ADD-WHENEVER = sub ($supply, &whenever-block) {
-                $state.active++;
-                my $tap = $supply.tap(
-                    -> \value {
-                        self!run-supply-code({ whenever-block(value) }, $state)
-                    },
-                    done => {
-                        $state.active-taps{nqp::objectid($tap)}:delete if $tap.DEFINITE;
-                        my @phasers := &whenever-block.phasers('LAST');
-                        if @phasers {
-                            self!run-supply-code({ .() for @phasers }, $state)
-                        }
-                        self!deactivate-one($state);
-                    },
-                    quit => -> \ex {
-                        $state.active-taps{nqp::objectid($tap)}:delete if $tap.DEFINITE;
-                        my $handled;
-                        my $phaser := &whenever-block.phasers('QUIT')[0];
-                        if $phaser.DEFINITE {
-                            self!run-supply-code({ $handled = $phaser(ex) === Nil }, $state)
-                        }
-                        if $handled {
+            $state.run-operation({
+                my &*ADD-WHENEVER = sub ($supply, &whenever-block) {
+                    $state.increment-active();
+                    my $tap = $supply.tap(
+                        -> \value {
+                            self!run-supply-code({ whenever-block(value) }, $state)
+                        },
+                        done => {
+                            $state.delete-active-tap($tap) if $tap.DEFINITE;
+                            my @phasers := &whenever-block.phasers('LAST');
+                            if @phasers {
+                                self!run-supply-code({ .() for @phasers }, $state)
+                            }
                             self!deactivate-one($state);
-                        }
-                        elsif $state.active {
-                            $state.quit().(ex) if $state.quit;
-                            $state.active = 0;
-                            self!teardown($state);
-                        }
-                    });
-                $state.active-taps{nqp::objectid($tap)} = $tap;
-            }
+                        },
+                        quit => -> \ex {
+                            $state.delete-active-tap($tap) if $tap.DEFINITE;
+                            self!run-supply-code({
+                                my $handled;
+                                my $phaser := &whenever-block.phasers('QUIT')[0];
+                                if $phaser.DEFINITE {
+                                    $handled = $phaser(ex) === Nil;
+                                }
+                                if $handled {
+                                    self!deactivate-one($state);
+                                }
+                                elsif $state.get-and-zero-active() {
+                                    $state.quit().(ex) if $state.quit;
+                                    self!teardown($state);
+                                }
+                            }, $state);
+                        });
+                    $state.add-active-tap($tap);
+                    $tap
+                }
 
-            my $emitter = {
-                my \ex := nqp::exception();
-                $state.emit().(nqp::getpayload(ex)) if $state.emit;
-                nqp::resume(ex)
-            }
-            my $done = {
-                $state.done().() if $state.done;
-                $state.active = 0;
-                self!teardown($state);
-            }
-            my $catch = {
-                my \ex = EXCEPTION(nqp::exception());
-                $state.quit().(ex) if $state.quit;
-                $state.active = 0;
-                self!teardown($state);
-            }
-            nqp::handle($state.lock.protect(&code),
-                'EMIT', $emitter(),
-                'DONE', $done(),
-                'CATCH', $catch());
+                my $emitter = {
+                    my \ex := nqp::exception();
+                    $state.emit().(nqp::getpayload(ex)) if $state.emit;
+                    nqp::resume(ex)
+                }
+                my $done = {
+                    $state.done().() if $state.done;
+                    $state.get-and-zero-active();
+                    self!teardown($state);
+                }
+                my $catch = {
+                    my \ex = EXCEPTION(nqp::exception());
+                    $state.quit().(ex) if $state.quit;
+                    $state.get-and-zero-active();
+                    self!teardown($state);
+                }
+                nqp::handle(code(),
+                    'EMIT', $emitter(),
+                    'DONE', $done(),
+                    'CATCH', $catch());
+            });
         }
 
         method !deactivate-one($state) {
-            $state.lock.protect({
-                if --$state.active == 0 {
+            $state.run-operation({
+                if $state.decrement-active() == 0 {
                     $state.done().() if $state.done;
                     self!teardown($state);
                 }
@@ -1679,8 +1749,7 @@ sub SUPPLY(&block) {
         }
 
         method !teardown($state) {
-            .close for $state.active-taps.values;
-            $state.active-taps = ();
+            .close for $state.consume-active-taps;
             while $state.close-phasers.pop() -> $close {
                 $close();
             }
