@@ -12,7 +12,12 @@ my class Thread {
     # Thread's (user-defined) name.
     has Str $.name;
 
-    submethod BUILD(:&code!, Bool() :$!app_lifetime = False, Str() :$!name = "<anon>") {
+    submethod BUILD(
+             :&code!,
+      Bool() :$!app_lifetime = False,
+      Str()  :$!name         = "<anon>"
+      --> Nil
+    ) {
         $!vm_thread := nqp::newthread(
             anon sub THREAD-ENTRY() {
                 my $*THREAD = self;
@@ -52,6 +57,9 @@ my class Thread {
     multi method Str(Thread:D:) {
         "Thread<$.id>($.name)"
     }
+    multi method gist(Thread:D:) {
+        "Thread #$.id" ~ ($!name ne '<anon>' ?? " ($!name)" !! '')
+    }
 
     method yield(Thread:U:) {
         nqp::threadyield();
@@ -59,7 +67,7 @@ my class Thread {
     }
 }
 
-multi sub INITIALIZE_DYNAMIC ('$*THREAD') {
+Rakudo::Internals.REGISTER-DYNAMIC: '$*THREAD', {
     my $init_thread := nqp::create(Thread);
     nqp::bindattr($init_thread, Thread, '$!vm_thread', nqp::currentthread());
     nqp::bindattr($init_thread, Thread, '$!app_lifetime', False);
