@@ -42,20 +42,18 @@ my class ThreadPoolScheduler does Scheduler {
                     my Mu $task := nqp::shift($!queue);
                     $!counts_lock.protect: { $!loads = $!loads + 1 };
                     try {
-                        try {
-                            if nqp::islist($task) {
-                                my Mu $code := nqp::shift($task);
-                                my \args = nqp::p6bindattrinvres(nqp::create(List), List, '$!reified', $task);
-                                $code(|args);
-                            }
-                            else {
-                                $task();
-                            }
-                            CONTROL {
-                                default {
-                                    my Mu $vm-ex := nqp::getattr(nqp::decont($_), Exception, '$!ex');
-                                    nqp::getcomp('perl6').handle-control($vm-ex);
-                                }
+                        if nqp::islist($task) {
+                            my Mu $code := nqp::shift($task);
+                            my \args = nqp::p6bindattrinvres(nqp::create(List), List, '$!reified', $task);
+                            $code(|args);
+                        }
+                        else {
+                            $task();
+                        }
+                        CONTROL {
+                            default {
+                                my Mu $vm-ex := nqp::getattr(nqp::decont($_), Exception, '$!ex');
+                                nqp::getcomp('perl6').handle-control($vm-ex);
                             }
                         }
                         CATCH {
@@ -72,7 +70,8 @@ my class ThreadPoolScheduler does Scheduler {
 
     submethod BUILD(
         Int :$!initial_threads = 0,
-        Int :$!max_threads = (%*ENV<RAKUDO_MAX_THREADS> // 16).Int,
+        Int :$!max_threads = (%*ENV<RAKUDO_MAX_THREADS> // 16).Int
+        --> Nil
     ) {
         die "Initial thread pool threads ($!initial_threads) must be less than or equal to maximum threads ($!max_threads)"
             if $!initial_threads > $!max_threads;
@@ -85,37 +84,49 @@ my class ThreadPoolScheduler does Scheduler {
         $!queue
     }
 
-    method cue(&code, :$at, :$in, :$every, :$times = 1, :&stop, :&catch ) {
+    method cue(&code, :$at, :$in, :$every, :$times = 1, :&stop is copy, :&catch ) {
         my class TimerCancellation is repr('AsyncTask') { }
         die "Cannot specify :at and :in at the same time"
           if $at.defined and $in.defined;
-        die "Cannot specify :every and :times at the same time"
-          if $every.defined and $times > 1;
+        die "Cannot specify :every, :times and :stop at the same time"
+          if $every.defined and $times > 1 and &stop;
         my $delay = $at ?? $at - now !! $in // 0;
         self!initialize unless $!started_any;
 
         # need repeating
         if $every {
 
+            # generate a stopper if needed
+            if $times > 1 {
+                my $todo = $times;
+                &stop = sub { $todo ?? !$todo-- !! True }
+            }
+
             # we have a stopper
             if &stop {
+                my $handle;
                 my $cancellation;
-                my $handle := nqp::timer($!queue,
+                sub cancellation() {
+                    $cancellation //=
+                      Cancellation.new(async_handles => [$handle]);
+                }
+                $handle := nqp::timer($!queue,
                     &catch
                       ?? -> {
-                          code();
-                          $cancellation.cancel if stop();
+                          stop()
+                            ?? cancellation().cancel
+                            !! code();
                           CATCH { default { catch($_) } };
                       }
                       !! -> {
-                          code();
-                          $cancellation.cancel if stop();
+                          stop()
+                            ?? cancellation().cancel
+                            !! code();
                       },
-                    ($delay * 1000).Int, ($every * 1000).Int,
+                    to-millis($delay), to-millis($every),
                     TimerCancellation);
                 self!maybe_new_thread();
-                return
-                  $cancellation = Cancellation.new(async_handles => [$handle]);
+                return cancellation()
             }
 
             # no stopper
@@ -124,7 +135,7 @@ my class ThreadPoolScheduler does Scheduler {
                     &catch
                       ?? -> { code(); CATCH { default { catch($_) } } }
                       !! &code,
-                    ($delay * 1000).Int, ($every * 1000).Int,
+                    to-millis($delay), to-millis($every),
                     TimerCancellation);
                 self!maybe_new_thread();
                 return Cancellation.new(async_handles => [$handle]);
@@ -137,11 +148,10 @@ my class ThreadPoolScheduler does Scheduler {
                 ?? -> { code(); CATCH { default { catch($_) } } }
                 !! &code;
             my @async_handles;
-            for 1 .. $times {
-                @async_handles.push(nqp::timer($!queue, $todo,
-                    ($delay * 1000).Int, 0, TimerCancellation));
-                $delay = 0;
-            }
+            $delay = to-millis($delay) if $delay;
+            @async_handles.push(
+              nqp::timer($!queue, $todo, $delay, 0, TimerCancellation)
+            ) for 1 .. $times;
             self!maybe_new_thread();
             return Cancellation.new(:@async_handles);
         }
@@ -162,6 +172,21 @@ my class ThreadPoolScheduler does Scheduler {
         $!loads
     }
 
+    multi to-millis(Int $value) {
+        1000 * $value
+    }
+    multi to-millis(Numeric $value) {
+        my $proposed = (1000 * $value).Int;
+        if $value && $proposed == 0 {
+            warn "Minimum timer resolution is 1ms; using that instead of {1000 * $value}ms";
+            $proposed = 1;
+        }
+        $proposed
+    }
+    multi to-millis($value) {
+        to-millis(+$value)
+    }
+
     method !initialize() {
         $!queue                  := nqp::create(Queue);
         $!thread_start_semaphore := Semaphore.new($!max_threads.Int);
@@ -171,7 +196,7 @@ my class ThreadPoolScheduler does Scheduler {
 }
 
 # This thread pool scheduler will be the default one.
-multi sub INITIALIZE_DYNAMIC('$*SCHEDULER') {
+Rakudo::Internals.REGISTER-DYNAMIC: '$*SCHEDULER', {
     PROCESS::<$SCHEDULER> = ThreadPoolScheduler.new();
 }
 
