@@ -65,19 +65,42 @@ my class ComplexStr is Complex is Str {
 multi sub infix:<cmp>(IntStr:D $a, IntStr:D $b) {
     $a.Int cmp $b.Int || $a.Str cmp $b.Str
 }
-
 multi sub infix:<cmp>(RatStr:D $a, RatStr:D $b) {
     $a.Rat cmp $b.Rat || $a.Str cmp $b.Str
 }
-
 multi sub infix:<cmp>(NumStr:D $a, NumStr:D $b) {
     $a.Num cmp $b.Num || $a.Str cmp $b.Str
 }
-
 multi sub infix:<cmp>(ComplexStr:D $a, ComplexStr:D $b) {
     $a.Complex cmp $b.Complex || $a.Str cmp $b.Str
 }
 
+multi sub infix:<eqv>(IntStr:D $a, IntStr:D $b) {
+    $a.Int eqv $b.Int || $a.Str eqv $b.Str
+}
+multi sub infix:<eqv>(RatStr:D $a, RatStr:D $b) {
+    $a.Rat eqv $b.Rat || $a.Str eqv $b.Str
+}
+multi sub infix:<eqv>(NumStr:D $a, NumStr:D $b) {
+    $a.Num eqv $b.Num || $a.Str eqv $b.Str
+}
+multi sub infix:<eqv>(ComplexStr:D $a, ComplexStr:D $b) {
+    $a.Complex eqv $b.Complex || $a.Str eqv $b.Str
+}
+
+multi sub infix:<===>(IntStr:D $a, IntStr:D $b) {
+    $a.Int === $b.Int && $a.Str === $b.Str
+}
+multi sub infix:<===>(RatStr:D $a, RatStr:D $b) {
+    $a.Rat === $b.Rat && $a.Str === $b.Str
+}
+multi sub infix:<===>(NumStr:D $a, NumStr:D $b) {
+    $a.Num === $b.Num && $a.Str === $b.Str
+}
+multi sub infix:<===>(ComplexStr:D $a, ComplexStr:D $b) {
+    $a.Complex === $b.Complex && $a.Str === $b.Str
+}
+
 multi sub val(*@maybevals) {
     @maybevals.list.map({ val($_) }).eager;
 }
@@ -144,13 +167,13 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
     my &parse_win := -> \newval {
         $val-or-fail
           ?? return newval
-          !! newval.isa(Num)
+          !! nqp::istype(newval, Num)
             ?? return NumStr.new(newval, $MAYBEVAL)
-            !! newval.isa(Rat)
+            !! nqp::istype(newval, Rat)
               ?? return RatStr.new(newval, $MAYBEVAL)
-              !! newval.isa(Complex)
+              !! nqp::istype(newval, Complex)
                 ?? return ComplexStr.new(newval, $MAYBEVAL)
-                !! newval.isa(Int)
+                !! nqp::istype(newval, Int)
                   ?? return IntStr.new(newval, $MAYBEVAL)
                   !! die "Unknown type {newval.^name} found in val() processing"
     }
@@ -162,10 +185,10 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
             return nqp::p6box_n(nqp::nan());
         }
 
-        # Handle any leading +/- sign
+        # Handle any leading +/-/− sign
         my int $ch  = nqp::ord($str, $pos);
-        my int $neg = nqp::iseq_i($ch, 45);                # '-'
-        if nqp::iseq_i($ch, 45) || nqp::iseq_i($ch, 43) {  # '-', '+'
+        my int $neg = nqp::iseq_i($ch, 45) || nqp::iseq_i($ch, 8722); # '-', '−'
+        if $neg || nqp::iseq_i($ch, 43) {  # '-', '−', '+'
             $pos = nqp::add_i($pos, 1);
             $ch  = nqp::islt_i($pos, $eos) && nqp::ord($str, $pos);
         }
@@ -181,7 +204,7 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
             my Int $int := 0;
             if nqp::isne_i($ch, 46) {  # '.'
                 parse_fail "Cannot convert radix of $radix (max 36)"
-                    if $radix > 36;
+                    if nqp::isgt_i($radix, 36);
                 $parse := nqp::radix_I($radix, $str, $pos, $neg, Int);
                 $p      = nqp::atpos($parse, 2);
                 parse_fail "base-$radix number must begin with valid digits or '.'"
@@ -189,12 +212,9 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
                 $pos    = $p;
 
                 $int   := nqp::atpos($parse, 0);
-                if nqp::isge_i($pos, $eos) {
-                    return $int;
-                }
-                else {
-                    $ch = nqp::ord($str, $pos);
-                }
+                nqp::isge_i($pos, $eos)
+                  ??  return $int
+                  !!  ($ch = nqp::ord($str, $pos));
             }
 
             # Fraction, if any
@@ -219,16 +239,32 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
                 parse_fail "'E' or 'e' style exponent only allowed on decimal (base-10) numbers, not base-$radix"
                     unless nqp::iseq_i($radix, 10);
 
-                $pos    = nqp::add_i($pos, 1);
-                $parse := nqp::radix_I(10, $str, $pos, 2, Int);
+                $pos = nqp::add_i($pos, 1);
+                # handle the sign
+                # XXX TODO: teach radix_I to handle '−' (U+2212) minus?
+                my int $ch  = nqp::islt_i($pos, $eos) && nqp::ord($str, $pos);
+                my int $neg-e = nqp::if(
+                    nqp::iseq_i($ch, 43), # '+'
+                    nqp::stmts(($pos = nqp::add_i($pos, 1)), 0),
+                    nqp::if( # '-', '−'
+                        nqp::iseq_i($ch, 45) || nqp::iseq_i($ch, 8722),
+                        nqp::stmts(($pos = nqp::add_i($pos, 1)), 1),
+                        0,
+                    )
+                );
+
+                $parse := nqp::radix_I(10, $str, $pos, $neg-e, Int);
                 $p      = nqp::atpos($parse, 2);
                 parse_fail "'E' or 'e' must be followed by decimal (base-10) integer"
                     if nqp::iseq_i($p, -1);
                 $pos    = $p;
 
-                my num $exp  = nqp::atpos($parse, 0).Num;
-                my num $coef = $frac ?? nqp::add_n($int.Num, nqp::div_n($frac.Num, $base.Num)) !! $int.Num;
-                return nqp::p6box_n(nqp::mul_n($coef, nqp::pow_n(10e0, $exp)));
+                return nqp::p6box_n(nqp::mul_n(
+                    $frac ?? nqp::add_n( $int.Num, nqp::div_n($frac.Num, $base.Num) )
+                          !! $int.Num,
+                    nqp::pow_n(10e0, nqp::atpos($parse, 0).Num)
+                )) # if we have a zero, handle the sign correctly
+                || nqp::if(nqp::iseq_i($neg, 1), -0e0, 0e0);
             }
 
             # Multiplier with exponent, if single '*' is present
@@ -240,7 +276,7 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
                 my $mult_base := parse-simple-number();
 
                 parse_fail "'*' multiplier base must be an integer"
-                    unless $mult_base.WHAT === Int;
+                    unless nqp::istype($mult_base, Int);
                 parse_fail "'*' multiplier base must be followed by '**' and exponent"
                     unless nqp::eqat($str,'**',$pos);
 
@@ -248,19 +284,15 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
                 my $mult_exp  := parse-simple-number();
 
                 parse_fail "'**' multiplier exponent must be an integer"
-                    unless $mult_exp.WHAT === Int;
+                    unless nqp::istype($mult_exp, Int);
 
                 my $mult := $mult_base ** $mult_exp;
                 $int     := $int  * $mult;
                 $frac    := $frac * $mult;
             }
 
-            # Return an Int if there was no radix point
-            return $int unless $base;
-
-            # Otherwise, return a Rat
-            my Int $numerator := $int * $base + $frac;
-            Rat.new($numerator, $base);
+            # Return an Int if there was no radix point, otherwise, return a Rat
+            nqp::unless($base, $int, Rat.new($int * $base + $frac, $base));
         }
 
         # Look for radix specifiers
@@ -275,7 +307,7 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
 
             $radix  = nqp::atpos($parse, 0);
             $ch     = nqp::islt_i($pos, $eos) && nqp::ord($str, $pos);
-            if    nqp::iseq_i($ch, 60) {  # '<'
+            if nqp::iseq_i($ch, 60) {  # '<'
                 $pos = nqp::add_i($pos, 1);
 
                 my $result := parse-int-frac-exp();
@@ -313,7 +345,7 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
 
                     $digit := nqp::atpos($parse, 0);
                     parse_fail "digit is larger than {$radix - 1} in ':$radix[]' style radix number"
-                        if $digit >= $radix;
+                        if nqp::isge_i($digit, $radix);
 
                     $result := $result * $radix + $digit;
                     $pos     = nqp::add_i($pos, 1)
@@ -388,7 +420,7 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
     # the magnitude of a pure imaginary number
     if nqp::iseq_i(nqp::ord($str, $pos), 105) {  # 'i'
         parse_fail "Imaginary component of 'NaN' or 'Inf' must be followed by \\i"
-            if $result == Inf || $result == NaN;
+            if nqp::isnanorinf($result.Num);
         $pos = nqp::add_i($pos, 1);
         $result := Complex.new(0, $result);
     }
@@ -398,8 +430,9 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
     }
     # Check for '+' or '-' indicating first parsed number was
     # the real part of a complex number
-    elsif nqp::iseq_i(nqp::ord($str, $pos), 45)    # '-'
-       || nqp::iseq_i(nqp::ord($str, $pos), 43) {  # '+'
+    elsif nqp::iseq_i(nqp::ord($str, $pos), 45)     # '-'
+       || nqp::iseq_i(nqp::ord($str, $pos), 43)     # '+'
+       || nqp::iseq_i(nqp::ord($str, $pos), 8722) { # '−'
         # Don't move $pos -- we want parse-real() to see the sign
         my $im := parse-real();
         parse_fail "imaginary part of complex number must be followed by 'i' or '\\i'"
@@ -407,7 +440,7 @@ multi sub val(Str:D $MAYBEVAL, :$val-or-fail) {
 
         if nqp::iseq_i(nqp::ord($str, $pos), 105) {  # 'i'
             parse_fail "Imaginary component of 'NaN' or 'Inf' must be followed by \\i"
-                if $im == Inf || $im == NaN;
+                if nqp::isnanorinf($im.Num);
             $pos = nqp::add_i($pos, 1);
         }
         elsif nqp::eqat($str,'\\i',$pos) {
