@@ -26,22 +26,23 @@ multi sub POSITIONS(\SELF, \pos, Callable :$eagerize = -> $idx {
         }
 
         method push(Mu \value) {
-            if nqp::istype(value,Callable) {
-                if nqp::istype($!star, Callable) {
-                    nqp::bindattr(self, IndicesReificationTarget, '$!star', $!star(*))
-                }
+            nqp::if(
+              nqp::istype(value,Callable),
+              nqp::stmts(
+                nqp::if(
+                  nqp::istype($!star,Callable),
+                  nqp::bindattr(self,IndicesReificationTarget,'$!star',$!star(*))
+                ),
                 # just using value(...) causes stage optimize to die
-                my &whatever := value;
-                if &whatever.count == Inf {
-                    nqp::push($!target, whatever(+$!star))
-                }
-                else {
-                    nqp::push($!target, whatever(|(+$!star xx &whatever.count)))
-                }
-            }
-            else {
-                nqp::push($!target, value)
-            }
+                (my &whatever := value),
+                nqp::if(
+                  &whatever.count == Inf,
+                  nqp::push($!target, whatever(+$!star)),
+                  nqp::push($!target, whatever(|(+$!star xx &whatever.count)))
+                )
+              ),
+              nqp::push($!target,value)
+            )
         }
     }
 
@@ -55,8 +56,11 @@ multi sub POSITIONS(\SELF, \pos, Callable :$eagerize = -> $idx {
         # There are lazy positions to care about too. We truncate at the first
         # one that fails to exists.
         my \rest-seq = Seq.new(pos-iter).flatmap: -> Int() $i {
-            last unless $eagerize($i);
-            $i
+            nqp::unless(
+              $eagerize($i),
+              last,
+              $i
+            )
         };
         my \todo := nqp::create(List::Reifier);
         nqp::bindattr(todo, List::Reifier, '$!reified', eager-indices);
@@ -83,38 +87,35 @@ multi sub postcircumfix:<[ ]>( \SELF, int $pos, Mu \assignee ) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, int $pos, Mu :$BIND! is raw) is raw {
     SELF.BIND-POS($pos, $BIND);
 }
-multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$SINK!, *%other ) is raw {
-    SLICE_ONE_LIST( SELF, $pos, (:$SINK), %other );
-}
 multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$delete!, *%other ) is raw {
-    $delete && !%other
+    $delete && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? SELF.DELETE-POS($pos)
-      !! SLICE_ONE_LIST( SELF, $pos, (:$delete), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'delete', $delete, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$exists!, *%other ) is raw {
-    $exists && !%other
+    $exists && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? SELF.EXISTS-POS($pos)
-      !! SLICE_ONE_LIST( SELF, $pos, (:$exists), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'exists', $exists, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$kv!, *%other ) is raw {
-    $kv && !%other
+    $kv && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? ($pos, SELF.AT-POS($pos)) !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$kv), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'kv', $kv, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$p!, *%other ) is raw {
-    $p && !%other
+    $p && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? Pair.new($pos,SELF.AT-POS($pos)) !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$p), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'p', $p, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$k!, *%other ) is raw {
-    $k && !%other
+    $k && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? $pos !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$k), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'k', $k, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, int $pos, :$v!, *%other ) is raw {
-    $v && !%other
+    $v && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? nqp::decont(SELF.AT-POS($pos)) !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$v), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'v', $v, %other );
 }
 
 # @a[Int 1]
@@ -127,38 +128,35 @@ multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, Mu \assignee ) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, Int:D $pos, Mu :$BIND! is raw) is raw {
     SELF.BIND-POS($pos, $BIND);
 }
-multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$SINK!, *%other ) is raw {
-    SLICE_ONE_LIST( SELF, $pos, (:$SINK), %other );
-}
 multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$delete!, *%other ) is raw {
-    $delete && !%other
+    $delete && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? SELF.DELETE-POS($pos)
-      !! SLICE_ONE_LIST( SELF, $pos, (:$delete), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'delete', $delete, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$exists!, *%other ) is raw {
-    $exists && !%other
+    $exists && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? SELF.EXISTS-POS($pos)
-      !! SLICE_ONE_LIST( SELF, $pos, (:$exists), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'exists', $exists, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$kv!, *%other ) is raw {
-    $kv && !%other
+    $kv && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? ($pos, SELF.AT-POS($pos)) !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$kv), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'kv', $kv, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$p!, *%other ) is raw {
-    $p && !%other
+    $p && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? Pair.new($pos,SELF.AT-POS($pos)) !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$p), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'p', $p, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$k!, *%other ) is raw {
-    $k && !%other
+    $k && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? $pos !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$k), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'k', $k, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Int:D $pos, :$v!, *%other ) is raw {
-    $v && !%other
+    $v && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS($pos) ?? nqp::decont(SELF.AT-POS($pos)) !! ())
-      !! SLICE_ONE_LIST( SELF, $pos, (:$v), %other );
+      !! SLICE_ONE_LIST( SELF, $pos, 'v', $v, %other );
 }
 
 # @a[$x]
@@ -171,38 +169,35 @@ multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, Mu \assignee ) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, Any:D \pos, Mu :$BIND! is raw) is raw {
     SELF.BIND-POS(pos.Int, $BIND);
 }
-multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$SINK!, *%other ) is raw {
-    SLICE_ONE_LIST( SELF, pos.Int, (:$SINK), %other );
-}
 multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$delete!, *%other ) is raw {
-    $delete && !%other
+    $delete && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? SELF.DELETE-POS(pos.Int)
-      !! SLICE_ONE_LIST( SELF, pos.Int, (:$delete), %other );
+      !! SLICE_ONE_LIST( SELF, pos.Int, 'delete', $delete, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$exists!, *%other ) is raw {
-    $exists && !%other
+    $exists && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? SELF.EXISTS-POS(pos.Int)
-      !! SLICE_ONE_LIST( SELF, pos.Int, (:$exists), %other );
+      !! SLICE_ONE_LIST( SELF, pos.Int, 'exists', $exists, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$kv!, *%other ) is raw {
-    $kv && !%other
+    $kv && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS(pos.Int) ?? (pos, SELF.AT-POS(pos.Int)) !! ())
-      !! SLICE_ONE_LIST( SELF, pos.Int, (:$kv), %other );
+      !! SLICE_ONE_LIST( SELF, pos.Int, 'kv', $kv, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$p!, *%other ) is raw {
-    $p && !%other
+    $p && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS(pos.Int) ?? Pair.new(pos, SELF.AT-POS(pos.Int)) !! ())
-      !! SLICE_ONE_LIST( SELF, pos.Int, (:$p), %other );
+      !! SLICE_ONE_LIST( SELF, pos.Int, 'p', $p, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$k!, *%other ) is raw {
-    $k && !%other
+    $k && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS(pos.Int) ?? pos !! ())
-      !! SLICE_ONE_LIST( SELF, pos.Int, (:$k), %other );
+      !! SLICE_ONE_LIST( SELF, pos.Int, 'k', $k, %other );
 }
 multi sub postcircumfix:<[ ]>( \SELF, Any:D \pos, :$v!, *%other ) is raw {
-    $v && !%other
+    $v && nqp::not_i(nqp::elems(nqp::getattr(%other,Map,'$!storage')))
       ?? (SELF.EXISTS-POS(pos.Int) ?? nqp::decont(SELF.AT-POS(pos.Int)) !! ())
-      !! SLICE_ONE_LIST( SELF, pos.Int, (:$v), %other );
+      !! SLICE_ONE_LIST( SELF, pos.Int, 'v', $v, %other );
 }
 
 # @a[@i]
@@ -286,40 +281,35 @@ multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, Mu \val ) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, :$BIND!) is raw {
     X::Bind::Slice.new(type => SELF.WHAT).throw;
 }
-multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, :$SINK!, *%other) is raw {
-    nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$SINK), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$SINK), %other )
-}
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos,:$delete!,*%other) is raw {
     nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$delete), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$delete), %other )
+        ?? SLICE_ONE_LIST( SELF, pos.Int, 'delete', $delete, %other )
+        !! SLICE_MORE_LIST(SELF,POSITIONS(SELF,pos),'delete',$delete,%other)
 }
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos,:$exists!,*%other) is raw {
     nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$exists), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$exists), %other )
+        ?? SLICE_ONE_LIST( SELF, pos.Int, 'exists', $exists, %other )
+        !! SLICE_MORE_LIST(SELF,POSITIONS(SELF,pos),'exists',$exists,%other)
 }
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, :$kv!, *%other) is raw {
     nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$kv), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$kv), %other )
+        ?? SLICE_ONE_LIST( SELF, pos.Int, 'kv', $kv, %other )
+        !! SLICE_MORE_LIST(SELF,POSITIONS(SELF,pos),'kv',$kv,%other)
 }
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, :$p!, *%other) is raw {
     nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$p), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$p), %other )
+        ?? SLICE_ONE_LIST( SELF, pos.Int, 'p', $p, %other )
+        !! SLICE_MORE_LIST(SELF,POSITIONS(SELF,pos),'p',$p,%other)
 }
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, :$k!, *%other) is raw {
     nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$k), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$k), %other )
+        ?? SLICE_ONE_LIST( SELF, pos.Int, 'k', $k, %other )
+        !! SLICE_MORE_LIST(SELF,POSITIONS(SELF,pos),'k',$k,%other)
 }
 multi sub postcircumfix:<[ ]>(\SELF, Iterable:D \pos, :$v!, *%other) is raw {
     nqp::iscont(pos)
-        ?? SLICE_ONE_LIST( SELF, pos.Int, (:$v), %other )
-        !! SLICE_MORE_LIST( SELF, POSITIONS(SELF, pos), (:$v), %other )
+        ?? SLICE_ONE_LIST( SELF, pos.Int, 'v', $v, %other )
+        !! SLICE_MORE_LIST(SELF,POSITIONS(SELF,pos),'v',$v,%other)
 }
 
 # @a[->{}]
@@ -334,51 +324,47 @@ multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, Mu \assignee ) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, :$BIND!) is raw {
     X::Bind::Slice.new(type => SELF.WHAT).throw;
 }
-multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, :$SINK!, *%other) is raw {
-    my $*INDEX = 'Effective index';
-    SLICE_MORE_LIST( SELF, POSITIONS(SELF,$block), (:$SINK), %other );
-}
 multi sub postcircumfix:<[ ]>(\SELF,Callable:D $block,:$delete!,*%other) is raw {
     my $*INDEX = 'Effective index';
     my $pos := $block(|(SELF.cache.elems xx ($block.count == Inf ?? 1 !! $block.count)));
     nqp::istype($pos,Int)
-      ?? SLICE_ONE_LIST(  SELF,  $pos, (:$delete), %other )
-      !! SLICE_MORE_LIST( SELF, @$pos, (:$delete), %other );
+      ?? SLICE_ONE_LIST(  SELF,  $pos, 'delete', $delete, %other )
+      !! SLICE_MORE_LIST( SELF, @$pos, 'delete', $delete, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF,Callable:D $block,:$exists!,*%other) is raw {
     my $*INDEX = 'Effective index';
     my $pos := $block(|(SELF.cache.elems xx ($block.count == Inf ?? 1 !! $block.count)));
     nqp::istype($pos,Int)
-      ?? SLICE_ONE_LIST(  SELF,  $pos, (:$exists), %other )
-      !! SLICE_MORE_LIST( SELF, @$pos, (:$exists), %other );
+      ?? SLICE_ONE_LIST(  SELF,  $pos, 'exists', $exists, %other )
+      !! SLICE_MORE_LIST( SELF, @$pos, 'exists', $exists, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, :$kv!, *%other) is raw {
     my $*INDEX = 'Effective index';
     my $pos := $block(|(SELF.cache.elems xx ($block.count == Inf ?? 1 !! $block.count)));
     nqp::istype($pos,Int)
-      ?? SLICE_ONE_LIST(  SELF,  $pos, (:$kv), %other )
-      !! SLICE_MORE_LIST( SELF, @$pos, (:$kv), %other );
+      ?? SLICE_ONE_LIST(  SELF,  $pos, 'kv', $kv, %other )
+      !! SLICE_MORE_LIST( SELF, @$pos, 'kv', $kv, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, :$p!, *%other) is raw {
     my $*INDEX = 'Effective index';
     my $pos := $block(|(SELF.cache.elems xx ($block.count == Inf ?? 1 !! $block.count)));
     nqp::istype($pos,Int)
-      ?? SLICE_ONE_LIST(  SELF,  $pos, (:$p), %other )
-      !! SLICE_MORE_LIST( SELF, @$pos, (:$p), %other );
+      ?? SLICE_ONE_LIST(  SELF,  $pos, 'p', $p, %other )
+      !! SLICE_MORE_LIST( SELF, @$pos, 'p', $p, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, :$k!, *%other) is raw {
     my $*INDEX = 'Effective index';
     my $pos := $block(|(SELF.cache.elems xx ($block.count == Inf ?? 1 !! $block.count)));
     nqp::istype($pos,Int)
-      ?? SLICE_ONE_LIST(  SELF,  $pos, (:$k), %other )
-      !! SLICE_MORE_LIST( SELF, @$pos, (:$k), %other );
+      ?? SLICE_ONE_LIST(  SELF,  $pos, 'k', $k, %other )
+      !! SLICE_MORE_LIST( SELF, @$pos, 'k', $k, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Callable:D $block, :$v!, *%other) is raw {
     my $*INDEX = 'Effective index';
     my $pos := $block(|(SELF.cache.elems xx ($block.count == Inf ?? 1 !! $block.count)));
     nqp::istype($pos,Int)
-      ?? SLICE_ONE_LIST(  SELF,  $pos, (:$v), %other )
-      !! SLICE_MORE_LIST( SELF, @$pos, (:$v), %other );
+      ?? SLICE_ONE_LIST(  SELF,  $pos, 'v', $v, %other )
+      !! SLICE_MORE_LIST( SELF, @$pos, 'v', $v, %other );
 }
 
 # @a[*]
@@ -391,27 +377,24 @@ multi sub postcircumfix:<[ ]>( \SELF, Whatever:D, Mu \assignee ) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$BIND!) is raw {
     X::Bind::Slice.new(type => SELF.WHAT).throw;
 }
-multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$SINK!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$SINK), %other );
-}
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$delete!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$delete), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'delete', $delete, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$exists!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$exists), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'exists', $exists, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$kv!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$kv), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'kv', $kv, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$p!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$p), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'p', $p, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$k!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$k), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'k', $k, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, Whatever:D, :$v!, *%other) is raw {
-    %other
-      ?? SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$v), %other )
+    nqp::elems(nqp::getattr(%other,Map,'$!storage'))
+      ?? SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'v', $v, %other )
       !! SELF[^SELF.cache.elems];
 }
 
@@ -427,151 +410,28 @@ multi sub postcircumfix:<[ ]>(\SELF, HyperWhatever:D $, Mu \assignee) is raw {
 multi sub postcircumfix:<[ ]>(\SELF, :$BIND!) is raw {
     X::Bind::ZenSlice.new(type => SELF.WHAT).throw;
 }
-multi sub postcircumfix:<[ ]>(\SELF, :$SINK!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$SINK), %other );
-}
 multi sub postcircumfix:<[ ]>(\SELF, :$delete!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$delete), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'delete', $delete, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, :$exists!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$exists), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'exists', $exists, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, :$kv!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$kv), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'kv', $kv, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, :$p!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$p), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'p', $p, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, :$k!, *%other) is raw {
-    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$k), %other );
+    SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'k', $k, %other );
 }
 multi sub postcircumfix:<[ ]>(\SELF, :$v!, *%other) is raw {
-    %other
-      ?? SLICE_MORE_LIST( SELF, ^SELF.cache.elems, (:$v), %other )
+    nqp::elems(nqp::getattr(%other,Map,'$!storage'))
+      ?? SLICE_MORE_LIST( SELF, ^SELF.cache.elems, 'v', $v, %other )
       !! SELF[^SELF.cache.elems];
 }
 multi sub postcircumfix:<[ ]>(\SELF, *%other) is raw {
     SELF.ZEN-POS(|%other);
 }
 
-proto sub postcircumfix:<[; ]>(|) is nodal { * }
-
-sub MD-ARRAY-SLICE-ONE-POSITION(\SELF, \indices, \idx, int $dim, \target) is raw {
-    my int $next-dim = $dim + 1;
-    if $next-dim < indices.elems {
-        if nqp::istype(idx, Iterable) && !nqp::iscont(idx) {
-            for idx {
-                MD-ARRAY-SLICE-ONE-POSITION(SELF, indices, $_, $dim, target)
-            }
-        }
-        elsif nqp::istype(idx, Int) {
-            MD-ARRAY-SLICE-ONE-POSITION(SELF.AT-POS(idx), indices, indices.AT-POS($next-dim), $next-dim, target)
-        }
-        elsif nqp::istype(idx, Whatever) {
-            for ^SELF.cache.elems {
-                MD-ARRAY-SLICE-ONE-POSITION(SELF.AT-POS($_), indices, indices.AT-POS($next-dim), $next-dim, target)
-            }
-        }
-        elsif nqp::istype(idx, Callable) {
-            MD-ARRAY-SLICE-ONE-POSITION(SELF, indices, idx.(|(SELF.cache.elems xx (idx.count == Inf ?? 1 !! idx.count))), $dim, target);
-        }
-        else  {
-            MD-ARRAY-SLICE-ONE-POSITION(SELF.AT-POS(idx.Int), indices, indices.AT-POS($next-dim), $next-dim, target)
-        }
-    }
-    else {
-        if nqp::istype(idx, Iterable) && !nqp::iscont(idx) {
-            for idx {
-                MD-ARRAY-SLICE-ONE-POSITION(SELF, indices, $_, $dim, target)
-            }
-        }
-        elsif nqp::istype(idx, Int) {
-            nqp::push(target, SELF.AT-POS(idx))
-        }
-        elsif nqp::istype(idx, Whatever) {
-            for ^SELF.cache.elems {
-                nqp::push(target, SELF.AT-POS($_))
-            }
-        }
-        elsif nqp::istype(idx, Callable) {
-            nqp::push(target, SELF.AT-POS(idx.(|(SELF.cache.elems xx (idx.count == Inf ?? 1 !! idx.count)))))
-        }
-        else {
-            nqp::push(target, SELF.AT-POS(idx.Int))
-        }
-    }
-}
-sub MD-ARRAY-SLICE(\SELF, @indices) is raw {
-    my \target = IterationBuffer.new;
-    MD-ARRAY-SLICE-ONE-POSITION(SELF, @indices, @indices.AT-POS(0), 0, target);
-    nqp::p6bindattrinvres(nqp::create(List), List, '$!reified', target)
-}
-
-multi sub postcircumfix:<[; ]>(\SELF, @indices) is raw {
-    my int $elems = @indices.elems;
-    my $indices := nqp::getattr(@indices,List,'$!reified');
-    my int $i = -1;
-
-    return MD-ARRAY-SLICE(SELF,@indices)
-      unless nqp::istype(nqp::atpos($indices,$i), Int)
-      while nqp::islt_i(++$i,$elems);
-
-    SELF.AT-POS(|@indices)
-}
-
-multi sub postcircumfix:<[; ]>(\SELF, @indices, Mu \assignee) is raw {
-    my int $elems = @indices.elems;
-    my $indices := nqp::getattr(@indices,List,'$!reified');
-    my int $i = -1;
-
-    return MD-ARRAY-SLICE(SELF, @indices) = assignee
-      unless nqp::istype(nqp::atpos($indices,$i), Int)
-      while nqp::islt_i(++$i,$elems);
-
-    SELF.ASSIGN-POS(|@indices, assignee)
-}
-
-multi sub postcircumfix:<[; ]>(\SELF, @indices, :$exists!) is raw {
-    if $exists {
-        my int $elems = @indices.elems;
-        my $indices := nqp::getattr(@indices,List,'$!reified');
-        my int $i = -1;
-        fail X::NYI.new(feature => ':exists on multi-dimensional slices')
-          unless nqp::istype(nqp::atpos($indices,$i), Int)
-          while nqp::islt_i(++$i,$elems);
-
-        SELF.EXISTS-POS(|@indices)
-    }
-    else {
-        SELF[@indices]
-    }
-}
-
-multi sub postcircumfix:<[; ]>(\SELF, @indices, :$delete!) is raw {
-    if $delete {
-        my int $elems = @indices.elems;
-        my $indices := nqp::getattr(@indices,List,'$!reified');
-        my int $i = -1;
-        fail X::NYI.new(feature => ':delete on multi-dimensional slices')
-          unless nqp::istype(nqp::atpos($indices,$i), Int)
-          while nqp::islt_i(++$i,$elems);
-
-        SELF.DELETE-POS(|@indices)
-    }
-    else {
-        SELF[@indices]
-    }
-}
-
-multi sub postcircumfix:<[; ]>(\SELF, @indices, :$BIND!) is raw {
-    my int $elems = @indices.elems;
-    my $indices := nqp::getattr(@indices,List,'$!reified');
-    my int $i = -1;
-    X::Bind::Slice.new(type => SELF.WHAT).throw
-      unless nqp::istype(nqp::atpos($indices,$i), Int)
-      while nqp::islt_i(++$i,$elems);
-
-    SELF.BIND-POS(|@indices, $BIND)
-}
-
 # vim: ft=perl6 expandtab sw=4
