@@ -1,6 +1,6 @@
 
 sub METAOP_ASSIGN(\op) {
-    -> Mu \a, Mu \b { a = op.( a // op.(), b) }
+    -> Mu \a, Mu \b { a = op.( ( a.DEFINITE ?? a !! op.() ), b) }
 }
 
 sub METAOP_TEST_ASSIGN:<//>(\lhs, $rhs) is raw { lhs // (lhs = $rhs()) }
@@ -24,8 +24,14 @@ sub METAOP_CROSS(\op, &reduce) {
         my $rop = lol.elems == 2 ?? op !! &reduce(op);
         my $laze = False;
         my @loi = eager for lol -> \elem {
-            $laze = True if elem.is-lazy;
-            nqp::iscont(elem) ?? (elem,).iterator !! elem.iterator;
+            if nqp::iscont(elem) {
+                $laze = False;
+                (elem,).iterator
+            }
+            else {
+                $laze = True if elem.is-lazy;
+                elem.iterator
+            }
         }
         my Mu $cache := nqp::list();
         my int $i = 0;
@@ -50,7 +56,7 @@ sub METAOP_CROSS(\op, &reduce) {
                     if $i >= $n { take lol.elems == 2 ?? $rop(|@v) !! $rop(@v); }
                     else { $i = $i + 1; @j.push($j); $j = 0; }
                 }
-                elsif !((my \value = @loi[$i].pull-one) =:= IterationEnd) {
+                elsif nqp::not_i(nqp::eqaddr((my \value = @loi[$i].pull-one),IterationEnd)) {
                     nqp::bindpos($sublist, $j, value);
                     redo;
                 }
@@ -74,14 +80,20 @@ sub METAOP_ZIP(\op, &reduce) {
         my $rop = $arity == 2 ?? op !! &reduce(op);
         my $laze = True;
         my @loi = eager for lol -> \elem {
-            $laze = False unless elem.is-lazy;
-            Rakudo::Internals::WhateverIterator.new(elem.iterator)
+            if nqp::iscont(elem) {
+                $laze = False;
+                Rakudo::Internals::WhateverIterator.new((elem,).iterator)
+            }
+            else {
+                $laze = False unless elem.is-lazy;
+                Rakudo::Internals::WhateverIterator.new(elem.iterator)
+            }
         }
         gather {
             loop {
                 my \z = @loi.map: {
                     my \value = .pull-one;
-                    last if value =:= IterationEnd;
+                    last if nqp::eqaddr(value,IterationEnd);
                     value
                 };
                 my $z = List.from-iterator(z.iterator);
@@ -97,19 +109,16 @@ proto sub METAOP_REDUCE_LEFT(|) { * }
 multi sub METAOP_REDUCE_LEFT(\op, \triangle) {
     if op.count > 2 and op.count < Inf {
         my $count = op.count;
-#?if jvm
-        my $ :=
-#?endif
         sub (+values) {
             my \source = values.iterator;
 
             my \first = source.pull-one;
-            return () if first =:= IterationEnd;
+            return () if nqp::eqaddr(first,IterationEnd);
 
             my @args.push: first;
             GATHER({
                 take first;
-                until (my \current = source.pull-one) =:= IterationEnd {
+                until nqp::eqaddr((my \current = source.pull-one),IterationEnd) {
                     @args.push: current;
                     if @args.elems == $count {
                         my \val = op.(|@args);
@@ -122,19 +131,16 @@ multi sub METAOP_REDUCE_LEFT(\op, \triangle) {
         }
     }
     else {
-#?if jvm
-        my $ :=
-#?endif
         sub (+values) {
             my \source = values.iterator;
 
             my \first = source.pull-one;
-            return () if first =:= IterationEnd;
+            return () if nqp::eqaddr(first,IterationEnd);
 
             my $result := first;
             GATHER({
                 take first;
-                until (my \value = source.pull-one) =:= IterationEnd {
+                until nqp::eqaddr((my \value = source.pull-one),IterationEnd) {
                     take ($result := op.($result, value));
                 }
             }).lazy-if(source.is-lazy);
@@ -145,17 +151,14 @@ multi sub METAOP_REDUCE_LEFT(\op, \triangle) {
 multi sub METAOP_REDUCE_LEFT(\op) {
     if op.count > 2 and op.count < Inf {
         my $count = op.count;
-#?if jvm
-        my $ :=
-#?endif
         sub (+values) {
             my \iter = values.iterator;
             my \first = iter.pull-one;
-            return op.() if first =:= IterationEnd;
+            return op.() if nqp::eqaddr(first,IterationEnd);
 
             my @args.push: first;
             my $result := first;
-            until (my \value = iter.pull-one) =:= IterationEnd {
+            until nqp::eqaddr((my \value = iter.pull-one),IterationEnd) {
                 @args.push: value;
                 if @args.elems == $count {
                     my \val = op.(|@args);
@@ -168,25 +171,32 @@ multi sub METAOP_REDUCE_LEFT(\op) {
         }
     }
     else {
-#?if jvm
-        my $ :=
-#?endif
-
-        op =:= &infix:<+>
+        nqp::eqaddr(op,&infix:<+>)
         ?? &sum
         !! sub (+values) {
-            my \iter = values.iterator;
-            my \first = iter.pull-one;
-            return op.() if first =:= IterationEnd;
-
-            my \second = iter.pull-one;
-            return op.count <= 1 ?? op.(first) !! first if second =:= IterationEnd;
-
-            my $result := op.(first, second);
-            until (my \value = iter.pull-one) =:= IterationEnd {
-                $result := op.($result, value);
-            }
-            $result;
+            nqp::stmts(
+              (my $iter := values.iterator),
+              nqp::if(
+                nqp::eqaddr((my $result := $iter.pull-one),IterationEnd),
+                op.(),                         # identity
+                nqp::if(
+                  nqp::eqaddr((my $value := $iter.pull-one),IterationEnd),
+                  nqp::if(
+                    nqp::isle_i(op.arity,1),
+                    op.($result),              # can call with 1 param
+                    $result                    # what we got
+                  ),
+                  nqp::stmts(
+                    ($result := op.($result,$value)),
+                    nqp::until(
+                      nqp::eqaddr(($value := $iter.pull-one),IterationEnd),
+                      ($result := op.($result,$value))
+                    ),
+                    $result                    # final result
+                  )
+                )
+              )
+            )
         }
     }
 }
@@ -195,18 +205,15 @@ proto sub METAOP_REDUCE_RIGHT(|) { * }
 multi sub METAOP_REDUCE_RIGHT(\op, \triangle) {
     if op.count > 2 and op.count < Inf {
         my $count = op.count;
-#?if jvm
-        my $ :=
-#?endif
         sub (+values) {
             my \source = values.reverse.iterator;
             my \first = source.pull-one;
-            return () if first =:= IterationEnd;
+            return () if nqp::eqaddr(first,IterationEnd);
 
             my @args.unshift: first;
             GATHER({
                 take first;
-                while !((my \current = source.pull-one) =:= IterationEnd) {
+                until nqp::eqaddr((my \current = source.pull-one),IterationEnd) {
                     @args.unshift: current;
                     if @args.elems == $count {
                         my \val = op.(|@args);
@@ -222,11 +229,11 @@ multi sub METAOP_REDUCE_RIGHT(\op, \triangle) {
         sub (+values) {
             my \iter = values.reverse.iterator;
             my $result := iter.pull-one;
-            return () if $result =:= IterationEnd;
+            return () if nqp::eqaddr($result,IterationEnd);
 
             gather {
                 take $result;
-                while !((my $elem := iter.pull-one) =:= IterationEnd) {
+                until nqp::eqaddr((my $elem := iter.pull-one),IterationEnd) {
                     take $result := op.($elem, $result)
                 }
             }.lazy-if(values.is-lazy);
@@ -236,17 +243,14 @@ multi sub METAOP_REDUCE_RIGHT(\op, \triangle) {
 multi sub METAOP_REDUCE_RIGHT(\op) {
     if op.count > 2 and op.count < Inf {
         my $count = op.count;
-#?if jvm
-        my $ :=
-#?endif
         sub (+values) {
             my \iter = values.reverse.iterator;
             my \first = iter.pull-one;
-           return op.() if first =:= IterationEnd;
+           return op.() if nqp::eqaddr(first,IterationEnd);
 
             my @args.unshift: first;
             my $result := first;
-            until (my \value = iter.pull-one) =:= IterationEnd {
+            until nqp::eqaddr((my \value = iter.pull-one),IterationEnd) {
                 @args.unshift: value;
                 if @args.elems == $count {
                     my \val = op.(|@args);
@@ -259,19 +263,16 @@ multi sub METAOP_REDUCE_RIGHT(\op) {
         }
     }
     else {
-#?if jvm
-        my $ :=
-#?endif
         sub (+values) {
             my \iter = values.reverse.iterator;
             my \first = iter.pull-one;
-            return op.() if first =:= IterationEnd;
+            return op.() if nqp::eqaddr(first,IterationEnd);
 
             my \second = iter.pull-one;
-            return op.(first) if second =:= IterationEnd;
+            return op.(first) if nqp::eqaddr(second,IterationEnd);
 
             my $result := op.(second, first);
-            until (my \value = iter.pull-one) =:= IterationEnd {
+            until nqp::eqaddr((my \value = iter.pull-one),IterationEnd) {
                 $result := op.(value, $result);
             }
             $result;
@@ -281,9 +282,6 @@ multi sub METAOP_REDUCE_RIGHT(\op) {
 
 proto sub METAOP_REDUCE_LIST(|) { * }
 multi sub METAOP_REDUCE_LIST(\op, \triangle) {
-#?if jvm
-    my $ :=
-#?endif
     sub (+values) {
         GATHER({
             my @list;
@@ -295,17 +293,11 @@ multi sub METAOP_REDUCE_LIST(\op, \triangle) {
     }
 }
 multi sub METAOP_REDUCE_LIST(\op) {
-#?if jvm
-    my $ :=
-#?endif
     sub (+values) { op.(|values) }
 }
 
 proto sub METAOP_REDUCE_LISTINFIX(|) { * }
 multi sub METAOP_REDUCE_LISTINFIX(\op, \triangle) {
-#?if jvm
-    my $ :=
-#?endif
     sub (|values) {
         my \p = values[0];
         return () unless p.elems;
@@ -322,9 +314,6 @@ multi sub METAOP_REDUCE_LISTINFIX(\op, \triangle) {
     }
 }
 multi sub METAOP_REDUCE_LISTINFIX(\op) {
-#?if jvm
-    my $ :=
-#?endif
     sub (+values) {
         op.(|values.map({nqp::decont($_)}));
     }
@@ -332,39 +321,31 @@ multi sub METAOP_REDUCE_LISTINFIX(\op) {
 
 proto sub METAOP_REDUCE_CHAIN(|) { * }
 multi sub METAOP_REDUCE_CHAIN(\op, \triangle) {
-#?if jvm
-    my $ :=
-#?endif
     sub (+values) {
         my $state = True;
         my \iter = values.iterator;
         my Mu $current = iter.pull-one;
         gather {
             take $state;
-            while $state && !((my $next := iter.pull-one) =:= IterationEnd) {
+            while $state && nqp::not_i(nqp::eqaddr((my $next := iter.pull-one),IterationEnd)) {
                 $state = op.($current, $next);
                 take $state;
                 $current := $next;
             }
             unless $state {
-                while !((my \v = iter.pull-one) =:= IterationEnd) {
-                    take False;
-                }
+                take False until nqp::eqaddr(iter.pull-one,IterationEnd);
             }
         }.lazy-if(values.is-lazy);
     }
 }
 multi sub METAOP_REDUCE_CHAIN(\op) {
-#?if jvm
-    my $ :=
-#?endif
     sub (+values) {
         my $state := True;
         my \iter = values.iterator;
         my $current := iter.pull-one;
-        return True if $current =:= IterationEnd;
+        return True if nqp::eqaddr($current,IterationEnd);
 
-        while !((my $next := iter.pull-one) =:= IterationEnd) {
+        until nqp::eqaddr((my $next := iter.pull-one),IterationEnd) {
             $state := op.($current, $next);
             return $state unless $state;
             $current := $next;
@@ -434,7 +415,7 @@ multi sub HYPER(&op, Associative:D \left, Associative:D \right, :$dwim-left, :$d
     my @keys = %keyset.keys;
     my $type = left.WHAT;
     my \result := $type.new;
-    result = @keys Z=> HYPER(&op, left{@keys}, right{@keys}, :$dwim-left, :$dwim-right);
+    result = quietly @keys Z=> HYPER(&op, left{@keys}, right{@keys}, :$dwim-left, :$dwim-right);
     nqp::iscont(left) ?? result.item !! result;
 }
 
@@ -462,7 +443,7 @@ multi sub HYPER(&operator, Positional:D \left, \right, :$dwim-left, :$dwim-right
         unless $elems == 1 or $elems > 1 and $dwim-right or $elems == 0 and $dwim-left || $dwim-right;
     my \lefti := left.iterator;
     my int $i = 0;
-    until (my \value := lefti.pull-one) =:= IterationEnd {
+    until nqp::eqaddr((my \value := lefti.pull-one),IterationEnd) {
         @result[$i++] := HYPER(&operator, value, right, :$dwim-left, :$dwim-right);
     }
     # Coerce to the original type if it's a subtype of List
@@ -478,7 +459,7 @@ multi sub HYPER(&operator, \left, Positional:D \right, :$dwim-left, :$dwim-right
         unless $elems == 1 or $elems > 1 and $dwim-left or $elems == 0 and $dwim-left || $dwim-right;
     my \righti := right.iterator;
     my int $i = 0;
-    until (my \value := righti.pull-one) =:= IterationEnd {
+    until nqp::eqaddr((my \value := righti.pull-one),IterationEnd) {
         @result[$i++] := HYPER(&operator, left, value, :$dwim-left, :$dwim-right);
     }
     # Coerce to the original type if it's a subtype of List
@@ -532,83 +513,7 @@ multi sub HYPER(\op, \obj) {
 proto sub deepmap(|) { * }
 
 multi sub deepmap(\op, \obj) {
-    #my Mu $rpa := nqp::list();
-    #my \objs := obj.list;
-    # as a wanted side-effect is-lazy reifies the list
-    #fail X::Cannot::Lazy.new(:action('deepmap')) if objs.is-lazy;
-    my \iterable = obj.DEFINITE && nqp::istype(obj, Iterable)
-            ?? obj
-            !! obj.list;
-
-    my \result := class :: does SlippyIterator {
-        has &!block;
-        has $!source;
-
-        method new(&block, $source) {
-            my $iter := nqp::create(self);
-            nqp::bindattr($iter, self, '&!block', &block);
-            nqp::bindattr($iter, self, '$!source', $source);
-            $iter
-        }
-
-        method is-lazy() {
-            $!source.is-lazy
-        }
-        method pull-one() is raw {
-            my int $redo = 1;
-            my $value;
-            my $result;
-            if $!slipping && !(($result := self.slip-one()) =:= IterationEnd) {
-                $result
-            }
-            elsif ($value := $!source.pull-one()) =:= IterationEnd {
-                $value
-            }
-            else {
-                nqp::while(
-                    $redo,
-                    nqp::stmts(
-                        $redo = 0,
-                        nqp::handle(
-                            nqp::stmts(
-                                nqp::if(
-                                    nqp::istype($value, Iterable),
-                                    nqp::stmts(
-                                        ($result := deepmap(&!block, $value).item),
-                                    ),
-                                    ($result := &!block($value)),
-                                ),
-                                nqp::if(
-                                    nqp::istype($result, Slip),
-                                    nqp::stmts(
-                                        ($result := self.start-slip($result)),
-                                        nqp::if(
-                                            nqp::eqaddr($result, IterationEnd),
-                                            nqp::stmts(
-                                                ($value := $!source.pull-one()),
-                                                ($redo = 1 unless nqp::eqaddr($value, IterationEnd))
-                                        ))
-                                    ))
-                            ),
-                            'NEXT', nqp::stmts(
-                                ($value := $!source.pull-one()),
-                                nqp::eqaddr($value, IterationEnd)
-                                    ?? ($result := IterationEnd)
-                                    !! ($redo = 1)),
-                            'REDO', $redo = 1,
-                            'LAST', ($result := IterationEnd))),
-                    :nohandler);
-                $result
-            }
-        }
-    }.new(op, iterable.iterator);
-
-    my $type = nqp::istype(obj, List) ?? obj.WHAT !! List; # keep subtypes of List
-    my \buffer := IterationBuffer.new;
-    result.push-all(buffer);
-    my \retval = $type.new;
-    nqp::bindattr(retval, List, '$!reified', buffer);
-    nqp::iscont(obj) ?? retval.item !! retval;
+    Rakudo::Internals.coremap(op, obj, :deep)
 }
 
 multi sub deepmap(\op, Associative \h) {
@@ -661,7 +566,7 @@ multi sub nodemap(\op, Associative \h) {
 
 proto sub duckmap(|) { * }
 multi sub duckmap(\op, \obj) {
-    nodemap(-> \arg { try { op.(arg) } // try { duckmap(op,arg) } }, obj);
+    Rakudo::Internals.coremap(sub (\arg) { CATCH { return arg ~~ Iterable:D ?? duckmap(op,arg) !! arg }; op.(arg); }, obj);
 }
 
 multi sub duckmap(\op, Associative \h) {
