@@ -37,6 +37,7 @@ public final class RakOps {
         public SixModelObject Str;
         public SixModelObject List;
         public SixModelObject ListIter;
+        public SixModelObject Iterable;
         public SixModelObject Array;
         public SixModelObject Nil;
         public SixModelObject Map;
@@ -50,6 +51,7 @@ public final class RakOps {
         public SixModelObject AutoThreader;
         public SixModelObject Positional;
         public SixModelObject PositionalBindFailover;
+        public SixModelObject Associative;
         public SixModelObject EMPTYARR;
         public SixModelObject EMPTYHASH;
         public RakudoJavaInterop rakudoInterop;
@@ -87,6 +89,12 @@ public final class RakOps {
         }
         return null;
     }
+
+    public static SixModelObject p6setitertype(SixModelObject type, ThreadContext tc) {
+        GlobalExt gcx = key.getGC(tc);
+        gcx.Iterable = type;
+        return type;
+    }
     
     public static SixModelObject p6settypes(SixModelObject conf, ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
@@ -101,6 +109,7 @@ public final class RakOps {
         gcx.Str = conf.at_key_boxed(tc, "Str");
         gcx.List = conf.at_key_boxed(tc, "List");
         gcx.ListIter = conf.at_key_boxed(tc, "ListIter");
+        gcx.Iterable = conf.at_key_boxed(tc, "Iterable");
         gcx.Array = conf.at_key_boxed(tc, "Array");
         gcx.Nil = conf.at_key_boxed(tc, "Nil");
         gcx.Map = conf.at_key_boxed(tc, "Map");
@@ -111,6 +120,7 @@ public final class RakOps {
         gcx.ContainerDescriptor = conf.at_key_boxed(tc, "ContainerDescriptor");
         gcx.False = conf.at_key_boxed(tc, "False");
         gcx.True = conf.at_key_boxed(tc, "True");
+        gcx.Associative = conf.at_key_boxed(tc, "Associative");
         gcx.JavaHOW = conf.at_key_boxed(tc, "Metamodel").st.WHO.at_key_boxed(tc, "JavaHOW");
         
         SixModelObject defCD = gcx.ContainerDescriptor.st.REPR.allocate(tc,
@@ -218,7 +228,7 @@ public final class RakOps {
         SixModelObject sig = cf.codeRef.codeObject
             .get_attribute_boxed(tc, gcx.Code, "$!signature", HINT_CODE_SIG);
         SixModelObject params = sig
-            .get_attribute_boxed(tc, gcx.Signature, "$!params", HINT_SIG_PARAMS);
+            .get_attribute_boxed(tc, gcx.Signature, "@!params", HINT_SIG_PARAMS);
         
         /* Run binder, and handle any errors. */
         Object[] error = new Object[3];
@@ -254,7 +264,7 @@ public final class RakOps {
         GlobalExt gcx = key.getGC(tc);
         CallSiteDescriptor csd = Binder.explodeCapture(tc, gcx, cap);
         SixModelObject params = sig.get_attribute_boxed(tc, gcx.Signature,
-            "$!params", HINT_SIG_PARAMS);
+            "@!params", HINT_SIG_PARAMS);
         
         Object[] error = new Object[3];
         switch (Binder.bind(tc, gcx, cf, params, csd, tc.flatArgs, false, error)) {
@@ -287,7 +297,7 @@ public final class RakOps {
         }
         
         SixModelObject params = sig.get_attribute_boxed(tc, gcx.Signature,
-            "$!params", HINT_SIG_PARAMS);
+            "@!params", HINT_SIG_PARAMS);
         SixModelObject codeObj = sig.get_attribute_boxed(tc, gcx.Signature,
             "$!code", HINT_SIG_CODE);
         CodeRef cr = (CodeRef)codeObj.get_attribute_boxed(tc, gcx.Code,
@@ -310,7 +320,7 @@ public final class RakOps {
     private static final CallSiteDescriptor STORE = new CallSiteDescriptor(
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ }, null);
     private static final CallSiteDescriptor storeThrower = new CallSiteDescriptor(
-        new byte[] { }, null);
+        new byte[] { CallSiteDescriptor.ARG_STR }, null);
     public static SixModelObject p6store(SixModelObject cont, SixModelObject value, ThreadContext tc) {
         ContainerSpec spec = cont.st.ContainerSpec;
         if (spec != null) {
@@ -328,7 +338,7 @@ public final class RakOps {
                     ExceptionHandling.dieInternal(tc, "Cannot assign to a non-container");
                 else
                     Ops.invokeDirect(tc, thrower,
-                        storeThrower, new Object[] { });
+                        storeThrower, new Object[] { Ops.typeName(cont, tc) });
             }
         }
         return cont;
@@ -360,7 +370,7 @@ public final class RakOps {
     public static SixModelObject p6scalarfromdesc(SixModelObject desc, ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
 
-        if (desc == null || desc instanceof TypeObject)
+        if ( Ops.isconcrete(desc, tc) == 0 )
             desc = gcx.defaultContainerDescriptor;
         SixModelObject defVal = desc.get_attribute_boxed(tc, gcx.ContainerDescriptor,
             "$!default", HINT_CD_DEFAULT);
@@ -422,7 +432,7 @@ public final class RakOps {
     
     private static final CallSiteDescriptor rvThrower = new CallSiteDescriptor(
         new byte[] { CallSiteDescriptor.ARG_OBJ, CallSiteDescriptor.ARG_OBJ }, null);
-    public static SixModelObject p6typecheckrv(SixModelObject rv, SixModelObject routine, SixModelObject failure, ThreadContext tc) {
+    public static SixModelObject p6typecheckrv(SixModelObject rv, SixModelObject routine, SixModelObject bypassType, ThreadContext tc) {
         GlobalExt gcx = key.getGC(tc);
         SixModelObject sig = routine.get_attribute_boxed(tc, gcx.Code, "$!signature", HINT_CODE_SIG);
         SixModelObject rtype = sig.get_attribute_boxed(tc, gcx.Signature, "$!returns", HINT_SIG_RETURNS);
@@ -433,8 +443,7 @@ public final class RakOps {
                  * an Int that can unbox into an int or similar. */
                 StorageSpec spec = rtype.st.REPR.get_storage_spec(tc, rtype.st);
                 if (spec.inlineable == 0 || Ops.istype(rtype, decontValue.st.WHAT, tc) == 0) {
-                    if (Ops.istype(failure, decontValue.st.WHAT, tc) == 0
-                    &&  Ops.istype(failure, gcx.Nil, tc) == 0) {
+                    if (Ops.istype(decontValue.st.WHAT, bypassType, tc) == 0) {
                         SixModelObject thrower = getThrower(tc, "X::TypeCheck::Return");
                         if (thrower == null)
                             throw ExceptionHandling.dieInternal(tc,
@@ -567,30 +576,6 @@ public final class RakOps {
         }
         return null;
     }
-
-    public static SixModelObject p6routinereturn(SixModelObject in, ThreadContext tc) {
-        CallFrame ctx = tc.curFrame;
-        SixModelObject cont = null;
-
-        for (ctx = ctx.caller; ctx != null; ctx = ctx.caller) {
-            cont = getremotelex(ctx, "RETURN");
-            if (cont != null) break;
-        }
-
-        if (!(cont instanceof LexoticInstance)) {
-            SixModelObject thrower = getThrower(tc, "X::ControlFlow::Return");
-            if (thrower == null)
-                ExceptionHandling.dieInternal(tc, "Attempt to return outside of any Routine");
-            else
-                Ops.invokeArgless(tc, thrower);
-        }
-
-        // rewinding is handled by finally blocks in the generated subs
-        LexoticException throwee = tc.theLexotic;
-        throwee.target = ((LexoticInstance)cont).target;
-        throwee.payload = in;
-        throw throwee;
-    }
     
     public static String tclc(String in, ThreadContext tc) {
         if (in.length() == 0)
@@ -663,7 +648,7 @@ public final class RakOps {
     public static SixModelObject p6finddispatcher(String usage, ThreadContext tc) {
         SixModelObject dispatcher = null;
         
-        CallFrame ctx = tc.curFrame;
+        CallFrame ctx = tc.curFrame.caller;
         while (ctx != null) {
             /* Do we have a dispatcher here? */
             StaticCodeInfo sci = ctx.codeRef.staticInfo;
